--[[
This script will decode audio, the decoder 
could be set using the option AUDIO_DECDER below.

]]--
-- Audio sample rate (may be overridden depending on codec)
SAMPLERATE    = SAMPLERATE or 48000;
AUDIO_DECODER = AUDIO_DECODER or "PCM";
--[[ Choose from :
     AAC
     QMA
     G711
     G722
     PCM
     G726
     G728     
]]--

INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/file.pcm.qbx";
-- Audio stereo/mono (may be overridden depending on codec)
AUDIO_STEREO = AUDIO_STEREO or 1;

if (AUDIO_DECODER == "AAC") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_QAC;
elseif (AUDIO_DECODER == "QMA") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_QMA;
elseif (AUDIO_DECODER == "G711") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_Q711;
   SAMPLERATE = 8000;
   AUDIO_STEREO = 0;
elseif (AUDIO_DECODER == "G722") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_Q722;
   SAMPLERATE = 16000;
   AUDIO_STEREO = 0;
elseif (AUDIO_DECODER == "PCM") then
   AUDIO_CODEC =  Q_AVDEC_CFP_AUDIO_CODEC_QPCM;
   -- SAMPLERATE is flexible
elseif (AUDIO_DECODER == "G726") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_Q726;
   -- AUDIO_FRAME_SIZE is flexible
   SAMPLERATE = 8000;
   AUDIO_STEREO = 0;
elseif (AUDIO_DECODER == "G728") then
   AUDIO_CODEC = Q_AVDEC_CFP_AUDIO_CODEC_Q728;
   -- AUDIO_FRAME_SIZE is flexible
   SAMPLERATE = 8000;
   AUDIO_STEREO = 0;
else
   error("Invalid audio decoder " .. tostring(AUDIO_DECODER))
end



if (SAMPLERATE == 8000) then
    board.setAudioSampleFreq({port=0, freq=16000});
    sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=2048000});
    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=2048000});
else
    board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
end
AUDIO_BUFFER_SIZE = 1000000;
AUDIO_STREAM_ID = 1;
VIDEO_STREAM_ID = 2;

avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});

avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC, vtype=Q_AVDEC_CFP_VIDEO_CODEC_NONE, audioBufferSize=AUDIO_BUFFER_SIZE});

rs0 = sysctl0:createReadStreamer({name="rs0"})

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
if (AUDIO_STEREO == 1) then
   avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
else
   avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
end
if (AUDIO_CODEC == Q_AVDEC_CFP_AUDIO_CODEC_QPCM) then
   avdec0:setParam({param=Q_AVDEC_CMP_QPCM_ENDIANNESS, value=Q_AVDEC_CFP_AUDIO_QPCM_BIG_ENDIAN});
end
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
--avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_OUTPUT, value=1});
avdec0:activateCfg();

avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
avoc0:activateCfg();

avsn0:bindAudioInput({src=avdec0});

ch0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
avoc0:setAudioChannelSource({ch=ch0, src=avsn0});
avoc0:enableAudioChannel({ch=ch0, enable=1});
avoc0:activateAudioChannelCfg();

rs0:bindDecoder({dec=avdec0});
bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={AUDIO_STREAM_ID}});

avoc0:start();
avsn0:start();
avdec0:start();
rs0:play({bsr=bsr0});
