--[[ This script can be used as a base to set up the rate conversion with 
audio encoder. The SAIN is binded to the audio filter unit which is
set for the rate conversion mode. The output of the audio filter is binded to 
the audio encoder, ie it encodes the audio after changing the sampling
rate to the desired value.
]]--


----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Audio Input
-- onboard = onboard audio, kite = kite daughtercard
AIN = AIN or 'onboard';

-- Audio sample rateec
SAMPLERATE = SAMPLERATE or 32000;
OUTSAMPLERATE = 8000;
-- Audio bitrate
ABITRATE = ABITRATE or 32000;

-- Controls the audio codec type

AUDIO_CODEC = AUDIO_CODEC or Q_AUDENC_CFP_CODEC_Q711;
IN_FRAME_SIZE = 1024;


AUD_SAMPLESIZE = 16;

AUDENC_NUMCH = Q_AUDENC_CFP_CHANNEL_MONO;


-- Audio filter enable
AUD_FILTER_ENABLE = AUD_FILTER_ENABLE or 1;

AUD_FILTER_TYPE = AUD_FILTER_TYPE or Q_AUDFLTR_CFP_TYPE_RATECONVERSION; 



AUD_FILTER_INPUTS = AUD_FILTER_INPUTS or 1;
AUD_FILTER_OUTPUTS = AUD_FILTER_OUTPUTS or 1;
AUD_FILTER_OUT_FREQ = OUTSAMPLERATE;
AUD_FILTER_OUT_FRAMESIZE =IN_FRAME_SIZE * OUTSAMPLERATE / SAMPLERATE ;
AUD_FILTER_IN_FREQ = SAMPLERATE;
AUD_FILTER_IN_FRAMESIZE = IN_FRAME_SIZE;

if (AUDENC_NUMCH == Q_AUDENC_CFP_CHANNEL_MONO) then
   AUD_FILTER_NUM_CHAN = 1;
else
   AUD_FILTER_NUM_CHAN = 2;
end
-- 0 for nothing, 1 for input preview, 2 for audio filter output
PREVIEW = PREVIEW or 0;

-- Audio output source
AUDIO_OUTPUT_SRC = AUDIO_OUTPUT_SRC or 0;

-- Option to turn off audio encoding when set to 0
AUDENC_ENABLE = AUDENC_ENABLE or 1;

OUTPUTFILENAME 	  = OUTPUTFILENAME or {};
OUTPUTFILENAME[1] = OUTPUTFILENAME[1] or "/media/sda1/aud/audenc0.qbx";
OUTPUTFILENAME[2] = OUTPUTFILENAME[2] or "/media/sda1/aud/audenc1.qbx";
OUTPUTFILENAME[3] = OUTPUTFILENAME[3] or "/media/sda1/aud/audenc2.qbx";
OUTPUTFILENAME[4] = OUTPUTFILENAME[4] or "/media/sda1/aud/audenc3.qbx";

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------
AUDCH = AUD_FILTER_OUTPUTS;


avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
ws = sysctl0:createWriteStreamer({name="ws"});

ws:start();

sain = {};
for i = 1, AUD_FILTER_INPUTS do
   board.setAudioInput({port=i-1, freq=SAMPLERATE, mode=AIN});
   sain[i] = sysctl0:sainCreate({name="sain"..i, port=i-1});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_MONO_L});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=AUD_SAMPLESIZE});
   sain[i]:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=IN_FRAME_SIZE});
   if (AIN == 'kite') then
      sain[i]:setParam({param=Q_SAIN_CMP_AUD_CLOCK_SOURCE, value=Q_SAIN_CFP_AUD_EXTERNAL_CLK});
   end
   sain[i]:activateCfg();
end

   

if (AUD_FILTER_ENABLE == 1) then
   afilter0 = sysctl0:audfilterCreate({name="afilter0", numOutputPorts=AUD_FILTER_OUTPUTS, numOutputFrames=32,
				      maxFramesSize=8192, filterType=AUD_FILTER_TYPE, cBufferLen=32});
   afilter0:setParam({param=Q_AUDFLTR_CMP_MAX_INPUT_PORTS, value=AUD_FILTER_INPUTS});
   afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FREQ, value=AUD_FILTER_OUT_FREQ});
   afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FRAME_NUM_SAMPLES, value=AUD_FILTER_OUT_FRAMESIZE});
   afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FREQ, value=AUD_FILTER_IN_FREQ});
   afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=AUD_FILTER_NUM_CHAN});
   afilter0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=AUD_FILTER_IN_FRAMESIZE});
   afilter0:activateCfg();

   for chidx = 1, AUD_FILTER_INPUTS do
      iport = afilter0:addInputPort();
      afilter0:setInputPortSource({inport=iport, src=sain[chidx], threshold=1});
   end
    
end

if (AUDENC_ENABLE == 1) then
    audenc = {};
    for i = 1, AUDCH do
        local audioStreamId = i;
        
        audenc[i] = sysctl0:audencCreate({name="audenc"..i, ws=ws, codecType=AUDIO_CODEC});
        
        if (AUD_FILTER_ENABLE == 0) then
	   audenc[i]:bindAudioInput({src=sain[1]});
        else
	   
	   audenc[i]:bindAudioInput({src=afilter0, port=i-1});
        end
        
        audenc[i]:setStreamID({streamid=audioStreamId});
        
        audenc[i]:setParam({param=Q_AUDENC_CMP_NUM_CHANNELS, value=AUDENC_NUMCH});
        audenc[i]:setParam({param=Q_AUDENC_CMP_Q711_LAW, value=Q_AUDENC_CFP_Q711_U_LAW});
        audenc[i]:setParam({param=Q_AUDENC_CMP_SAMPLE_RATE, value=AUD_FILTER_OUT_FREQ});
        audenc[i]:setParam({param=Q_AUDENC_CMP_SAMPLE_SIZE, value=16});
        audenc[i]:setParam({param=Q_AUDENC_CMP_BIT_RATE, value=ABITRATE});
        audenc[i]:activateCfg();
        
        ws:open({type="file", file=OUTPUTFILENAME[i], sid = {audioStreamId}});
    end
end



avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
avoc0:activateCfg();

avsn0:bindAudioInput({src=sain[1]});

ch0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
avoc0:setAudioChannelSource({ch=ch0, src=avsn0});
avoc0:enableAudioChannel({ch=ch0, enable=1});

avoc0:activateAudioChannelCfg();

avoc0:start();
avsn0:start();




if (AUDENC_ENABLE == 1) then
   for i = 1, AUDCH do
      audenc[i]:record();
   end
end


if (AUD_FILTER_ENABLE == 1) then
   afilter0:start();
   for i = 1, AUD_FILTER_INPUTS do
      sain[i]:start();
   end
else
   sain[1]:start();
end


--[[
for i = 0, 10, 1 do
   sysctl0:echo({param=0});
    os.execute("sleep 1");
end
sain[1]:flush();
--afilter[1]:stop();
--audenc[1]:stop();
ws:close({type="file", file=OUTPUTFILENAME[1]});
--]]