--[[

This script is used to encode eight CIF streams from 1 NTSC stream via svideo input.  
The output is 8 video frames displayed simultaneously on the display. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Init
BITRATE = BITRATE or {};
BITRATE_2 = BITRATE_2 or {};

-- Number of live encoded channels
ENCCH = ENCCH or 8;

-- Number of derived channels
ENCCH_2 = ENCCH_2 or 0;

-- 0 for nothing, 1 for live, 2 for reconstructed frames
PREVIEW = PREVIEW or 2;

VIN					= VIN or "svideo"
VOUT				= VOUT or "hdmi"
VIDEO_FORMAT_IN		= VIDEO_FORMAT_IN or "480i"
VIDEO_FORMAT_OUT	= VIDEO_FORMAT_OUT or "1080i"

-- Host IP and base port for udp live stream (port, port+1, port+2...)
STREAMING = STREAMING or 0;
HOST = HOST or "192.168.40.38";
PORT = PORT or 9700;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 768;

-- Port for 2nd streams (port, port+1...)
PORT_2 = PORT_2  or 9750;

-- Width and height of each live encoded stream
ENCW = ENCW or 352;
ENCH = ENCH or 240;

-- Bitrate of each live encoded stream
for i=0,ENCCH-1 do
  BITRATE[i] = 500000;
end

-- Put override bitrates here
BITRATE[1] = BITRATE[1] or 100000;

-- Width and height of each derived encoded stream
ENCW_2 = ENCW_2 or 352;
ENCH_2 = ENCH_2 or 240;

-- Bitrate of each derived encoded stream
for i=0,ENCCH_2-1 do
  BITRATE_2[i] = 500000;
end

-- Number of rendered images and their scaled size
OUTCH = OUTCH or 8;
OUTW  =	OUTW or	352;
OUTH  =	OUTH or 240;

-- Width and height of the output grid
OUTGRIDW = OUTGRIDW or 4;
OUTGRIDH =OUTGRIDH or 2;

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF = 1;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

BITSTREAM_OUTPUT=1;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=1, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format=VIDEO_FORMAT_OUT, mode=VOUT, sync=vout_sync});
board.setVideoInput({port=0, format=VIDEO_FORMAT_IN, mode=VIN});

sysctl0:setVerboseLevel({param=3});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

ENCCH = ENCCH - 1;
OUTCH = OUTCH - 1;
ENCCH_2 = ENCCH_2 - 1;
if (OUTCH > ENCCH) then OUTCH = ENCCH end

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=ENCW, maxHeight=ENCH, numVidFrame=11, maxPMEDSFactor=1});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

--dofile(os.getenv("QUARC_ROOT").."/merlinsw/exthost/scripts/libvpp.lua");
lvpp0:crop({cropWidth=720, cropHeight=480, vertOffset=0, horzOffset=0});
lvpp0:size({outputWidth=ENCW, outputHeight=ENCH});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:setParam({param=Q_AVCENC_CMP_PME_ENABLE, value=0});
lvpp0:activateCfg();

-- CREATE ENCODERS
nvppenc2 = {};
avcenc = {};
avcenc2 = {};
avsnenc = {};
videoStreamId = {};

-- AVC SETUP
--dofile(os.getenv("QUARC_ROOT").."/merlinsw/exthost/scripts/libavcenc.lua");

for i = 0,ENCCH do
  avcenc[i] = sysctl0:avcencCreate({name="avcenc", ws=ws, maxwidth=ENCW, maxheight=ENCH, numreferenceframes=10});
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"});
  videoStreamId = i + 2;

  avcenc_480i(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, 0)
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_PME_ENABLE, value=0});
  avcenc[i]:activateVideoEncCfg();

  avcenc[i]:bindVideoInput({src=lvpp0});
  if STREAMING == 1 then
  ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", HOST, PORT+i, MAX_PACKET_SIZE), sid = {videoStreamId}});
  end
  avcenc[i]:setStreamID({streamid=videoStreamId});
  avcenc[i]:setAckInterval({interval=8});
end

for i = 0, ENCCH_2 do
  nvppenc2[i] = sysctl0:nvppCreate({name="nvpp0", maxWidth=ENCW_2, maxHeight=ENCH_2, maxPMEDSFactor=1});
  avcenc2[i] = sysctl0:avcencCreate({name="avcenc", ws=ws, maxwidth=ENCW_2, maxheight=ENCH_2, numreferenceframes=4});
  -- delta of 16 between live and derived channels
  videoStreamId = i + 16;

  -- workaround - crop instead of scale 2nd channel
  Setup_VPP(nvppenc2[i], ENCW, ENCH, 0, 0, 2, 2)
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PME_ENABLE,           value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  nvppenc2[i]:activateCfg();

  avcenc_480i(avcenc2[i], "high", 31, BITRATE_2[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, 0)
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_PME_ENABLE, value=0});
  avcenc2[i]:activateVideoEncCfg();

  nvppenc2[i]:bindInput({src=lvpp0});
  avcenc2[i]:bindVideoInput({src=nvppenc2[i]});
  if STREAMING == 1 then
  ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", HOST, PORT_2+i, MAX_PACKET_SIZE), sid = {videoStreamId}});
  end
  avcenc2[i]:setStreamID({streamid=videoStreamId});
  avcenc2[i]:setAckInterval({interval=8});
end

vch = {};
x = 0;
y = 0;
for i = 0,OUTCH do
  vch[i] = avoc0:addVideoChannel();

  if PREVIEW == 2 then
      avsnenc[i]:bindVideoInput({src=avcenc[i]});
--      avsnenc[i]:bindVideoInput({src=nvppenc2[i]});
    avsnenc[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-15015});
    avsnenc[i]:activateCfg();
  elseif PREVIEW == 1  then
      print("binding video input to live\n");
      avsnenc[i]:bindVideoInput({src=lvpp0});
  end
  avoc0:setVideoChannelSource({ch=vch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- START
avoc0:start();

for i = 0,OUTCH do
  avsnenc[i]:start();
end

for i = 0,ENCCH do
  avcenc[i]:record();
end

for i = 0,ENCCH_2 do
  avcenc2[i]:record();
  nvppenc2[i]:start();
end

lvpp0:start();

function done()
  lvpp0:flush();

  waitForEvents({objid=avcenc[0], timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});

  os.execute("echo flushed");

  for i = 0,ENCCH do
    ws:close({type="h4v", file=string.format("test%d.h4v", i)});
  end
end

--os.execute("sleep 10");
--done();

function q()
  lvpp0:query();
  for i = 0,ENCCH do
    avcenc[i]:query();
  end
end
