NUMCH           = NUMCH or 4;
OUTCH           = OUTCH or 4;
OUTW            = OUTW or 320;
OUTH            = OUTH or 240;
COMPWIDTH       = COMPWIDTH or 640;
COMPHEIGHT      = COMPHEIGHT or 480;
DISPLAYWIDTH    = DISPLAYWIDTH or 1280;
DISPLAYHEIGHT   = DISPLAYHEIGHT or 720;
VIDEO_FORMAT    = VIDEO_FORMAT or "720p";
VOUT            = VOUT or "hdmi";

-- Set up the output port
board.setVideoOutput({port=0, format=VIDEO_FORMAT, mode=VOUT});
board.setVideoInput({port=0, format='480i', mode='kite'});

sysctl0:setVerboseLevel({param=1});

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x80101080, interleave=NUMCH, syncmode=Q_SYS_CFP_VPORT_SYNC_MODE_MOBI, syncheight=525});
sysctl0:setVPortMode({port=1, live=0});

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput720p60();
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=DISPLAYWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=DISPLAYHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_RATE_DIVIDER, value=2});
avoc0:activateCfg();

NUMCH = NUMCH - 1;
OUTCH = OUTCH - 1;
lvpp = {};
avsn = {};

for i = 0,NUMCH do
  lvpp[i] = sysctl0:lvppCreate({name="lvpp" .. i, slot=i, port=0, maxWidth=720, maxHeight=480, maxPMEDSFactor=2, numVidFrame=8, maxDeintMode=3});
  avsn[i] = sysctl0:avsnCreate({name="avsn" .. i});
end

for i = 0,NUMCH do
  lvpp[i]:crop({cropWidth=720, cropHeight=480, horzOffset=0, vertOffset=0});
  lvpp[i]:size({outputWidth=720, outputHeight=480});
  lvpp[i]:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=262});
  lvpp[i]:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp[i]:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
  lvpp[i]:setParam({param=Q_LVPP_CMP_DEST_PROG, value=1});
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=6000});
  lvpp[i]:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3000});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PMEDS_H, value=2});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PMEDS_V, value=2});
  lvpp[i]:setParam({param=Q_LVPP_CMP_PMEDS_EN, value=1});
  lvpp[i]:setParam({param=Q_LVPP_CMP_DEINT_MODE, value=3});
  lvpp[i]:setParam({param=Q_LVPP_CMP_INPUT_DECIMATION, value=2});

  lvpp[i]:setParam({param=Q_LVPP_CMP_HW_BYPASS_TSTAT, value=0});
  lvpp[i]:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=0});
  lvpp[i]:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=0});

  lvpp[i]:activateCfg();
end

for i = 0,OUTCH do
  avsn[i]:bindVideoInput({src=lvpp[i]});
  avsn[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-3000});
  avsn[i]:activateCfg();
end

vch = {};
x = 0;
y = 0;
for i = 0,OUTCH do
  vch[i] = avoc0:addVideoChannel();
  avoc0:setVideoChannelSource({ch=vch[i], src=avsn[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=720});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=480});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

avoc0:start();

for i = 0,OUTCH do
  avsn[i]:start();
end

for i = 0,NUMCH do
  lvpp[i]:start();
end
