--[[

This script is used to encode four D1 streams from 1 NTSC stream via svideo input.  
The output is 4 frames displayed simultaneously on the display. 

--]]
------------------------------------------------------------------------------
--                        Configuration parameters                          --
------------------------------------------------------------------------------
-- Video input can be 480i or 576i
INPUTFORMAT                     = INPUTFORMAT                   or "480i"

-- A/V playback is not stable in 4.0 therefore the DVR script
-- only plays back the video content of a AV recorded stream.
-- To enable AV playback, set this variable to 0
VIDEO_PLAYBACK_ONLY             = VIDEO_PLAYBACK_ONLY           or 1

-- Video input can be svideo, composite, kite or VA85
VIN                             = VIN                           or "kite"
-- Audio input can be onboard, kite or VA85
AIN                             = AIN                           or "onboard"

-- Video output can be 1080i, 576i or 480i
FORMAT                          = FORMAT                        or "1080i"
FORMATFPS                       = FORMATFPS                     or 60
-- Video output is set to hdmi or svideo
VOUT                            = VOUT                          or "hdmi"

-- Number of live encoded channels
ENCCH                           = ENCCH                         or 4

-- Number of decoded channels
DECCH                           = DECCH                         or 1

-- Number of derived channels
ENCCH_2                         = ENCCH_2                       or 1

-- Number of MJPEG channels
MJPEGCH                         = MJPEGCH                       or 0

-- JPEG snapshot channel
SNAPSHOTCH                      = SNAPSHOTCH                    or 0

-- 0 for nothing, 1 for live, 2 for reconstructed preview, 3 for decoder 
PREVIEW                         = PREVIEW                       or 1

-- Use the high speed bitstream port for encoded output instead of EM transfer
HSBP_OUTPUT                     = HSBP_OUTPUT                   or 0

-- 0:Disable 1: Enable. OBJECT_TRACKING 
OBJECT_TRACKING                 = OBJECT_TRACKING               or 0

-- 0:Disable 1: Enable. OBJECTTRACKING_DISP  1 - for 1st channel, 2, 3 and 4 are 2nd 3rd and 4th
-- channels respectively. Use 5 to have multichannel tracking and display (all 4 channels).
if (OBJECT_TRACKING == 1) then
  OBJECTTRACKING_DISP = OBJECTTRACKING_DISP or 5
else
  OBJECTTRACKING_DISP = 0;
end

-- Enable Analytics Streaming
-- 0 : Don't stream analytics
-- 1 : Stream analytics as RTP metadata with rtpwriter
-- 2 : Stream analytics in Private SEI with rtpwriter
ENABLE_ANALYTICS_STREAMING = ENABLE_ANALYTICS_STREAMING or 0; 

if (ENABLE_ANALYTICS_STREAMING == 0) then
   WS_QUEUE_SIZE = WS_QUEUE_SIZE or 500
else
   WS_QUEUE_SIZE = WS_QUEUE_SIZE or 750 
end
 
if (ENABLE_ANALYTICS_STREAMING == 1) then
   SESSION_SUFFIX = SESSION_SUFFIX or "_D1"   -- The GUI used for analytic streaming requires _D1 at end
else
   SESSION_SUFFIX = SESSION_SUFFIX or ""
end

-- RTCP Session Enable Flag
RTCP_FLAG = RTCP_FLAG or 0;

-- RTCP Sender Report Interval in milliseconds
RTCP_SR_INTERVAL = RTCP_SR_INTERVAL or 5000;

-- Session Timeout Interval in Seconds (for RTCP)
SESSION_TIMEOUT = SESSION_TIMEOUT or 60;

------------------------------------------------------------------------------
-- Audio Configuration
--
-- AUD_ENCODE 0 video only 1 AV
-- 
-- Audio Codec (AUD_CODEC):
-- "aac"    = acc-lc or ld      - Q_AUDENC_CFP_CODEC_QAC
-- "mp2"    = mpeg 1 layer 2    - Q_AUDENC_CFP_CODEC_QMA
-- "pcm"    =                   - Q_AUDENC_CFP_CODEC_QPCM
-- "g711"   = PCM               - Q_AUDENC_CFP_CODEC_Q711   

DEFAULT_AUD_CH = ENCCH;
if ((DECCH > DEFAULT_AUD_CH) and (VIDEO_PLAYBACK_ONLY == 0)) then
    DEFAULT_AUD_CH = DECCH;
end

AUDIO_NUM_CHANNELS              = AUDIO_NUM_CHANNELS            or DEFAULT_AUD_CH
AUDIO_SAMPLE_RATE               = AUDIO_SAMPLE_RATE             or 16000
AUDIO_SAMPLE_SIZE               = AUDIO_SAMPLE_SIZE             or 16
AUDIO_BIT_RATE                  = AUDIO_BIT_RATE                or 16000
AUDIO_CODEC                     = AUDIO_CODEC                   or "pcm"
AUDIO_CHANNEL_TYPE              = AUDIO_CHANNEL_TYPE            or "mono"

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- Streaming protocol 0 for nothing, 1 for UDP, 2 for RTP, 3 for QBOXUDP

STREAMING_ENCCH                 = STREAMING_ENCCH               or 0
STREAMING_ENCCH_2               = STREAMING_ENCCH_2             or 0
STREAMING_MJPEGCH               = STREAMING_MJPEGCH             or 0

SERVER                          = SERVER                        or "NULL"
HOST                            = HOST                          or "192.168.40.38"
RTP_PORT                        = RTP_PORT                      or 554
UDP_PORT_ENCCH                  = UDP_PORT_ENCCH                or 9700
UDP_PORT_ENCCH_2                = UDP_PORT_ENCCH_2              or 9800
UDP_PORT_MJPEGCH                = UDP_PORT_MJPEGCH              or 9900
MAXPACKETSIZE                   = MAXPACKETSIZE                 or 1472

-- By default, don't stream the primary channels
ENCCH_TO_STREAM                 = ENCCH_TO_STREAM               or 0
ENCCH_2_TO_STREAM               = ENCCH_2_TO_STREAM             or 4 
MJPEGCH_TO_STREAM               = MJPEGCH_TO_STREAM             or 4

-- Number of encoded streams to store to disk
ENCCH_TO_STORE                  = ENCCH_TO_STORE                or 0
ENCCH_2_TO_STORE                = ENCCH_2_TO_STORE              or 0
MJPEGCH_TO_STORE                = MJPEGCH_TO_STORE              or 0

-- Port for 2nd streams (port, port+1...)
PORT_2                          = PORT_2                        or 9750

-- Width and height of each live encoded stream
if INPUTFORMAT == "576i" then
    ENCW                            = ENCW                          or 720
    ENCH                            = ENCH                          or 576
else
    ENCW                            = ENCW                          or 720
    ENCH                            = ENCH                          or 480
end

-- Width and height of each decoded stream
DECW                            = DECW                          or ENCW
DECH                            = DECH                          or ENCH

-- Number of reference frames for AVC decoder
AVCDEC_REFERENCE_FRAMES         = AVCDEC_REFERENCE_FRAMES       or 8

-- Maximum number of AUs per decoder's input buffer
DECODER_VIDEO_BUFFER_AU_SIZE    = DECODER_VIDEO_BUFFER_AU_SIZE  or 1024

-- AVOC_EXTERNAL_DEINTERLACER set to 1 to prevent line doubling on output
AVOC_EXTERNAL_DEINTERLACER = AVOC_EXTERNAL_DEINTERLACER or 1;

-- Bitrate of each live encoded stream
BITRATE                         = BITRATE                       or {}
BITRATE[1]                      = BITRATE[1]                    or 1500000
BITRATE[2]                      = BITRATE[2]                    or 1500000
BITRATE[3]                      = BITRATE[3]                    or 1500000
BITRATE[4]                      = BITRATE[4]                    or 1500000

-- Width and height of each derived encoded stream
if INPUTFORMAT == "576i" then
    ENCW_2                          = ENCW_2                        or 360
    ENCH_2                          = ENCH_2                        or 288
else
    ENCW_2                          = ENCW_2                        or 360
    ENCH_2                          = ENCH_2                        or 240
end

-- Bitrate of each derived encoded stream
BITRATE_2                       = BITRATE_2                     or {}
BITRATE_2[1]                    = BITRATE_2[1]                  or 500000
BITRATE_2[2]                    = BITRATE_2[2]                  or 400000
BITRATE_2[3]                    = BITRATE_2[3]                  or 300000
BITRATE_2[4]                    = BITRATE_2[4]                  or 200000

-- Bitrate of each MJPEG stream
BITRATE_MJPEG                   = BITRATE_MJPEG                 or {}
BITRATE_MJPEG[1]                = BITRATE_MJPEG[1]              or 5000000
BITRATE_MJPEG[2]                = BITRATE_MJPEG[2]              or 5000000
BITRATE_MJPEG[3]                = BITRATE_MJPEG[3]              or 5000000
BITRATE_MJPEG[4]                = BITRATE_MJPEG[4]              or 5000000

-- Picture quality of JPEG snapshots, 0 (worst) to 100 (best)
SNAPSHOT_QUALITY                = SNAPSHOT_QUALITY or 50;

-- Parmaters to change fps for JPEG encoding. 
JPEG_CAPTURE_INTERVAL           = JPEG_CAPTURE_INTERVAL or -1;
JPEG_FPS                        = JPEG_FPS or 30;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES              = NUM_AVC_REFERENCES            or 8

-- Preview time delays
if INPUTFORMAT == "576i" then
    AVC_RECON_PREVIEW_TIME_DELAY = AVC_RECON_PREVIEW_TIME_DELAY or -14400;
    LVPP_LIVE_PREVIEW_TIME_DELAY = LVPP_LIVE_PREVIEW_TIME_DELAY or -3600;
else
    AVC_RECON_PREVIEW_TIME_DELAY = AVC_RECON_PREVIEW_TIME_DELAY or -12012
    LVPP_LIVE_PREVIEW_TIME_DELAY = LVPP_LIVE_PREVIEW_TIME_DELAY or -3003;
end
-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF                      = RENDER_BFF                    or 0

-- GOP structure can be IP or IBBRBP
MAIN_GOP_STRUCTURE              = MAIN_GOP_STRUCTURE            or "IP"
SECONDARY_GOP_STRUCTURE         = SECONDARY_GOP_STRUCTURE       or "IP"

-- frame structure, either frame, field or mbaff
MAIN_FRAME_STRUCTURE            = MAIN_FRAME_STRUCTURE          or "mbaff"
SECONDARY_FRAME_STRUCTURE       = SECONDARY_FRAME_STRUCTURE     or "frame"

-- If in 480i mode, we will have to create 1/2 height vertical
-- images for display.  We can optionally do this by just dropping
-- a field which is done by setting this variable to 1.
DISPLAY_USING_ONE_FIELD        = DISPLAY_USING_ONE_FIELD        or 0

-- send this many blocks per interrupt
MAIN_AVCENC_XFER_SIZE           = MAIN_AVCENC_XFER_SIZE         or 4
SECONDARY_AVCENC_XFER_SIZE      = SECONDARY_AVCENC_XFER_SIZE    or 4 

-- Interval between interrupts (if non-zero) in microseconds
INTERRUPT_PERIOD                = INTERRUPT_PERIOD              or 0;

-- We need to set these to 0 for VLC 0.8.6h compatibility
-- SEE MAIN_PME_FRAME_MODE
MAIN_IP_FIELD_CODING            = MAIN_IP_FIELD_CODING          or 0
SECONDARY_IP_FIELD_CODING       = SECONDARY_IP_FIELD_CODING     or 0

-- Burn in current time into video
BURNIN_TIMECODE                 = BURNIN_TIMECODE               or 1

-- Enable burn-in at the LVPP object
-- 0: Burn-in done at AVC encoder
-- 1: Burn-in done at LVPP
BURNIN_AT_LVPP                  = BURNIN_AT_LVPP or 0;

if INPUT_FORMAT == "576i" then
   BURNIN_FPS = BURNIN_FPS or 25
else
   BURNIN_FPS = BURNIN_FPS or 30
end
-- 8, 16 or 32
BURNIN_FONTSIZE                 = BURNIN_FONTSIZE               or 16

BURNIN_FONT = {}
BURNIN_FONT[8]                  = "/var/mobiapp/fonts/MGSysFont08.bin"
BURNIN_FONT[16]                 = "/var/mobiapp/fonts/MGSysFont16.bin"
BURNIN_FONT[32]                 = "/var/mobiapp/fonts/MGSysFont32.bin"

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE                = MEMORY_XFER_MODE              or 0

-- Deinterlacer settings
MAIN_DEINT_MODE                 = MAIN_DEINT_MODE               or 0

-- Video input decimation
MAIN_INPUT_DECIMATION           = MAIN_INPUT_DECIMATION         or 1
SECONDARY_INPUT_DECIMATION      = SECONDARY_INPUT_DECIMATION    or 1

-- Fill rencder buffer color
RENDER_BUFFER_COLOR             = RENDER_BUFFER_COLOR           or 0x00108080

-- Custom display layout
CUSTOM_DISPLAY_LAYOUT           = CUSTOM_DISPLAY_LAYOUT         or 0

-- AVOC settings (no defaults on purpose)
AVOC_CMP_VPU_EXTERNAL_SYNC      = AVOC_CMP_VPU_EXTERNAL_SYNC
AVOC_CMP_VPU_START_OFFSET_V     = AVOC_CMP_VPU_START_OFFSET_V
AVOC_CMP_VPU_START_OFFSET_H     = AVOC_CMP_VPU_START_OFFSET_H
AVOC_CMP_VPU_SELECT_FIELD       = AVOC_CMP_VPU_SELECT_FIELD

-- Writestreamer options
WRITE_STREAMER_UMA              = WRITE_STREAMER_UMA            or 1
WRITE_STREAMER_SENDFILE         = WRITE_STREAMER_SENDFILE       or 1

MULTISTREAM_GATHER_ENABLE       = MULTISTREAM_GATHER_ENABLE     or 0
MULTISTREAM_GATHER_BUFFERSIZE   = MULTISTREAM_GATHER_BUFFERSIZE or (8*1024*1024)

-- Readstreamer options
READ_STREAMER_PROTOCOL          = READ_STREAMER_PROTOCOL        or "qbox"

-- Zoom channel options
ZOOMCH                          = ZOOMCH                        or 1

-- Livemedia RTP workaround
LIVEMEDIA_WORKAROUND            = LIVEMEDIA_WORKAROUND          or 0

-- Variable frame rate with NTP timestamp support
VARIABLE_FRAME_RATE_WITH_NTP    = VARIABLE_FRAME_RATE_WITH_NTP  or 0

------------------------------------------------------------------------------
--       Don't change below this line unless you know what you are doing    --
------------------------------------------------------------------------------

-- Force sample rate to 8kHz if audio codec is G711 or G726
if (AUDIO_CODEC == "g711") or (AUDIO_CODEC == "g726") then
    AUDIO_SAMPLE_RATE = 8000;
end

-- Currently the VA85 card has a bug with sample rate == 8000.
-- Set audio input to 16K hz and then use audio filter to resample to 8000.
VA85_WORK_AROUND = 0;
VA85_SAMPLE_RATE = AUDIO_SAMPLE_RATE;
if (AIN == "VA85") and (AUDIO_SAMPLE_RATE == 8000) then
   VA85_WORK_AROUND = 1;
   VA85_SAMPLE_RATE = 16000;
end

-- Transfer size must be 1 if multistream gather is enabled
if (MULTISTREAM_GATHER_ENABLE == 1) then
    MAIN_AVCENC_XFER_SIZE = 1
    SECONDARY_AVCENC_XFER_SIZE = 1
end

chinfo              = {} -- collects information about channels and displays
                         -- it at the end of execution
chinfo.config       = {}
chinfo.config['FORMAT']                 = FORMAT
chinfo.config['FORMATFPS']              = FORMATFPS
chinfo.config['INPUTFORMAT']            = INPUTFORMAT
chinfo.config['VIN']                    = VIN
chinfo.config['VOUT']                   = VOUT
chinfo.config['AIN']                    = AIN
chinfo.config["AUDIO_NUM_CHANNELS"]     = AUDIO_NUM_CHANNELS
chinfo.config["AUDIO_SAMPLE_RATE"]      = AUDIO_SAMPLE_RATE
chinfo.config["AUDIO_SAMPLE_SIZE"]      = AUDIO_SAMPLE_SIZE
chinfo.config["AUDIO_BIT_RATE"]         = AUDIO_BIT_RATE
chinfo.config["AUDIO_CHANNEL_TYPE"]     = AUDIO_CHANNEL_TYPE
chinfo.config["AUDIO_CODEC"]            = AUDIO_CODEC
------------------------------------------------------------------------------
-- The following tables will contain references to the object created.
--
lvpp                = {} -- LVPP objects
nvpp                = {} -- NVPP object
avcenc              = {} -- main channels AVCENC
audenc              = {} -- main channels AUDENC
avsnenc             = {} -- recon avcenc avsn
avsnlive            = {} -- live  avcenc avsn
nvppenc2            = {} -- secondary channels NVPPs
avcenc2             = {} -- secondary channel AVCENC
jpegenc             = {} -- JPEG encoders
avdec               = {} -- AV decoder objects
avsndec             = {} -- decoder avsn
avsnzoom            = {} -- zoom avsn
sain                = {} -- stereo input
asplit              = {} -- audio splitter - Added in SDK4.0

------------------------------------------------------------------------------
-- Audio related information
lach                = {} -- live audio channels
dach                = {} -- decoded audio channels

ActiveAudioChannel  = nil

------------------------------------------------------------------------------
-- Live display related information
evch                = {} -- recon video channels
EncRenderDisable    = {} -- one entry per video channel - 0 means enabled 
                         --                               1 disabled
EncChannelX         = {} -- X coordinate of each video channels 
EncChannelY         = {} -- Y coordinate of each video channel

------------------------------------------------------------------------------
-- Live display related information
lvch                = {} -- recon video channels
LiveRenderDisable   = {} -- one entry per video channel - 0 means enabled
                         --                               1 disabled
LiveChannelX        = {} -- X coordinate of each video channels
LiveChannelY        = {} -- Y coordinate of each video channels

------------------------------------------------------------------------------
-- Zoom display related information
zvch                = {} -- zoom video channels
ZoomRenderDisable   = {} -- one entry per video channel - 0 means enabled
                         --                               1 disabled
ZoomChannelX        = {} -- X coordinate of each video channels
ZoomChannelY        = {} -- Y coordinate of each video channels

------------------------------------------------------------------------------
-- Decoder display related information
dvch                = {} -- Decoder video channels
DecRenderDisable    = {} -- one entry per video channel - 0 means enabled
                         --                               1 disabled
DecChannelX         = {} -- X coordinate of each video channels
DecChannelY         = {} -- Y coordinate of each video channels

FileToDecCh         = {} -- Translation from file to decode channel
DecChActive         = {} -- Flag to indicate whether a decode channel is active

bsr                 = {} -- references to bitstream readers

gvch                = {} -- global video channel table



AUDIO_CODEC         = string.lower(AUDIO_CODEC)
AUDIO_CHANNEL_TYPE  = string.lower(AUDIO_CHANNEL_TYPE)

AUDIO_CODECS = 
{
    ['aac']   =   Q_AUDENC_CFP_CODEC_QAC,
    ['aac-ld']=   Q_AUDENC_CFP_CODEC_QAC,    
    ['pcm']   =   Q_AUDENC_CFP_CODEC_QPCM,
    ['mp2']   =   Q_AUDENC_CFP_CODEC_QMA,
    ['g711']  =   Q_AUDENC_CFP_CODEC_Q711,
    ['g722']  =   Q_AUDENC_CFP_CODEC_Q722,
    ['g726']  =   Q_AUDENC_CFP_CODEC_Q726,
    ['g728']  =   Q_AUDENC_CFP_CODEC_Q728,
}

AUDIO_CHANNEL_TYPES = 
{
    ['mono']  =   Q_AUDENC_CFP_CHANNEL_MONO,
    ['stereo']=   Q_AUDENC_CFP_CHANNEL_STEREO  
}

AUDIO_FRAME_SIZES = 
{
    ['aac']   =   1024,
    ['aac-ld']=    512,   
    ['pcm']   =   1024,
    ['mp2']   =   1152,
    ['g711']  =   1024,  
    ['g722']  =   1024,  
    ['g726']  =   1024,  
    ['g728']  =   1024,  
}

AUDIO_AOT = 
{
    ['aac']   =    2,
    ['aac-ld']=   23,   
    ['pcm']   =    2,
    ['mp2']   =    2,
    ['g711']  =    2,  
    ['g722']  =    2,  
    ['g726']  =    2, 
    ['g728']  =    2,
}

AUDIO_RTP_TYPE = 
{
    ['aac']   =   'aac',
    ['aac-ld']=   'null',   
    ['pcm']   =   'null',
    ['mp2']   =   'null',
    ['g711']  =   'g711u',  
    ['g722']  =   'null',  
    ['g726']  =   'g726',  
    ['g728']  =   'null',  
}

------------------------------------------------------------------------------
-- Main Encoder Configurations
--
-- MAIN_TF_Y_BYPASS
-- MAIN_PME_ENABLE
-- MAIN_PME_FRAME_MODE is being set to 1. Make sure this is only set if we are 
--                     doing frame coding or mbaff. Setting this to 1 will save 
--                     some BW, since we are not padding input frames to 512
--                     Since I-frame could be coded as IP field pair. Lets turn 
--                     it off. 
-- MAIN_PME_W
-- MAIN_PME_H
-- MAIN_STC_PER_FRAME
-- MAIN_STC_PER_OUTPUT_FRAME
-- MAIN_SINGLE_FIELD_DOWNSCALE
-- MAIN_CLOSED_GOP
-- MAIN_NVPP_INTERLACED

MAIN_TF_Y_BYPASS                = MAIN_TF_Y_BYPASS              or 0
MAIN_PME_ENABLE                 = MAIN_PME_ENABLE               or 1
MAIN_PME_FRAME_MODE             = MAIN_PME_FRAME_MODE           or 1
MAIN_PME_W                      = MAIN_PME_W                    or 2
MAIN_PME_H                      = MAIN_PME_H                    or 4

if(MAIN_FRAME_STRUCTURE == "field") then
    MAIN_PME_FRAME_MODE = 0;
end

if INPUTFORMAT == "576i" then
    MAIN_STC_PER_FRAME              = MAIN_STC_PER_FRAME            or 3600
    MAIN_STC_PER_OUTPUT_FRAME       = MAIN_STC_PER_OUTPUT_FRAME     or 3600
else
    MAIN_STC_PER_FRAME              = MAIN_STC_PER_FRAME            or 3003
    MAIN_STC_PER_OUTPUT_FRAME       = MAIN_STC_PER_OUTPUT_FRAME     or 3003
end
MAIN_ENCODER_FRAME_RATE         = MAIN_ENCODER_FRAME_RATE       or 3003
MAIN_SINGLE_FIELD_DOWNSCALE     = MAIN_SINGLE_FIELD_DOWNSCALE   or 0
MAIN_CLOSED_GOP                 = MAIN_CLOSED_GOP               or 1
MAIN_LVPP_INTERLACED            = MAIN_LVPP_INTERLACED          or 1
MAIN_CAPTURE_FRAMES             = MAIN_CAPTURE_FRAMES           or 7


------------------------------------------------------------------------------
-- SECONDARY_TF_Y_BYPASS
-- SECONDARY_PME_ENABLE
-- SECONDARY_PME_FRAME_MODE
-- SECONDARY_PME_W
-- SECONDARY_PME_H
-- SECONDARY_STC_PER_FRAME
-- SECONDARY_STC_PER_OUTPUT_FRAME
-- SECONDARY_SINGLE_FIELD_DOWNSCALE
-- SECONDARY_CLOSED_GOP
-- SECONDARY_NVPP_INTERLACED
-- SECONDARY_AVC_REFERENCES
-- SECONDARY_NVPP_BYPASS      If set to 1, the secondary avcenc are connected 
--                            directly to the lvpp (bypassing the NVPP)
-- SECONDARY_NVPP_FRAMES

SECONDARY_TF_Y_BYPASS           = SECONDARY_TF_Y_BYPASS         or 0
SECONDARY_PME_ENABLE            = SECONDARY_PME_ENABLE          or 0
SECONDARY_PME_FRAME_MODE        = SECONDARY_PME_FRAME_MODE      or 1
SECONDARY_PME_W                 = SECONDARY_PME_W               or 0
SECONDARY_PME_H                 = SECONDARY_PME_H               or 0
SECONDARY_STC_PER_FRAME         = SECONDARY_STC_PER_FRAME       or MAIN_STC_PER_FRAME
SECONDARY_STC_PER_OUTPUT_FRAME  = SECONDARY_STC_PER_OUTPUT_FRAME or MAIN_STC_PER_OUTPUT_FRAME
SECONDARY_ENCODER_FRAME_RATE    = SECONDARY_ENCODER_FRAME_RATE or MAIN_STC_PER_OUTPUT_FRAME
SECONDARY_SINGLE_FIELD_DOWNSCALE = SECONDARY_SINGLE_FIELD_DOWNSCALE or 1
SECONDARY_CLOSED_GOP            = SECONDARY_CLOSED_GOP          or 1
SECONDARY_NVPP_INTERLACED       = SECONDARY_NVPP_INTERLACED     or 0

SECONDARY_AVC_REFERENCES        = SECONDARY_AVC_REFERENCES      or 2

SECONDARY_NVPP_BYPASS           = SECONDARY_NVPP_BYPASS         or 0
SECONDARY_NVPP_FRAMES           = SECONDARY_NVPP_FRAMES         or 4

if(SECONDARY_FRAME_STRUCTURE == "field") then
    SECONDARY_PME_FRAME_MODE = 0;
end
------------------------------------------------------------------------------
--
MAIN_ENC_BUFFER_SIZE            = MAIN_ENC_BUFFER_SIZE          or 1000000
SECONDARY_ENC_BUFFER_SIZE       = SECONDARY_ENC_BUFFER_SIZE     or 500000

MAIN_ENC_GUARD_BAND             = MAIN_ENC_GUARD_BAND           or 200
SECONDARY_ENC_GUARD_BAND        = SECONDARY_ENC_GUARD_BAND      or 200
JPEG_ENC_GUARD_BAND             = JPEG_ENC_GUARD_BAND           or 300

MAIN_AUD_ENC_BUFFER_SIZE        = MAIN_AUD_ENC_BUFFER_SIZE      or 128000

MJPEG_ENC_BUFFER_SIZE           = MJPEG_ENC_BUFFER_SIZE         or 2000000
SNAPSHOT_ENC_BUFFER_SIZE        = SNAPSHOT_ENC_BUFFER_SIZE      or 2000000

DECODER_VIDEO_BUFFER_SIZE       = DECODER_VIDEO_BUFFER_SIZE     or 1000000
DECODER_AUDIO_BUFFER_SIZE       = DECODER_AUDIO_BUFFER_SIZE     or 200000

------------------------------------------------------------------------------
--
MAX_LVPP_USED                   = MAX_LVPP_USED                 or 4

DECODER_PRIORITY                = DECODER_PRIORITY              or 1

BITSTREAM_OUTPUT                = BITSTREAM_OUTPUT              or 1


------------------------------------------------------------------------------
-- Set display configuration parameters based on the output format
--
if (FORMAT == "1080i") then

  -- A 3x2 grid of 480x480 images is possible, but typically only 4 sources are displayed
  DISPLAYWIDTH                  = DISPLAYWIDTH                  or 1920
  DISPLAYHEIGHT                 = DISPLAYHEIGHT                 or 1080
  DISPLAYOFFSET_X               = DISPLAYOFFSET_X               or 0
  DISPLAYOFFSET_Y               = DISPLAYOFFSET_Y               or 0
  COMP_PIXEL_AR_X               = COMP_PIXEL_AR_X               or 1
  COMP_PIXEL_AR_Y               = COMP_PIXEL_AR_Y               or 1

  if INPUTFORMAT == "576i" then
      OUTW                      = OUTW                          or 720
      OUTH                      = OUTH                          or 540
  else
      OUTW                      = OUTW                          or 720
      OUTH                      = OUTH                          or 480
  end

  OUTGRIDW                      = OUTGRIDW                      or 2
  OUTGRIDH                      = OUTGRIDH                      or 2

elseif (FORMAT == "720p") then

  -- 2x2 image grid will be scaled down
  DISPLAYWIDTH                  = DISPLAYWIDTH                  or 1280;
  DISPLAYHEIGHT                 = DISPLAYHEIGHT                 or 720;
  DISPLAYOFFSET_X               = DISPLAYOFFSET_X               or 0;
  DISPLAYOFFSET_Y               = DISPLAYOFFSET_Y               or 0;
  COMP_PIXEL_AR_X               = COMP_PIXEL_AR_X               or 1;
  COMP_PIXEL_AR_Y               = COMP_PIXEL_AR_Y               or 1;
  OUTW                          = OUTW                          or 720;
  OUTH                          = OUTH                          or 480;
  OUTGRIDW                      = OUTGRIDW                      or 2;
  OUTGRIDH                      = OUTGRIDH                      or 2;

elseif (FORMAT == "480i") then

  DISPLAYWIDTH                  = DISPLAYWIDTH                  or 720
  DISPLAYHEIGHT                 = DISPLAYHEIGHT                 or 480
  DISPLAYOFFSET_X               = DISPLAYOFFSET_X               or 0
  DISPLAYOFFSET_Y               = DISPLAYOFFSET_Y               or 0
  COMP_PIXEL_AR_X               = COMP_PIXEL_AR_X               or 8
  COMP_PIXEL_AR_Y               = COMP_PIXEL_AR_Y               or 9
  OUTW                          = OUTW                          or 360
  OUTH                          = OUTH                          or 240
  OUTGRIDW                      = OUTGRIDW                      or 2
  OUTGRIDH                      = OUTGRIDH                      or 2

elseif (FORMAT == "576i") then

  DISPLAYWIDTH                  = DISPLAYWIDTH                  or 720
  DISPLAYHEIGHT                 = DISPLAYHEIGHT                 or 576
  DISPLAYOFFSET_X               = DISPLAYOFFSET_X               or 0
  DISPLAYOFFSET_Y               = DISPLAYOFFSET_Y               or 0
  COMP_PIXEL_AR_X               = COMP_PIXEL_AR_X               or 16
  COMP_PIXEL_AR_Y               = COMP_PIXEL_AR_Y               or 15
  OUTW                          = OUTW                          or 360
  OUTH                          = OUTH                          or 288
  OUTGRIDW                      = OUTGRIDW                      or 2
  OUTGRIDH                      = OUTGRIDH                      or 2
  
end

COMPWIDTH                       = COMPWIDTH                     or OUTGRIDW*OUTW
COMPHEIGHT                      = COMPHEIGHT                    or OUTGRIDH*OUTH

ZOOMOUTW                        = ZOOMOUTW                      or COMPWIDTH
ZOOMOUTH                        = ZOOMOUTH                      or COMPHEIGHT

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC
end

-- Set if each file has A/V channels 1 and 2, instead of different channels per file
SINGLE_CHANNEL_FILES = SINGLE_CHANNEL_FILES or 0;

MainOutputFileName = MainOutputFileName or {}
MainOutputFileName[1] = MainOutputFileName[1] or "/media/sda1/vid/main0.qbx"
MainOutputFileName[2] = MainOutputFileName[2] or "/media/sda1/vid/main1.qbx"
MainOutputFileName[3] = MainOutputFileName[3] or "/media/sda1/vid/main2.qbx"
MainOutputFileName[4] = MainOutputFileName[4] or "/media/sda1/vid/main3.qbx"

SecOutputFileName = SecOutputFileName or {}
SecOutputFileName[1] = SecOutputFileName[1] or "/media/sda1/vid/sec0.qbx"
SecOutputFileName[2] = SecOutputFileName[2] or "/media/sda1/vid/sec1.qbx"
SecOutputFileName[3] = SecOutputFileName[3] or "/media/sda1/vid/sec2.qbx"
SecOutputFileName[4] = SecOutputFileName[4] or "/media/sda1/vid/sec3.qbx"

JpgOutputFileName = JpgOutputFileName or {}
JpgOutputFileName[1] = JpgOutputFileName[1] or "/media/sda1/vid/jpeg0.qbx"
JpgOutputFileName[2] = JpgOutputFileName[2] or "/media/sda1/vid/jpeg1.qbx"
JpgOutputFileName[3] = JpgOutputFileName[3] or "/media/sda1/vid/jpeg2.qbx"
JpgOutputFileName[4] = JpgOutputFileName[4] or "/media/sda1/vid/jpeg3.qbx"

ShotOutputFileName = ShotOutputFileName or "/media/sda1/vid/snapshot";

MainInputFileName = MainInputFileName or {}
MainInputFileName[1] = MainInputFileName[1] or "/media/sda1/vid/main0.qbx"
MainInputFileName[2] = MainInputFileName[2] or "/media/sda1/vid/main1.qbx"
MainInputFileName[3] = MainInputFileName[3] or "/media/sda1/vid/main2.qbx"
MainInputFileName[4] = MainInputFileName[4] or "/media/sda1/vid/main3.qbx"


------------------------------------------------------------------------------
-- Set input configuration paramaters based on input format
--

if (INPUTFORMAT == "480i") then
    VINBFF                = 0
    VINSYNCHEIGHT         = 525
    VINFIELDOFFSET        = 263
    VINSTARTVLINE         = 22
    VINWIDTH              = 720
    VINHEIGHT             = 480
    VIN_PIXEL_AR_X        = 8
    VIN_PIXEL_AR_Y        = 9
elseif (INPUTFORMAT == "576i") then
    VINBFF                = 0
    VINSYNCHEIGHT         = 625
    if (VIN == "VA85") then
       VINFIELDOFFSET        = 314
       VINSTARTVLINE         = 23
    else
       VINFIELDOFFSET        = 313
       VINSTARTVLINE         = 24
    end
    VINWIDTH              = 720
    VINHEIGHT             = 576
    VIN_PIXEL_AR_X        = 16
    VIN_PIXEL_AR_Y        = 15
end


if VIN == "kite" or VIN == "VA85" then

    VIDEOCONTROL            = VIDEOCONTROL or 0x80101080
    VININTERLEAVE           = VININTERLEAVE or 4
    VINSYNCMODE             = Q_SYS_CFP_VPORT_SYNC_MODE_MOBI
    VINFIELDOFFSET          = VINFIELDOFFSET - 1
       
else

    VIDEOCONTROL            = 0x00101000
    VININTERLEAVE           = 1
    VINSYNCMODE             = Q_SYS_CFP_VPORT_SYNC_MODE_NONE

end


if (MAX_LVPP_USED > VININTERLEAVE) then
  MAX_LVPP_USED = VININTERLEAVE
end
AUDIO_MAX_SAIN  = 0 -- computed based on the number of audio channels



------------------------------------------------------------------------------
-- System controller configuration.
--

sysctl0:setVPortMode    {
                            port                =   0, 
                            live                =   1, 
                            interlaced          =   1, 
                            bottomfieldfirst    =   VINBFF, 
                            videocontrol        =   VIDEOCONTROL, 
                            interleave          =   VININTERLEAVE, 
                            syncmode            =   VINSYNCMODE, 
                            syncheight          =   VINSYNCHEIGHT
                        }

-- Interrupt aggregation
if (INTERRUPT_PERIOD > 0) then
  print("Sysctl: Aggregating interrupts every "..INTERRUPT_PERIOD.." usecs\n");
  sysctl0:configure({param=Q_SYS_CFG_EVENT_CYCLE_IN_USEC, value=INTERRUPT_PERIOD});
end
                        
sysctl0:setVPortMode    {
                            port                =   1, 
                            live                =   0
                        }

if AUDIO_NUM_CHANNELS > 0 then
  if (VA85_WORK_AROUND == 0) then
    board.setAudioInput { port=0, freq=AUDIO_SAMPLE_RATE, mode=AIN }
    if (AIN == "onboard") then 
      board.setAudioInput { port=1, freq=AUDIO_SAMPLE_RATE, mode=AIN }
    end
  else
    board.setAudioInput { port=0, freq=VA85_SAMPLE_RATE, mode=AIN }
  end
end

board.setVideoInput     {
                            port                =   0, 
                            format              =   INPUTFORMAT, 
                            mode                =   VIN
                        }

if board.getType() == "merlinbup" then
  vout_sync = 'external'
end

clock_ntsc = 1
if FORMATFPS == 50 then
  clock_ntsc = 0
end


board.setVideoOutput    {
                            port                =   0, 
                            format              =   FORMAT, 
                            mode                =   VOUT, 
                            sync                =   vout_sync,
                            clockntsc           =   clock_ntsc
                        }

sysctl0:setVerboseLevel {param=1}

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0
-- show or hide rs update 0 for off, 1 for on
SHOWUPDATE = 0

------------------------------------------------------------------------------
-- Allocate bitstream partition memory
-- Each value should be rounded up to a multiple of 4096, because individual buffers will be
-- allocated using that alignment.
DEMUX_ADEC_MEM = (DECODER_AUDIO_BUFFER_SIZE + 4096) / 4096 * 4096;
DEMUX_VDEC_MEM = (DECODER_VIDEO_BUFFER_SIZE + 4096) / 4096 * 4096;
HSBP_ENC_MEM = 0;
if (HSBP_OUTPUT > 0) then
    HSBP_ENC_VID_MEM = MAIN_ENC_BUFFER_SIZE * ENCCH;
    HSBP_ENC_AUD_MEM = MAIN_AUD_ENC_BUFFER_SIZE * AUDIO_NUM_CHANNELS;
    HSBP_ENC2_VID_MEM = SECONDARY_ENC_BUFFER_SIZE * ENCCH_2;
    HSBP_MJPEG_MEM = MJPEG_ENC_BUFFER_SIZE * MJPEGCH;
    HSBP_SNAPSHOT_MEM = SNAPSHOT_ENC_BUFFER_SIZE;
    HSBP_ENC_MEM = HSBP_ENC_VID_MEM + HSBP_ENC_AUD_MEM + HSBP_ENC2_VID_MEM + HSBP_MJPEG_MEM + HSBP_SNAPSHOT_MEM;
end
INPUT_PARTITION_SIZE = ((DECCH) * (DEMUX_VDEC_MEM + DEMUX_ADEC_MEM) + HSBP_ENC_MEM);

sysctl0:configure
            {
                param   =   Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE,
                value   =   INPUT_PARTITION_SIZE
            }

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create and configure the AVOC
if(OBJECTTRACKING_DISP > 0) then

    if (FORMAT == "720p") or (FORMAT == "1080i") then
        OSD_MEMORY_SIZE     = 8*1024*1024;
    else
        OSD_MEMORY_SIZE     = 4*1024*1024;
    end

    avoc0 = sysctl0:avocCreate  {
                name          =   "avoc0", 
                maxWidth      =   COMPWIDTH, 
                maxHeight     =   COMPHEIGHT,
                gfx0Size      =   OSD_MEMORY_SIZE,
                allocGfx      =   1
    }

else

    avoc0 = sysctl0:avocCreate  {
                name          =   "avoc0", 
                maxWidth      =   COMPWIDTH, 
                maxHeight     =   COMPHEIGHT
    }

end

avoc0:fillRenderBuffers     { yuv               =   RENDER_BUFFER_COLOR    }

if (FORMAT == "1080i") then
    if (FORMATFPS == 50) then
        avoc0:setOutput1080i50({sync=vout_sync});
    else
        avoc0:setOutput1080i60({sync=vout_sync});
    end
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "480i") then
  avoc0:setOutput480i()
elseif (FORMAT == "576i") then
  avoc0:setOutput576i()
end

-- prevent field doubling per channel
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_EXTERNAL_DEINTERLACER,
                            value   =   AVOC_EXTERNAL_DEINTERLACER 
                 }

if AVOC_CMP_VPU_EXTERNAL_SYNC ~= nil then
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VPU_EXTERNAL_SYNC,
                            value   =   AVOC_CMP_VPU_EXTERNAL_SYNC
                 }
end
if AVOC_CMP_VPU_START_OFFSET_V ~= nil then
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VPU_START_OFFSET_V,
                            value   =   AVOC_CMP_VPU_START_OFFSET_V
                 }
end
if AVOC_CMP_VPU_START_OFFSET_H ~= nil then
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VPU_START_OFFSET_H,
                            value   =   AVOC_CMP_VPU_START_OFFSET_H
                 }
end
if AVOC_CMP_VPU_SELECT_FIELD ~= nil then
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VPU_SELECT_FIELD,
                            value   =   AVOC_CMP_VPU_SELECT_FIELD
                 }
end

avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, 
                            value   =   DISPLAYOFFSET_X
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, 
                            value   =   DISPLAYOFFSET_Y
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_DISPLAY_WIDTH,    
                            value   =   DISPLAYWIDTH
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT,
                            value   =   DISPLAYHEIGHT
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_COMP_ENABLE,      
                            value   =   1
                 }
avoc0:setParam   {
                            param   =  Q_AVOC_CMP_VID_0_COMP_WIDTH,       
                            value   =  COMPWIDTH
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_COMP_HEIGHT,      
                            value   =   COMPHEIGHT
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_COMP_PIXEL_AR_X,  
                            value   =   COMP_PIXEL_AR_X
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_COMP_PIXEL_AR_Y,  
                            value   =   COMP_PIXEL_AR_Y
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VID_0_COMP_BFF,         
                            value   =   RENDER_BFF
                 }
avoc0:setParam   {
                            param   =   Q_AVOC_CMP_VPU_BG_COLOR,           
                            value   =   RENDER_BUFFER_COLOR * 256
                 }
                 
if AUDIO_NUM_CHANNELS > 0 then

    avoc0:setParam   {
                                param   =   Q_AVOC_CMP_AUD_SERIAL_MODE, 
                                value   =   Q_AVOC_CFP_AUD_SERIAL_MODE_I2S
                     }
    avoc0:setParam   {
                                param   =   Q_AVOC_CMP_AUD_MASTER_CLOCK, 
                                value   =   Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS
                     }
    avoc0:setParam   {
                                param   =   Q_AVOC_CMP_AUD_SAMPLE_RATE, 
                                value   =   AUDIO_SAMPLE_RATE
                     }
    avoc0:setParam   {
                                param   =   Q_AVOC_CMP_AUD_SAMPLE_SIZE, 
                                value   =   AUDIO_SAMPLE_SIZE
                     }

end

avoc0:activateCfg()
if (FORMAT == "720p") then
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_RATE_DIVIDER, value=2});
end

avoc0:activateCfg();


------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- SAIN and Audio Splitter creation

if AUDIO_NUM_CHANNELS > 0 then
   if (AIN == "VA85") then
      -- VA85 only uses 1 audio input port (0)
      AUDIO_MAX_SAIN = 1;
      sain[1] = sysctl0:sainCreate({name="sain1", port=0});
      sain[1]:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM,
			  value=Q_SAIN_CFP_CHANNEL_STEREO});

      sain[1]:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE,
			  value=Q_SAIN_CFP_AUD_SERIAL_MODE_I2S});

      sain[1]:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK,
			  value=Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS});

      if (VA85_WORK_AROUND == 0) then
	 sain[1]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE,
			     value=AUDIO_SAMPLE_RATE});
	 sain[1]:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE,
			  value=AUDIO_FRAME_SIZES[AUDIO_CODEC]});
      else
	 sain[1]:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE,
			     value=VA85_SAMPLE_RATE});
	 sain[1]:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE,
			     value=2*AUDIO_FRAME_SIZES[AUDIO_CODEC]});
	 -- creates a resampling filter (16K to 8K)
	 audfil0 = sysctl0:audfilterCreate({name="audfil0", 
					      numOutputPorts=1, 
					      maxFramesSize=2*AUDIO_FRAME_SIZES[AUDIO_CODEC],
					      filterType=Q_AUDFLTR_CFP_TYPE_RATECONVERSION});
	 audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FREQ, value=8000});
	 audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_OUT_FRAME_NUM_SAMPLES, value=AUDIO_FRAME_SIZES[AUDIO_CODEC]});
	 audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FREQ, value=16000});
	 audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_NUM_CHANNELS, value=2}); --stereo
	 audfil0:setParam({param=Q_AUDFLTR_CMP_RATE_CONV_IN_FRAME_NUM_SAMPLES, value=2*AUDIO_FRAME_SIZES[AUDIO_CODEC]});
	 audfil0:activateCfg();
	 
	 afinport = audfil0:addInputPort();
	 audfil0:setInputPortSource({inport=afinport, src=sain[1], threshold=1});
      end

      sain[1]:setParam({param=Q_SAIN_CMP_AUD_CLOCK_SOURCE,
			  value=Q_SAIN_CFP_AUD_EXTERNAL_CLK});

      if VARIABLE_FRAME_RATE_WITH_NTP == 1 then
	 sain[1]:setParam({param=Q_SAIN_CMP_AUD_NTP_CLOCK_ENABLE, 
			     value=1});
      end

      sain[1]:activateCfg();

      num_split = math.floor(AUDIO_NUM_CHANNELS/2) + AUDIO_NUM_CHANNELS % 2
      for i = 1, num_split do
	 asplit[i] = sysctl0:asplitCreate
                            {
                                name    =  "asplit".. i, 
                                port    =   i-1, 
                                numOutput=  math.min(AUDIO_NUM_CHANNELS-((i-1)*2), 2)
                            }
        if (VA85_WORK_AROUND == 0) then
	   asplit[i]:bindAudioInput{   src =   sain[1] }
	else
	   asplit[i]:bindAudioInput{   src =   audfil0, port=0 }
	end
     end

      
  else
    -- 2 channels per SAIN - need to be rounded up to biggest integer
    AUDIO_MAX_SAIN = math.floor(AUDIO_NUM_CHANNELS/2) + AUDIO_NUM_CHANNELS % 2
      
    local cnum = Q_SAIN_CFP_CHANNEL_STEREO
      
    for i = 1, AUDIO_MAX_SAIN do
        
        -- Check if the number of channels in odd and set the last SAIN
        -- to mono
        if i == AUDIO_MAX_SAIN and AUDIO_NUM_CHANNELS%2 > 0 then
            print("SAIN set to channel mono\n")
            cnum = Q_SAIN_CFP_CHANNEL_MONO_L
        end
        
        sain[i]     = sysctl0:sainCreate    {name   =   "sain".. i, port=i-1}
        
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_CHANNEL_NUM, 
                                value   =   cnum
                            }
        
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_SERIAL_MODE, 
                                value   =   Q_SAIN_CFP_AUD_SERIAL_MODE_I2S
                            }
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_MASTER_CLOCK, 
                                value   =   Q_SAIN_CFP_AUD_MASTER_CLOCK_256FS
                            }
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_SAMPLE_RATE, 
                                value   =   AUDIO_SAMPLE_RATE
                            }
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_SAMPLE_SIZE, 
                                value   =   AUDIO_SAMPLE_SIZE
                            }
        sain[i]:setParam    {
                                param   =   Q_SAIN_CMP_AUD_FRAME_SIZE, 
                                value   =   AUDIO_FRAME_SIZES[AUDIO_CODEC]
                            }
                            
	if (AIN == "kite") or (AIN == "VA85") then
            sain[i]:setParam{
                                param   =   Q_SAIN_CMP_AUD_CLOCK_SOURCE, 
                                value   =   Q_SAIN_CFP_AUD_EXTERNAL_CLK
                            }    
        end
        
        if VARIABLE_FRAME_RATE_WITH_NTP == 1 then
            sain[i]:setParam{
                                param   =   Q_SAIN_CMP_AUD_NTP_CLOCK_ENABLE, 
                                value   =   1
                            }    
        end
        
        sain[i]:activateCfg()
        
        -- Create the audio splitter
        -- In case of odd number of channels, last asplit has its number 
        -- of output set to 1
        
        --print("[AUDIO ] SPLITTER CONFIG", i, math.min(AUDIO_NUM_CHANNELS-((i-1)*2), 2))

        asplit[i] = sysctl0:asplitCreate    
                            {
                                name    =  "asplit".. i, 
                                port    =   i-1, 
                                numOutput=  math.min(AUDIO_NUM_CHANNELS-((i-1)*2), 2)
                            }
                            
        asplit[i]:bindAudioInput{   src =   sain[i] }

        
     end

  end
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Load fonts if needed
if (BURNIN_TIMECODE ~= 0) then

  -- check if font file exists and if not then turn off burnin
  if (io.open(BURNIN_FONT[BURNIN_FONTSIZE]) == nil) then
    
    print("Warning: Could not load font file "..BURNIN_FONT[BURNIN_FONTSIZE].."\n")
    BURNIN_TIMECODE = 0
    
  else
  
    BurnInFontAddr = sysctl0:allocBurnInFont{fontsize=BURNIN_FONTSIZE}
    
    sysctl0:loadBurnInFont  
                {
                    fontaddr    =   BurnInFontAddr, 
                    fontsize    =   BURNIN_FONTSIZE, 
                    filename    =   BURNIN_FONT[BURNIN_FONTSIZE]
                }
  
  end
  
end

------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Create and configure the LVPP
--

for i = 1,MAX_LVPP_USED do

  if MAIN_DEINT_MODE == 0 then

  lvpp[i] = sysctl0:lvppCreate  
                        {
                             name               =   "lvpp" .. i, 
                             slot               =   i-1, 
                             port               =   0, 
                             maxWidth           =   ENCW, 
                             maxHeight          =   ENCH, 
                             maxPMEDSFactor     =   MAIN_PME_W, 
                             numVidFrame        =   MAIN_CAPTURE_FRAMES,
                             burninEnable       =   BURNIN_AT_LVPP
                        }


  Setup_VPP(lvpp[i], VINWIDTH, VINHEIGHT, MAIN_LVPP_INTERLACED, 0, MAIN_PME_W, MAIN_PME_H)

  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PICT_WIDTH,
                            value   =   ENCW
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PICT_HEIGHT,
                            value   =   ENCH
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_FIELD_OFFSET,         
                            value   =   VINFIELDOFFSET
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_START_HPIXEL,         
                            value   =   0
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_START_VLINE,          
                            value   =   VINSTARTVLINE
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_STC_PER_FRAME,        
                            value   =   MAIN_STC_PER_FRAME
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, 
                            value   =   MAIN_STC_PER_OUTPUT_FRAME
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PMEDS_EN,             
                            value   =   MAIN_PME_ENABLE
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PIXEL_AR_X,           
                            value   =   VIN_PIXEL_AR_X
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PIXEL_AR_Y,           
                            value   =   VIN_PIXEL_AR_Y
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTY,     
                            value   =   MAIN_TF_Y_BYPASS
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTC,     
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_MB_STAT,    
                            value   =   0
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PMEDS_FRAME_MODE,     
                            value   =   MAIN_PME_FRAME_MODE
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_INPUT_DECIMATION,
                            value   =   MAIN_INPUT_DECIMATION
                        }

  else

  lvpp[i] = sysctl0:lvppCreate
                        {
                             name               =   "lvpp" .. i,
                             slot               =   i-1,
                             port               =   0,
                             maxWidth           =   ENCW,
                             maxHeight          =   ENCH,
                             maxPMEDSFactor     =   0,
                             numVidFrame        =   6,
                             burninEnable       =   BURNIN_AT_LVPP
                        }

  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_CROP_WIDTH,
                            value   =   VINWIDTH
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_CROP_HEIGHT,
                            value   =   VINHEIGHT
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PICT_WIDTH,
                            value   =   ENCW
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PICT_HEIGHT,
                            value   =   ENCH
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_FIELD_OFFSET,
                            value   =   VINFIELDOFFSET
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_START_HPIXEL,
                            value   =   0
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_START_VLINE,
                            value   =   VINSTARTVLINE
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_STC_PER_FRAME,
                            value   =   MAIN_STC_PER_FRAME
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_STC_PER_OUTPUT_FRAME,
                            value   =   MAIN_STC_PER_OUTPUT_FRAME
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PMEDS_EN,
                            value   =   0
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PIXEL_AR_X,
                            value   =   VIN_PIXEL_AR_X
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_PIXEL_AR_Y,
                            value   =   VIN_PIXEL_AR_Y
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTY,
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTC,
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_TSTAT,
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_MB_STAT,
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_ALL_STAT,
                            value   =   1
                        }
  lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_HW_BYPASS_REGION,
                            value   =   1
                        }

  nvpp[i] = sysctl0:nvppCreate
                {
                            name            =   "lnvpp" .. i,
                            maxWidth        =   ENCW,
                            maxHeight       =   ENCH,
                            maxPMEDSFactor  =   MAIN_PME_W,
                            numVidFrame     =   MAIN_CAPTURE_FRAMES,
                            maxDeintMode    =   MAIN_DEINT_MODE
                }

  Setup_VPP(nvpp[i], ENCW, ENCH, 0, 0, MAIN_PME_W, MAIN_PME_H)

  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PICT_WIDTH,
                            value   =   ENCW
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PICT_HEIGHT,
                            value   =   ENCH
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_STC_PER_FRAME,
                            value   =   MAIN_STC_PER_FRAME * 2
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_STC_PER_OUTPUT_FRAME,
                            value   =   MAIN_STC_PER_OUTPUT_FRAME
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PMEDS_EN,
                            value   =   MAIN_PME_ENABLE
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PIXEL_AR_X,
                            value   =   VIN_PIXEL_AR_X
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PIXEL_AR_Y,
                            value   =   VIN_PIXEL_AR_Y
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_TFILTY,
                            value   =   MAIN_TF_Y_BYPASS
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_TFILTC,
                            value   =   1
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_MB_STAT,
                            value   =   0
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PMEDS_FRAME_MODE,
                            value   =   MAIN_PME_FRAME_MODE
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_DEINT_MODE,
                            value   =   MAIN_DEINT_MODE
                }
  nvpp[i]:setParam
                {
                            param   =   Q_NVPP_CMP_INPUT_DECIMATION,
                            value   =   2
                }

  if VARIABLE_FRAME_RATE_WITH_NTP == 1 then
    nvpp[i]:setParam 
                {
                            param   =   Q_NVPP_CMP_PTS_PASSTHROUGH, 
                            value   =   1
                }
  end
  
  nvpp[i]:activateCfg()

  nvpp[i]:bindInput {src=lvpp[i]}

  end

  if i <= AUDIO_NUM_CHANNELS then -- If there is a corresponding audio channel
  
    lvpp[i]:setParam      {
                            param   =   Q_LVPP_CMP_AV_SYNCH_ENABLE, 
                            value   =   1
                        }
                        
  end
  
  if VARIABLE_FRAME_RATE_WITH_NTP == 1 then
    lvpp[i]:setParam {
                            param   =   Q_LVPP_CMP_VARIABLE_FRAME_RATE, 
                            value   =   1
                     }
    lvpp[i]:setParam {
                            param   =   Q_LVPP_CMP_NTP_CLOCK_ENABLE, 
                            value   =   1
                     }
  end
  
  if (BURNIN_TIMECODE ~= 0) and (BURNIN_AT_LVPP == 1) then
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_ENABLE, 
                              value   =   1
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_FONT_ADDR, 
                              value   =   BurnInFontAddr
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_FONT_SIZE, 
                              value   =   BURNIN_FONTSIZE
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_UPD_MODE, 
                              value   =   0
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_X, 
                              value   =   80
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_Y, 
                              value   =   32
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_LVPP_CMP_BURNIN_STR_LEN, 
                              value   =   8
                  }
      lvpp[i]:setBurnInText
                  {
                              index   =   0, text="CAMERA ".. i
                  }
      
      lvpp[i]:activateCfg()
  
      lvpp[i]:setBurnInParam
                  {
                              index   =   1, 
                              param   =   Q_LVPP_CMP_BURNIN_FONT_ADDR, 
                              value   =   BurnInFontAddr
                  }
      lvpp[i]:setBurnInParam
                  {
                              index   =   1, 
                              param   =   Q_LVPP_CMP_BURNIN_FONT_SIZE, 
                              value   =   BURNIN_FONTSIZE
                  }
      lvpp[i]:setBurnInAutoTime
                  {
                              index   =   1, 
                              enable  =   1, 
                              fps     =   BURNIN_FPS, 
                              x       =   80, 
                              y       =   64, 
                              droptime=   0
                  }
  end          


  lvpp[i]:activateCfg()
end


for i = MAX_LVPP_USED + 1,VININTERLEAVE do
  lvpp[i] = sysctl0:lvppCreate
            {
                name            =   "lvpp" .. i, 
                slot            =   i-1, 
                port            =   0, 
                maxWidth        =   64, 
                maxHeight       =   64, 
                maxPMEDSFactor  =   0, 
                numVidFrame     =   4,
                burninEnable    =   0
            }
            
  Setup_VPP(lvpp[i], 64, 64, MAIN_LVPP_INTERLACED, 0, 4, 4)
  
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_FIELD_OFFSET,
            value   =   VINFIELDOFFSET
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_START_HPIXEL,
            value   =   0
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_START_VLINE,
            value   =   VINSTARTVLINE
            }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_STC_PER_FRAME,
            value   =   MAIN_STC_PER_FRAME
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_STC_PER_OUTPUT_FRAME,
            value   =   MAIN_STC_PER_OUTPUT_FRAME
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_PMEDS_EN,
            value   =   0
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_PIXEL_AR_X,
            value   =   VIN_PIXEL_AR_X
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_PIXEL_AR_Y,
            value   =   VIN_PIXEL_AR_Y
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTY,
            value   =   1
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_HW_BYPASS_TFILTC,
            value   =   1
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_HW_BYPASS_ALL_STAT,
            value   =   1
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_HW_BYPASS_MB_STAT,
            value   =   1
        }
  lvpp[i]:setParam
        {
            param   =   Q_LVPP_CMP_INPUT_DECIMATION,
            value   =   MAIN_INPUT_DECIMATION
        }

  lvpp[i]:activateCfg()
end

-- Trick used later on to create NVPPs as if they were created from distinct LVPP
-- or to start the component
for i = VININTERLEAVE + 1,ENCCH do
  lvpp[i] = lvpp[1];
  nvpp[i] = nvpp[1];
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create and configure the Write Streamers
--
ws = sysctl0:createWriteStreamer
                {
                    name        =   "ws0",
                    uma         =   WRITE_STREAMER_UMA,
                    sendfile    =   WRITE_STREAMER_SENDFILE,
                    queuesize   =   WS_QUEUE_SIZE
                }
                
ws:showrate     {   enable  = SHOWRATE}

ws:configure    {
                    param       =   Q_WS_CFG_READ_BITSTREAMS, 
                    value       =   1
                }
ws:configure    {
                    param       =   Q_WS_CFG_MULTISTREAM_GATHER_SUPPORT,
                    value       =   MULTISTREAM_GATHER_ENABLE
                }
                
if (ENABLE_ANALYTICS_STREAMING > 0) then
  print("Enable metadata....");
  ws:configure  {
                    param       =   Q_WS_CFG_METADATA, 
                    value       =   1
                }
end                                
ws:start()

if (MULTISTREAM_GATHER_ENABLE == 1) then
    sysctl0:configure 
                {
                    param       =   Q_SYS_CFG_MULTISTREAM_GATHER_BUFFER_SIZE,
                    value       =   MULTISTREAM_GATHER_BUFFERSIZE
                }
end

videoStreamId = {}
RTP_CH = 0

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create and configure the primary channel encoders

for i = 1,ENCCH do

  audioStreamId = 2*(i-1) + 1
  videoStreamId = 2*(i-1) + 2

  avcenc[i] = sysctl0:avcencCreate
                {
                    name                =   "avcenc" .. videoStreamId, 
                    ws                  =   ws, 
                    maxwidth            =   ENCW, 
                    maxheight           =   ENCH, 
                    numreferenceframes  =   NUM_AVC_REFERENCES, 
                    outputbuffersize    =   MAIN_ENC_BUFFER_SIZE,
                    usebitstreambuffermem = HSBP_OUTPUT,
                    objectTrackingEnable  = OBJECT_TRACKING,
                 }
  chinfo.mainenc = chinfo.mainenc or {}
  chinfo.mainenc[i] = {}
  chinfo.mainenc[i].MAIN_GOP_STRUCTURE             = MAIN_GOP_STRUCTURE
  chinfo.mainenc[i].MAIN_FRAME_STRUCTURE           = MAIN_FRAME_STRUCTURE
  chinfo.mainenc[i].BITRATE                        = BITRATE[i]               

  if MAIN_GOP_STRUCTURE == "IBBRBP" then
    avcenc_480i(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP, MAIN_FRAME_STRUCTURE)
  else
    avcenc_480i(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, MAIN_FRAME_STRUCTURE)
  end
  
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, 
                            value   =   MAIN_STC_PER_OUTPUT_FRAME
                }
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_MIN_FRAME_DURATION,
                            value   =   MAIN_ENCODER_FRAME_RATE
                }                
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_BITSTREAM_OUTPUT, 
                            value   =   BITSTREAM_OUTPUT
                }
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_CLOSED_GOP, 
                            value   =   MAIN_CLOSED_GOP
                }
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_NEW_SPS_EACH_IDR, 
                            value   =   1
                }
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_SEI_PICT_TIMING_ENABLE,
                            value   =   LIVEMEDIA_WORKAROUND
                }
                
  if (PREVIEW == 2) then
  
    avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_ENCODER_PREVIEW, 
                            value   =   1
                }
  else 
  
    avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_ENCODER_PREVIEW, 
                            value   =   0
                }
                
  end
  
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_PME_ENABLE, 
                            value   =   MAIN_PME_ENABLE
                }
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, 
                            value   =   MAIN_ENC_GUARD_BAND
                }

  -- We need to set this to 0 for compatibility with VLC 0.8.6h
  avcenc[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_IP_FIELD_CODING, 
                            value   =   MAIN_IP_FIELD_CODING
                }

  avcenc[i]:setTransferSize
                {
                            size    =   MAIN_AVCENC_XFER_SIZE
                }

  -------------------------------
  -- Burn in name and timecode --
  -------------------------------
  
  chinfo.mainenc[i].BURNIN_TIMECODE                        = BURNIN_TIMECODE 
  
  if (BURNIN_TIMECODE ~= 0) and (BURNIN_AT_LVPP == 0) then
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_ENABLE, 
                              value   =   1
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_FONT_ADDR, 
                              value   =   BurnInFontAddr
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_FONT_SIZE, 
                              value   =   BURNIN_FONTSIZE
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_UPD_MODE, 
                              value   =   0
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_X, 
                              value   =   80
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_Y, 
                              value   =   32
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   0, 
                              param   =   Q_AVCENC_CMP_BURNIN_STR_LEN, 
                              value   =   8
                  }
      avcenc[i]:setBurnInText
                  {
                              index   =   0, text="CAMERA ".. i
                  }
                
      avcenc[i]:activateVideoEncCfg()
  
      avcenc[i]:setBurnInParam
                  {
                              index   =   1, 
                              param   =   Q_AVCENC_CMP_BURNIN_FONT_ADDR, 
                              value   =   BurnInFontAddr
                  }
      avcenc[i]:setBurnInParam
                  {
                              index   =   1, 
                              param   =   Q_AVCENC_CMP_BURNIN_FONT_SIZE, 
                              value   =   BURNIN_FONTSIZE
                  }
      avcenc[i]:setBurnInAutoTime
                  {
                              index   =   1, 
                              enable  =   1, 
                              fps     =   BURNIN_FPS, 
                              x       =   80, 
                              y       =   64, 
                              droptime=   0
                  }
  end 

  if ((OBJECT_TRACKING == 1) ) then
    io.write(string.format("ENABLING TRACKING FOR CODEC NUMBER\n",i))
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                          value=1});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_FACTOR,              value=4});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_OFFSET,              value=(i-1)});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,               value=12});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,           value=10});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,           value=15});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,  value=256});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,  value=0});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,               value=9});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                      value=30});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                      value=30});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,            value=1});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,            value=0});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,    value=1});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,                value=9});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,         value=600});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,          value=255});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,                  value=20});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,           value=2});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,       value=0});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,              value=0});
    avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,            value=0});
  end

  if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
     avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NTP_ENABLE, value=1});
  end

  avcenc[i]:activateVideoEncCfg()

  -----------------------------------------------------------------------------
  -- This works for multi-channel or single channel input as lvpp[i] is set to 
  -- lvpp[1] if single channel.
  if MAIN_DEINT_MODE == 0 then
    avcenc[i]:bindVideoInput{ src     =   lvpp[i] }
  else
    avcenc[i]:bindVideoInput{ src     =   nvpp[i] }
  end
  
  if i <= AUDIO_NUM_CHANNELS then
  
    -- create and bind audio encoder
    audenc[i] = sysctl0:audencCreate 
                {
                    name        =   "audenc"..audioStreamId, 
                    ws          =   ws, 
                    codecType   =   AUDIO_CODECS[AUDIO_CODEC],
                    outputbuffersize = MAIN_AUD_ENC_BUFFER_SIZE,
                    usebitstreambuffermem = HSBP_OUTPUT
                }
    
    audenc[i]:setStreamID
                {
                    streamid = audioStreamId
                }

    audenc[i]:bindAudioInput
                {
                    src=asplit[math.ceil(i/2)], 
                    port = (i+1)%2 
                }

    -- configure audio encoder
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_NUM_CHANNELS, 
                    value   =   Q_AUDENC_CFP_CHANNEL_MONO
                }
            
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_SAMPLE_RATE, 
                    value   =   AUDIO_SAMPLE_RATE
                }
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_SAMPLE_SIZE, 
                    value   =   AUDIO_SAMPLE_SIZE
                }
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_BIT_RATE, 
                    value   =   AUDIO_BIT_RATE
                }
    --codec specific configuration
    
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_Q711_LAW, 
                    value   =   Q_AUDENC_CFP_Q711_U_LAW
                }
    audenc[i]:setParam
                {
                    param   =   Q_AUDENC_CMP_QAC_AUDIO_OBJECT_TYPE, 
                    value   =   AUDIO_AOT[AUDIO_CODEC]
                }
    
    audenc[i]:activateCfg()
    
    audenc[i]:setAckInterval  {   interval    =   8    }
    
  end

  if i <= ENCCH_TO_STREAM then
  
    if (STREAMING_ENCCH == 1) then
    
      chinfo.mainenc[i].streaming = {}
      chinfo.mainenc[i].streaming.type = 'udp'
      chinfo.mainenc[i].streaming.host = HOST
      chinfo.mainenc[i].streaming.port = tostring(UDP_PORT_ENCCH + i - 1)
      ws:open
                {
                        type        =   "net", 
                        file        =   string.format(
                                            "host=%s,port=%d,protocol=udp,velem=1,max=%d", 
                                            HOST, UDP_PORT_ENCCH + i - 1,MAXPACKETSIZE
                                        ),
                        sid         = {audioStreamId, videoStreamId}
                }
                
    elseif (STREAMING_ENCCH == 2) then
    
      chinfo.mainenc[i].streaming = {}
      chinfo.mainenc[i].streaming.type      = 'rtp'
      chinfo.mainenc[i].streaming.session   = "stream" .. tostring(RTP_CH)
      chinfo.mainenc[i].streaming.port      = RTP_PORT

      if i <= AUDIO_NUM_CHANNELS then

            ws:open
                    {
                            type        =   "rtp",
                            file        =   string.format(
                                                "sip=%s,prt=%d,ses=stream%d%s,vid=h264,aud=%s,mod=0,msz=%d,aim=%d,rtcp=%d,sri=%d,sto=%d", 
                                                SERVER, RTP_PORT, RTP_CH, SESSION_SUFFIX, AUDIO_RTP_TYPE[AUDIO_CODEC], MAXPACKETSIZE, ENABLE_ANALYTICS_STREAMING, RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT
                                            ),
                            flags = Q_WS_CFP_FLAG_ENABLE_METADATA,                                                    
                            sid         =   {audioStreamId, videoStreamId}
                    }
     
      else
            ws:open
                    {
                            type        =   "rtp",
                            file        =   string.format(
                                                "sip=%s,prt=%d,ses=stream%d%s,vid=h264,mod=0,msz=%d,aim=%d,rtcp=%d,sri=%d,sto=%d", 
                                                SERVER, RTP_PORT, RTP_CH, SESSION_SUFFIX, MAXPACKETSIZE, ENABLE_ANALYTICS_STREAMING, RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT
                                            ),
                            flags = Q_WS_CFP_FLAG_ENABLE_METADATA,                                            
                            sid         =   {videoStreamId}
                    }
     
      end
                 
    RTP_CH = RTP_CH + 1
    
    elseif (STREAMING_ENCCH == 3) then
    
      chinfo.mainenc[i].streaming = {}
      chinfo.mainenc[i].streaming.type = 'qboxudp'
      chinfo.mainenc[i].streaming.host = HOST
      chinfo.mainenc[i].streaming.port = tostring(UDP_PORT_ENCCH + i - 1)

      ws:open
                {
                        type        =   "qboxudp",
                        file        =   string.format(
                                            "host=%s,port=%d,max=%d",
                                            HOST, UDP_PORT_ENCCH + i - 1, MAXPACKETSIZE
                                        ),
                        sid         =   {audioStreamId, videoStreamId}
                }

    end

  end

  if (i <= ENCCH_TO_STORE) then
    print("Opening file ", MainOutputFileName[i], "video stream ID", videoStreamId)
    
    chinfo.mainenc[i].storage = {}
    chinfo.mainenc[i].storage.type        = 'file'
    chinfo.mainenc[i].storage.file      = MainOutputFileName[i]
    chinfo.mainenc[i].storage.vsid      = videoStreamId 
    
    ws:open
            {
                    type        =   "file",
                    file        =   MainOutputFileName[i], 
                    sid         =   {audioStreamId, videoStreamId}
            }
  end

  avcenc[i]:setStreamID     {   streamid    =   videoStreamId   }
  avcenc[i]:setAckInterval  {   interval    =   8               }
  
end


------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create and configure the secondary channel encoders
--
for i = 1, ENCCH_2 do

  -- delta of 16 between live and derived channels
  audioStreamId = 2*(i-1) + 1
  videoStreamId = 2*(i-1) + 16
    
  chinfo.secenc = chinfo.secenc or {}
  chinfo.secenc[i] = {}
  chinfo.secenc[i].SECONDARY_GOP_STRUCTURE             = SECONDARY_GOP_STRUCTURE
  chinfo.secenc[i].SECONDARY_FRAME_STRUCTURE           = SECONDARY_FRAME_STRUCTURE
  chinfo.secenc[i].BITRATE                             = BITRATE_2[i] 

  avcenc2[i] = sysctl0:avcencCreate
                {
                        name                =   "avcenc" .. videoStreamId, 
                        ws                  =   ws, 
                        maxwidth            =   ENCW_2, 
                        maxheight           =   ENCH_2, 
                        numreferenceframes  =   SECONDARY_AVC_REFERENCES, 
                        outputbuffersize    =   SECONDARY_ENC_BUFFER_SIZE,
                        usebitstreambuffermem = HSBP_OUTPUT
                }

  if SECONDARY_GOP_STRUCTURE == "IBBRBP" then
    avcenc_480i(avcenc2[i], "high", 31, BITRATE_2[i], Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP, SECONDARY_FRAME_STRUCTURE)
  else
    avcenc_480i(avcenc2[i], "high", 31, BITRATE_2[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, SECONDARY_FRAME_STRUCTURE)
  end
  
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, 
                            value   =   SECONDARY_STC_PER_OUTPUT_FRAME
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_MIN_FRAME_DURATION,
                            value   =   SECONDARY_ENCODER_FRAME_RATE
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_BITSTREAM_OUTPUT, 
                            value   =   BITSTREAM_OUTPUT
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_CLOSED_GOP, 
                            value   =   SECONDARY_CLOSED_GOP
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_NEW_SPS_EACH_IDR, 
                            value   =   1
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_PME_ENABLE, 
                            value   =   SECONDARY_PME_ENABLE
                }
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_AU_GUARD_BAND_PER_MB, 
                            value   =   SECONDARY_ENC_GUARD_BAND
                }

  -- We need to set this to 0 for compatibility with VLC 0.8.6h
  avcenc2[i]:setVideoEncParam
                {
                            param   =   Q_AVCENC_CMP_IP_FIELD_CODING, 
                            value   =   SECONDARY_IP_FIELD_CODING
                }

  avcenc2[i]:setTransferSize {   size    =   SECONDARY_AVCENC_XFER_SIZE  }

  if (VARIABLE_FRAME_RATE_WITH_NTP == 1) then
     avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_NTP_ENABLE, value=1});
  end

  avcenc2[i]:activateVideoEncCfg()

  if (SECONDARY_NVPP_BYPASS == 0) then
  
    nvppenc2[i] = sysctl0:nvppCreate
                {
                            name            =   "nvpp" .. videoStreamId, 
                            maxWidth        =   ENCW_2, 
                            maxHeight       =   ENCH_2, 
                            maxPMEDSFactor  =   SECONDARY_PME_W, 
                            numVidFrame     =   SECONDARY_NVPP_FRAMES
                }

    Setup_VPP(nvppenc2[i], ENCW, ENCH, SECONDARY_NVPP_INTERLACED, 0, SECONDARY_PME_W, SECONDARY_PME_H)
    
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PICT_WIDTH,
                            value   =   ENCW_2
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PICT_HEIGHT,
                            value   =   ENCH_2
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PIXEL_AR_X,
                            value   =   VIN_PIXEL_AR_X
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PIXEL_AR_Y,
                            value   =   VIN_PIXEL_AR_Y 
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PIXEL_AR_FIXED,
                            value   =   1
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_STC_PER_FRAME,
                            value   =   SECONDARY_STC_PER_FRAME
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_STC_PER_OUTPUT_FRAME,
                            value   =   SECONDARY_STC_PER_OUTPUT_FRAME
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_PMEDS_EN,
                            value   =   SECONDARY_PME_ENABLE
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_TFILTY,
                            value   =   SECONDARY_TF_Y_BYPASS
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_TFILTC,
                            value   =   1
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_HW_BYPASS_MB_STAT,
                            value   =   0
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_NVPP_CMP_SCALE_FIELD_MODE,
                            value   =   SECONDARY_SINGLE_FIELD_DOWNSCALE
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_LVPP_CMP_PMEDS_FRAME_MODE,
                            value   =   SECONDARY_PME_FRAME_MODE
                }
    nvppenc2[i]:setParam
                {
                            param   =   Q_LVPP_CMP_INPUT_DECIMATION,
                            value   =   SECONDARY_INPUT_DECIMATION
                }
                
    if VARIABLE_FRAME_RATE_WITH_NTP == 1 then
      nvppenc2[i]:setParam 
                {
                            param   =   Q_NVPP_CMP_PTS_PASSTHROUGH, 
                            value   =   1
                }
    end
                
    nvppenc2[i]:activateCfg()
    
    -- Allow for more derived streams than there are inputs, for performance testing
    nvppenc2[i]:bindInput {src=lvpp[((i-1)%MAX_LVPP_USED)+1]}

    avcenc2[i]:bindVideoInput{ src =   nvppenc2[i] }
    
  else
    avcenc2[i]:bindVideoInput{ src =   lvpp[i]     }
  end

  if i <= ENCCH_2_TO_STREAM then

    if (STREAMING_ENCCH_2 == 1) then

      chinfo.secenc[i].streaming = {}
      chinfo.secenc[i].streaming.type      = 'net'
      chinfo.secenc[i].streaming.host      = HOST
      chinfo.secenc[i].streaming.port      = tostring(UDP_PORT_ENCCH_2 + i - 1)

      ws:open
            {
                type    =   "net", 
                file    =   string.format(
                                            "host=%s,port=%d,protocol=udp,velem=1,max=%d", 
                                            HOST, UDP_PORT_ENCCH_2 + i - 1,MAXPACKETSIZE
                                         ),
                sid     =   {videoStreamId}
            }

    elseif (STREAMING_ENCCH_2 == 2) then
    
      chinfo.secenc[i].streaming = {}
      chinfo.secenc[i].streaming.type      = 'rtp'
      chinfo.secenc[i].streaming.session   = "stream" .. tostring(RTP_CH)
      chinfo.secenc[i].streaming.port      = RTP_PORT
      
      if i <= AUDIO_NUM_CHANNELS then
            ws:open
                    {
                            type        =   "rtp",
                            file        =   string.format(
                                                "sip=%s,prt=%d,ses=stream%d%s,vid=h264,aud=%s,mod=0,msz=%d,aim=%d,rtcp=%d,sri=%d,sto=%d", 
                                                SERVER, RTP_PORT, RTP_CH, SESSION_SUFFIX, AUDIO_RTP_TYPE[AUDIO_CODEC], MAXPACKETSIZE,
                                                ENABLE_ANALYTICS_STREAMING, RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT),
                            flags       =   Q_WS_CFP_FLAG_ENABLE_METADATA,                                                    
                            sid         =   {audioStreamId, videoStreamId}
                    }
     
      else
            ws:open
                    {
                            type        =   "rtp",
                            file        =   string.format(
                                                "sip=%s,prt=%d,ses=stream%d%s,vid=h264,mod=0,msz=%d,aim=%d,rtcp=%d,sri=%d,sto=%d", 
                                                SERVER, RTP_PORT, RTP_CH, SESSION_SUFFIX, MAXPACKETSIZE, ENABLE_ANALYTICS_STREAMING, RTCP_FLAG, RTCP_SR_INTERVAL, SESSION_TIMEOUT),
                            flags       =   Q_WS_CFP_FLAG_ENABLE_METADATA,                                            
                            sid         =   {videoStreamId}
                    }
     
      end

      RTP_CH = RTP_CH + 1

    elseif (STREAMING_ENCCH_2 == 3) then
    
      chinfo.mainenc[i].streaming = {}
      chinfo.mainenc[i].streaming.type = 'qboxudp'
      chinfo.mainenc[i].streaming.host = HOST
      chinfo.mainenc[i].streaming.port = tostring(UDP_PORT_ENCCH_2 + i - 1)

      ws:open
                {
                        type        =   "qboxudp",
                        file        =   string.format(
                                            "host=%s,port=%d,max=%d",
                                            HOST, UDP_PORT_ENCCH_2 + i - 1, MAXPACKETSIZE
                                        ),
                        sid         =   {audioStreamId, videoStreamId}
                }

    end

  end

  avcenc2[i]:setStreamID    {   streamid    =   videoStreamId   }
  avcenc2[i]:setAckInterval {   interval    =   8               }

  if (i <= ENCCH_2_TO_STORE) then
  
        chinfo.secenc[i].storage = {}
        chinfo.secenc[i].storage.type      = 'file'
        chinfo.secenc[i].storage.file   = "stream" .. SecOutputFileName[i]
        chinfo.secenc[i].storage.vsid      = videoStreamId
  
          ws:open
            {
                type    =   "file", 
                file    =   SecOutputFileName[i], 
                sid = {videoStreamId}
            }
  end
  
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- MJPEG Setup and configuration
--

for i = 1, MJPEGCH do

  chinfo.mjpeg      = chinfo.mjpeg or {}
  chinfo.mjpeg[i]   = {} 
  -- delta of 32 between live and MJPEG
  videoStreamId = i-1 + 32

  jpegenc[i] = sysctl0:jpegencCreate
                {
                    name            =   "jpegenc"..videoStreamId, 
                    ws              =   ws, 
                    maxwidth        =   ENCW, 
                    maxheight       =  ENCH, 
                    outputbuffersize=  MJPEG_ENC_BUFFER_SIZE,
                    usebitstreambuffermem = HSBP_OUTPUT
                }              
  
  if MAIN_DEINT_MODE == 0 then
    jpegenc[i]:bindVideoInput{ src     =   lvpp[i] }
  else
    jpegenc[i]:bindVideoInput{ src     =   nvpp[i] }
  end
  
  jpegenc[i]:setParam
                {
                            param   =   Q_JPEGENC_CMP_CAPTURE_INTERVAL, 
                            value   =   JPEG_CAPTURE_INTERVAL   
                }
  jpegenc[i]:setParam
                {
                            param   =   Q_JPEGENC_CMP_CAPTURE_START_INDEX, 
                            value   =   0
                }
  jpegenc[i]:setParam
                {
                            param   =   Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, 
                            value   =   -1
                }
  jpegenc[i]:setParam
                {
                            param   =   Q_JPEGENC_CMP_BITSTREAM_BUFFER, 
                            value   =   BITSTREAM_OUTPUT
                }
  jpegenc[i]:setParam
                {
                            param   =   Q_JPEGENC_CMP_AU_GUARD_BAND_PER_MB,
                            value   =   JPEG_ENC_GUARD_BAND
                }
  
  jpegenc[i]:activateCfg()

  jpegenc[i]:setAckInterval     {   interval        =   8               }
  jpegenc[i]:setStreamID        {   streamid        =   videoStreamId   }
  jpegenc[i]:setQualityFactor   {   qualityfactor   =   30              }
  

  jpegenc[i]:bitrateControl     
                {
                            bitrate     =   BITRATE_MJPEG[i],
                            fps         =   JPEG_FPS,
                            behavior    =   0
                }
  chinfo.mjpeg[i].BIT_RATE = BITRATE_MJPEG[i]
  chinfo.mjpeg[i].FPS      = JPEG_FPS

  if i <= MJPEGCH_TO_STREAM then
  
    if (STREAMING_MJPEGCH == 1) then
    
      chinfo.mjpeg[i].streaming = {}
      chinfo.mjpeg[i].streaming.type      = 'net'
      chinfo.mjpeg[i].streaming.host      = HOST
      chinfo.mjpeg[i].streaming.port      = UDP_PORT_MJPEGCH+i-1 
    
      ws:open
                {
                        type    =   "net", 
                        file    =   string.format(
                                                "host=%s,port=%d,protocol=udp,max=%d,jelem=%d", 
                                                HOST, UDP_PORT_MJPEGCH+i-1,MAXPACKETSIZE,1
                                                ), 
                        sid     = { videoStreamId   }
                }
    elseif (STREAMING_MJPEGCH == 2) then

      chinfo.mjpeg[i].streaming = {}
      chinfo.mjpeg[i].streaming.type      = 'rtp'
      chinfo.mjpeg[i].streaming.session   = "stream" .. tostring(RTP_CH)
      chinfo.mjpeg[i].streaming.port      = RTP_PORT 
    
      ws:open
                {
                        type    =   "rtp", 
                        file    =   string.format(
                                                "sip=%s,prt=%d,ses=stream%d,vid=mjpeg,mod=0,msz=%d", 
                                                SERVER, RTP_PORT, RTP_CH, MAXPACKETSIZE
                                                ), 
                        sid     = { videoStreamId   }
                }
      RTP_CH = RTP_CH + 1
      
    end
    
  end

  if (i <= MJPEGCH_TO_STORE) then

       
    chinfo.mjpeg[i].storage = {}
    chinfo.mjpeg[i].storage.type        = 'file'
    chinfo.mjpeg[i].storage.file      = JpgOutputFileName[i]
    chinfo.mjpeg[i].storage.vsid      = videoStreamId 

    ws:open
                {
                        type    =   "file", 
                        file    =   JpgOutputFileName[i], 
                        sid     = { videoStreamId   }
                }

  end

end

-- For now, just assume one jpeg encoder for taking snapshot
if (SNAPSHOTCH == 1) then
  videoStreamId = 36
   
  snapshotenc = sysctl0:jpegencCreate
                        {
                                name        =   "snapshotenc", 
                                ws          =   ws, 
                                maxwidth    =   ENCW, 
                                maxheight   =   ENCH, 
                                outputbuffersize = SNAPSHOT_ENC_BUFFER_SIZE,
                                usebitstreambuffermem = HSBP_OUTPUT
                        }
    
  snapshotenc:bindVideoInput{ src = lvpp[1] }
  snapshotenc:setParam  {
                                param   =   Q_JPEGENC_CMP_CAPTURE_INTERVAL, 
                                value   =   -1
                        }
  snapshotenc:setParam  {
                                param   =   Q_JPEGENC_CMP_CAPTURE_START_INDEX, 
                                value   =   0
                        }
    
  -- set to 0 to make it sit idle and wait for snapshot commands
  snapshotenc:setParam  {
                                param   =   Q_JPEGENC_CMP_CAPTURE_MAX_ENCODED_FRAMES, 
                                value   =   0
                        }
  snapshotenc:setParam  {
                                param   =   Q_JPEGENC_CMP_BITSTREAM_BUFFER, 
                                value   =   1
                        }
  snapshotenc:setParam  {
                                param   =   Q_JPEGENC_CMP_AU_GUARD_BAND_PER_MB,
                                value   =   300
                        }
   snapshotenc:activateCfg()

   snapshotenc:setAckInterval { interval = 1 }
   snapshotenc:setStreamID{streamid=videoStreamId}
   snapshotenc:setQualityFactor{qualityfactor=SNAPSHOT_QUALITY}
   
   -- jelem=1 to strip away qbox headers so that encoded frames can be viewed by ordinary jpeg viewers
   outFileStr = string.format("basefilename=%s,single=1,jelem=1", ShotOutputFileName);
   
   ws:open
                        {
                                type    =   "jpeg", 
                                file    =   outFileStr, 
                                sid     =   {videoStreamId}
                        }
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Configure recon preview Display
--
if (PREVIEW == 2) then

  x = 0
  y = 0

  -- create one avsn for each encoder, either bound to live or encoder
  for i = 1,ENCCH do
    avsnenc[i] = sysctl0:avsnCreate{name="avsnenc"..i}
    
    evch[i] = avoc0:addVideoChannel()

    avsnenc[i]:bindVideoInput{src=avcenc[i]}
    
    avsnenc[i]:setParam
                {
                            param   =   Q_AVSN_CMP_EXTERNAL_STC_OFFSET, 
                            value   =   AVC_RECON_PREVIEW_TIME_DELAY
                }
    avsnenc[i]:activateCfg()

    avoc0:setVideoChannelSource{    ch  =   evch[i], src    =   avsnenc[i]  }
    
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   ENCW
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   ENCH
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, 
                            value   =   0
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, 
                            value   =   0
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, 
                            value   =   OUTW
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, 
                            value   =   OUTH
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x
                }
    avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y
                }

    -- optionally set vertical decimation by field drop but only in
    -- svideo mode!
    if ((FORMAT == "480i") and (DISPLAY_USING_ONE_FIELD == 1)) then
    
      avoc0:setVideoChannelParam
                {           
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SCALE_FIELD_MODE, 
                            value   =   1
                }
    
    end
    
    -- remember that encoders are all enabled on the display
    EncRenderDisable[i] = 0

    -- Remember x, y
    EncChannelX[i] = x
    EncChannelY[i] = y

    x = x + OUTW
    if x == COMPWIDTH then
      x = 0
      y = y + OUTH
    end
  end
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Configure Live preview Display
--

x = 0
y = 0

-- We create avsn for live display no matter what has been requested as we can switch
-- on the fly
for i = 1,MAX_LVPP_USED do

  avsnlive[i] = sysctl0:avsnCreate{ name    =   "avsnlive".. i  }
  
  -- Do not set the STC offset for audio+video channels
  if i <= AUDIO_NUM_CHANNELS then
    avsnlive[i]:setParam
            {   
                param   =   Q_AVSN_CMP_EXTERNAL_STC_OFFSET, 
                value   =   0
            }
  else
    avsnlive[i]:setParam
            {   
                param   =   Q_AVSN_CMP_EXTERNAL_STC_OFFSET, 
                value   =   LVPP_LIVE_PREVIEW_TIME_DELAY
            }
  end
  avsnlive[i]:activateCfg()
  
  lvch[i] = avoc0:addVideoChannel()

  -----------------------------------------------------------------------------
  -- This works for multi-channel or single channel input as lvpp[i] is set to 
  -- lvpp[1] if single channel.
  avsnlive[i]:bindVideoInput{   src =   lvpp[i]}

  avoc0:setVideoChannelSource
                {           
                            ch      =   lvch[i], 
                            src     =   avsnlive[i]
                }
  avoc0:setVideoChannelParam
                {           
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   ENCW
                }
  avoc0:setVideoChannelParam
                {           
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   ENCH
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, 
                            value   =   0
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, 
                            value   =   0
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, 
                            value   =   OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, 
                            value   =   OUTH
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y
                }

  -- optionally set vertical decimation by field drop but only in
  -- svideo mode!
  if ((FORMAT == "480i") and (DISPLAY_USING_ONE_FIELD == 1)) then
  
      avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SCALE_FIELD_MODE, 
                            value   =   1
                }
  
  end
  
  if (i <= AUDIO_NUM_CHANNELS) then
    asplit_id = math.ceil(i/2)
    port_id = (i+1)%2
    avsnlive[i]:bindAudioInput
                { 
                            src     =   asplit[asplit_id],
                            port    =   port_id
                }
    lach[i] = avoc0:addAudioChannel{channel=Q_AVOC_CFP_AUD_MONO_L}
    
    avoc0:setAudioChannelSource
                {
                            ch      =   lach[i],
                            src     =   0
                }
    avoc0:enableAudioChannel
                {
                            ch      =   lach[i],
                            enable  =   0
                }
  end

  -- Disable live display if not requested
  if (PREVIEW ~= 1) then

    avoc0:setVideoChannelSource
                {
                            ch      =   lvch[i],
                            src     =   0
                }

    -- remember that encoders are all disabled on the display
    LiveRenderDisable[i] = 1
    
  end
  
  -- Enables live audio ch 1 if live or recon preview is selected
  if ( (PREVIEW == 1) or (PREVIEW == 2) ) then
    if (i == 1) and (i <= AUDIO_NUM_CHANNELS) then
      avoc0:setAudioChannelSource
                {
                            ch      =   lach[i],
                            src     =   avsnlive[i]
                }
      avoc0:enableAudioChannel
                {
                            ch      =   lach[i],
                            enable  =   1
                }
      ActiveAudioChannel = lach[i]
    end
  end

  -- Remember x, y
  LiveChannelX[i] = x
  LiveChannelY[i] = y

  x = x + OUTW
  if x == COMPWIDTH then
    x = 0
    y = y + OUTH
  end
  
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create Decoders
--

sysctl0:configure({param=Q_SYS_CFG_AVC_DECODE_PRIORITY, value=DECODER_PRIORITY});

for i = 1,DECCH do

    local audioStreamId = 2*(i-1) + 1
    local videoStreamId = 2*(i-1) + 2

    avdec[i] = sysctl0:avdecCreate
                {
                    name            =   "avdec" .. videoStreamId, 
                    atype           =   AUDIO_CODECS[AUDIO_CODEC], 
                    vtype           =   VIDEO_CODEC, 
                    audioBufferSize =   DECODER_AUDIO_BUFFER_SIZE, 
                    videoBufferSize =   DECODER_VIDEO_BUFFER_SIZE, 
                    maxWidth        =   DECW, 
                    maxHeight       =   DECH, 
                    numoutputframes =   AVCDEC_REFERENCE_FRAMES, 
                    videoAUs        =   DECODER_VIDEO_BUFFER_AU_SIZE,
                    ioMode          =   MEMORY_XFER_MODE
                }

    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_AUDIO_STREAM_ID, 
                            value   =   audioStreamId
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_STREAM_ID, 
                            value   =   videoStreamId
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, 
                            value   =   AUDIO_SAMPLE_RATE
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, 
                            value   =   AUDIO_SAMPLE_SIZE
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, 
                            value   =   Q_AVDEC_CFP_AUDIO_CHANNEL_MONO
                }
    --[[
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, 
                            value   =   Q_AVDEC_CFP_TRUE
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, 
                            value   =   3003
                }
    --]]
    -- must set this to match decoder buffer size
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, 
                            value   =   750000
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_UNDERFLOW_AU_THRESHOLD, 
                            value   =   768
                }
    avdec[i]:setParam
                {
                            param   =   Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, 
                            value   =   10000
                }
    avdec[i]:activateCfg()
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create decoder display
--

x = 0
y = 0

for i = 1,DECCH do
  dvch[i] = avoc0:addVideoChannel()
  avsndec[i] = sysctl0:avsnCreate   {   name   =   "avsndec".. i }

  -- bind to the matching decoder
  avsndec[i]:bindVideoInput         {   src     =   avdec[i]    }
  
  avoc0:setVideoChannelSource
                {
                            ch      =   dvch[i], 
                            src     =   avsndec[i]
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   DECW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   DECH
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, 
                            value   =   0
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, 
                            value   =   0
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, 
                            value   =   OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, 
                            value   =   OUTH
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y
                }

  DecChannelX[i] = x
  DecChannelY[i] = y

  if VIDEO_PLAYBACK_ONLY == 0 then
    dach[i] = avoc0:addAudioChannel{channel=Q_AVOC_CFP_AUD_MONO_L}
    avsndec[i]:bindAudioInput         {   src     =   avdec[i]    }
    avoc0:setAudioChannelSource
                {
                            ch      =   dach[i],
                            src     =   0
                }
    avoc0:enableAudioChannel
                {
                            ch      =   dach[i],
                            enable  =   0
                }
  end
  
  -- Enables decoded audio ch 1 if decode preview is selected
  if (PREVIEW == 3) then
    if (i == 1) and (i <= AUDIO_NUM_CHANNELS) then
      avoc0:setAudioChannelSource
                {
                            ch      =   dach[i],
                            src     =   avsndec[i]
                }
      avoc0:enableAudioChannel
                {
                            ch      =   dach[i],
                            enable  =   1
                }
      ActiveAudioChannel = dach[i]
    end
  end
                
  if (PREVIEW ~= 3) then
    avoc0:setVideoChannelSource
                {
                            ch      =   dvch[i],
                            src     =   0
                }
                
    DecRenderDisable[i] = 1
  else
    DecRenderDisable[i] = 0
  end

  x = x + OUTW
  if x == COMPWIDTH then
    x = 0
    y = y + OUTH
  end
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create zoom display
--

for i = 1,ZOOMCH do
  zvch[i] = avoc0:addVideoChannel()

  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, 
                            value   =   ZOOMOUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, 
                            value   =   ZOOMOUTH
                }

  ZoomChannelX[i] = 0
  ZoomChannelY[i] = 0

  -- zoom is disabled by default
  ZoomRenderDisable[i] = 1
end

------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Create decoder read streamer
--

rs0 = sysctl0:createReadStreamer    {name="rs0"}
rs0:setParam
                {
                            param   =   Q_RS_CFG_SHOW_UPDATE, 
                            value   =   SHOWUPDATE
                }

------------------------------------------------------------------------------
------------------------------------------------------------------------------
--- Print Configuration information


------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- Helper Functions

function help()

    print(
    [[
    Info
    ----
        showconfig()            - show script configuration
        qmain(i)                - show query results for main encode channel
        qsec(i)                 - show query results for secondary encode channel
        qjpeg(i)                - show query results for MJPEG
        qdec(i)                 - show query results for the av decoder
    Display
    -------
        showlive(i)             - Starts rendering a live encode channel.
        showrecon(i)            - Starts rendering a reconstructed encode channel.
        showplayback(i)         - Starts rendering a decode channel.
        showzoom(i)             - Starts rendering a zoom channel.
        hidelive(i)             - Stops rendering a live encode channel.
        hiderecon(i)            - Stops rendering a reconstructed encode channel.
        hideplayback(i)         - Stops rendering decode channel.
        hidezoom(i)             - Stops rendering zoom channel.
        setlive(i, x, y)        - Positions live video channel on the display.
        setrecon(i, x, y)       - Positions reconstucter video channel on the display.
        setplayback(i, x, y)    - Positions a decode video channel on the display.
        setzoom(i, x, y)        - Positions zoom video channel on the display.
        
        zoomlive(i)             - Sets zoom channel to live.
        zoomrecon(i)            - Sets zoom channel to reconstructed.
        zoomplayback(i)         - Sets zoom channel to decode.
    Audio
    -----
        showliveaudio(i)        - Enables live audio channel i to output
        showdecodeaudio(i)      - Enables decoded audio channel i to output
        hideliveaudio(i)        - Disables live audio channel i to output
        hidedecodeaudio(i)      - Disables decoded audio channel i to output
    Record
    ------    
        startrecord(i)          - Starts recodering channel i to a file.
    Playback
    --------    
        startplayback(fch, dch) - Starts decoding channel fch associated file.
        stopplayback(fch)       - Stops rendering a live encode channel.
        pauseplayback(fch)      - Pause the decoder.
        resumeplayback(fch)     - Resumes the decoding.
        slowplayback(fch, n)    - Slow motion playback.
        stepplayback(fch)       - Single step.
    ]]
)
end

function showconfig()

    printf("SCRIPT CONFIGURATION\n")
    for k, v in pairs (chinfo.config) do
        printf("\t%-20s %s\n", k, tostring(v))
    end
    
    if chinfo.mainenc then
        for i, ifo in ipairs(chinfo.mainenc) do
            printf("MAIN CHANNEL %s\n", i)
            printtbl(ifo, "    ")
        end
    end    

    if chinfo.secenc then
        for i, ifo in ipairs(chinfo.secenc) do
            printf("SECONDARRY CHANNEL %s\n", i)
            printtbl(ifo, "    ")
        end
    end 

    if chinfo.mjpeg then
        for i, ifo in ipairs(chinfo.mjpeg) do
            printf("MJPEG CHANNEL %s\n", i)
            printtbl(ifo, "    ")
        end
    end 


end

------------------------------------------------------------------------------
--- Binds specific live audio channel input to output.
--
-- This function allows to connect the specified live audio input to the 
-- output (AVOC).
--
-- @param i Live audio channel to output
--
-- @return nil

function showliveaudio(i)
    if lach[i] == nil or avsnlive[i] == nil then
        print("WARNING: No audio channel " ..i.. " available")
        return
    end
    
    if ActiveAudioChannel == lach[i] then
        return
    end
    
    -- There is another active audio channel, disable it first
    if ActiveAudioChannel ~= nil then
      avoc0:setAudioChannelSource{ch=ActiveAudioChannel, src=0}
      avoc0:enableAudioChannel{ch=ActiveAudioChannel, enable=0}
      avoc0:activateAudioChannelCfg()
    end
    
    -- Set up live audio output
    avoc0:setAudioChannelSource{ch=lach[i], src=avsnlive[i]}
    avoc0:enableAudioChannel{ch=lach[i], enable=1}
    avoc0:activateAudioChannelCfg()
    
    ActiveAudioChannel = lach[i]
end

------------------------------------------------------------------------------
--- Binds specific decode audio channel input to output.
--
-- This function allows to connect the specified decoded audio to the 
-- output (AVOC).
--
-- @param i Decode audio channel to output
--
-- @return nil

function showdecodeaudio(i)
    if dach[i] == nil or avsndec[i] == nil then
        print("WARNING: No audio channel " ..i.. " available")
        return
    end
    
    if ActiveAudioChannel == dach[i] then
        return
    end
    
    -- There is another active audio channel, disable it first
    if ActiveAudioChannel ~= nil then
      avoc0:setAudioChannelSource{ch=ActiveAudioChannel, src=0}
      avoc0:enableAudioChannel{ch=ActiveAudioChannel, enable=0}
      avoc0:activateAudioChannelCfg()
    end

    -- Set up decoded audio output
    avoc0:setAudioChannelSource{ch=dach[i], src=avsndec[i]}
    avoc0:enableAudioChannel{ch=dach[i], enable=1}
    avoc0:activateAudioChannelCfg()
    
    ActiveAudioChannel = dach[i]
end

------------------------------------------------------------------------------
--- Unbinds specific live audio channel input to output.
--
-- @param i Live audio channel to output
--
-- @return nil

function hideliveaudio(i)
    if lach[i] == nil or avsnlive[i] == nil then
        print("WARNING: No audio channel " ..i.. " available")
        return
    end
    
    if ActiveAudioChannel ~= lach[i] then
        return
    end
    
    -- Set up live audio output
    avoc0:setAudioChannelSource{ch=lach[i], src=0}
    avoc0:enableAudioChannel{ch=lach[i], enable=0}
    avoc0:activateAudioChannelCfg()
    
    ActiveAudioChannel = nil
end

------------------------------------------------------------------------------
--- Unbinds specific decode audio channel input to output.
--
-- @param i Decode audio channel to output
--
-- @return nil

function hidedecodeaudio(i)
    if dach[i] == nil or avsndec[i] == nil then
        print("WARNING: No audio channel " ..i.. " available")
        return
    end
    
    if ActiveAudioChannel ~= dach[i] then
        return
    end

    -- Set up decoded audio output
    avoc0:setAudioChannelSource{ch=dach[i], src=0}
    avoc0:enableAudioChannel{ch=dach[i], enable=0}
    avoc0:activateAudioChannelCfg()
    
    ActiveAudioChannel = nil
end

------------------------------------------------------------------------------
--- Starts rendering a live encode channel.
--
-- This function starts rendering the video channel associated to the main
-- encode channel i.
--
-- @param i Live encode channel to display
--
-- @return nil
function showlive(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   lvch[i],
                            src     =   avsnlive[i]
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  LiveRenderDisable[i] = 0
end

------------------------------------------------------------------------------
--- Starts rendering a reconstructed encode channel.
--
--
-- This function starts rendering the video channel (recon) associated to the
-- main encode channel i.
--
-- @param i Reconstructed encode channel to display
--
-- @return nil
function showrecon(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   evch[i],
                            src     =   avsnenc[i]
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  EncRenderDisable[i] = 0
end

------------------------------------------------------------------------------
--- Starts rendering a decode channel.
--
--
-- This function starts rendering the video channel associated to the
-- decode channel i.
--
-- @param i decode channel to display
--
-- @return nil
function showplayback(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   dvch[i],
                            src     =   avsndec[i]
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  DecRenderDisable[i] = 0
end

------------------------------------------------------------------------------
--- Starts rendering a zoom channel.
--
--
-- This function starts rendering the video channel associated to the
-- zoom channel i.
--
-- @param i zoom channel to display
--
-- @return nil
function showzoom(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   zvch[i],
                            src     =   avsnzoom[i]
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  ZoomRenderDisable[i] = 0
end



------------------------------------------------------------------------------
--- Stops rendering a live encode channel.
--
-- This function stops rendering the video channel associated to the main
-- encode channel i.
--
-- @param i Live encode channel to hide
--
-- @return nil
function hidelive(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   lvch[i],
                            src     =   0
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  LiveRenderDisable[i] = 1
end

------------------------------------------------------------------------------
--- Stops rendering a reconstructed encode channel.
--
-- This function stops rendering the video channel associated to the main
-- encode channel i.
--
-- @param i Reconstructed encode channel to hide
--
-- @return nil
function hiderecon(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   evch[i],
                            src     =   0
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  EncRenderDisable[i] = 1
end

------------------------------------------------------------------------------
--- Stops rendering a decode channel.
--
-- This function stops rendering the video channel associated to the decode
-- channel i.
--
-- @param i decode channel to hide
--
-- @return nil
function hideplayback(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   dvch[i],
                            src     =   0
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  DecRenderDisable[i] = 1
end

------------------------------------------------------------------------------
--- Stops rendering a zoom channel.
--
-- This function stops rendering the video channel associated to the zoom
-- channel i.
--
-- @param i zoom channel to hide
--
-- @return nil
function hidezoom(i)
  avoc0:setVideoChannelSource
                {
                            ch      =   zvch[i],
                            src     =   0
                }
  avoc0:activateVideoChannelCfg()

  -- remember state
  ZoomRenderDisable[i] = 1
end

------------------------------------------------------------------------------
--- Positions live video channel on the display.
--
-- Positions the specified live video channel at the specified offset on the 
-- display. The offset is specified as a multiple of video channels. For 
-- instance, an offset of 0,0 positions the top corner of the video channel
-- at 0,0 on the display while an offset of 1,1 positions the video channel
-- on the same diagonal as 0,0 (4th quadrant on a 480i output)
--
-- @param i Live encode channel to position
-- @param x multiple of the video channel width
-- @param y multiple of the video channel height
--
-- @return nil
function setlivem(i, x, y)
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x*OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   lvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y*OUTH
                }

  LiveChannelX[i] = x
  LiveChannelY[i] = y
end
function setlive(i, x, y)
  setlivem(i, x, y)
  avoc0:activateVideoChannelCfg()
end


------------------------------------------------------------------------------
--- Positions reconstucter video channel on the display.
--
-- Positions the specified recon video channel at the specified offset on the 
-- display. The offset is specified as a multiple of video channels. For 
-- instance, an offset of 0,0 positions the top corner of the video channel
-- at 0,0 on the display while an offset of 1,1 positions the video channel
-- on the same diagonal as 0,0 (4th quadrant on a 480i output)
--
-- @param i Reconstructed encode channel to position
-- @param x multiple of the video channel width
-- @param y multiple of the video channel height
--
-- @return nil
function setrecon(i, x, y)

  avoc0:setVideoChannelParam
                {
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x*OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   evch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y*OUTH
                }
  avoc0:activateVideoChannelCfg()

  EncChannelX[i] = x
  EncChannelY[i] = y
  
end


------------------------------------------------------------------------------
--- Positions a decode video channel on the display.
--
-- Positions the specified decode video channel at the specified offset on the 
-- display. The offset is specified as a multiple of video channels. For 
-- instance, an offset of 0,0 positions the top corner of the video channel
-- at 0,0 on the display while an offset of 1,1 positions the video channel
-- on the same diagonal as 0,0 (4th quadrant on a 480i output)
--
-- @param i decode channel to position
-- @param x multiple of the video channel width
-- @param y multiple of the video channel height
--
-- @return nil
function setplaybackm(i, x, y)
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x*OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   dvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y*OUTH
                }

  DecChannelX[i] = x
  DecChannelY[i] = y
end
function setplayback(i, x, y)
  setplaybackm(i, x, y)
  avoc0:activateVideoChannelCfg()
end


------------------------------------------------------------------------------
--- Positions a zoom video channel on the display.
--
-- Positions the specified zoom video channel at the specified offset on the 
-- display. The offset is specified as a multiple of video channels. For 
-- instance, an offset of 0,0 positions the top corner of the video channel
-- at 0,0 on the display while an offset of 1,1 positions the video channel
-- on the same diagonal as 0,0 (4th quadrant on a 480i output)
--
-- @param i zoom channel to position
-- @param x multiple of the video channel width
-- @param y multiple of the video channel height
--
-- @return nil
function setzoomm(i, x, y)
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, 
                            value   =   x*OUTW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, 
                            value   =   y*OUTH
                }

  DecChannelX[i] = x
  DecChannelY[i] = y
end
function setzoom(i, x, y)
  setzoomm(i, x, y)
  avoc0:activateVideoChannelCfg()
end


------------------------------------------------------------------------------
--- Sets zoom video channel to live.
--
-- @param i live channel to set
--
-- @return nil
function zoomlive(i)
  avsnzoom[i] = avsnlive[i];

  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   ENCW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   ENCH
                }
end


------------------------------------------------------------------------------
--- Sets zoom video channel to recon.
--
-- @param i recon channel to set
--
-- @return nil
function zoomrecon(i)
  avsnzoom[i] = avsnenc[i];

  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   ENCW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   ENCH
                }
end


------------------------------------------------------------------------------
--- Sets zoom video channel to decode.
--
-- @param i decode channel to set
--
-- @return nil
function zoomplayback(i)
  avsnzoom[i] = avsndec[i];

  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, 
                            value   =   DECW
                }
  avoc0:setVideoChannelParam
                {
                            ch      =   zvch[i], 
                            param   =   Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, 
                            value   =   DECH
                }
end



------------------------------------------------------------------------------
---- Starts recording channel i to a file.
--
-- This function open a qbox bitstream writer which starts recording the 
-- bitstream to the main output file
--
-- @param i Bitstream writer channel
--
-- @return nil
function startrecord(i)

  local encAudioStreamId = 2*(i-1) + 1
  local encVideoStreamId = 2*(i-1) + 2
  
  printf("Recording to %s\n", MainOutputFileName[i])
  ws:open   
              {
                  type    =   "file", 
                  file    =   MainOutputFileName[i], 
                  sid     =   {encAudioStreamId, encVideoStreamId}
              }
end


------------------------------------------------------------------------------
---- Stops recording channel i.
--
-- This function close the bitstream writer to stop the current recording
--
-- @param i Bitstream writer channel 
--
-- @return nil
function stoprecord(i)
  printf("Stop recording channel %d\n", i)
  ws:close
             {
                 type   = "file",
                 file   = MainOutputFileName[i]
             }
end


------------------------------------------------------------------------------
--- Starts decoding channel i associted file.
--
-- Starts decoding the channel specified (from file).
--
-- @param fch file channel
-- @param dch decode channel (optional, default is channel 1)
--
-- @return nil
function startplayback(fch, dch)
  local srcsid = { nil, nil };
  local codecid = { nil, nil };
  
  if (dch == nil) then
    dch = 1;
  end
  
  if (FileToDecCh[fch] ~= nil) then
    printf("Error: Channel %d is already playing on display %d\n", fch, FileToDecCh[fch])
    return
  end
  
  if (DecChActive[dch] ~= nil) then
    printf("Error: Decode channel %d is used already\n", dch)
    return
  end

  --   Video stream ID   Audio stream ID
  if (SINGLE_CHANNEL_FILES == 0) then
    srcsid[1] = 2 + 2*(fch-1);
  else
    srcsid[1] = 2;
  end
  codecid[1] = 2 + 2*(fch-1);
  
  if VIDEO_PLAYBACK_ONLY == 0 then
    if (SINGLE_CHANNEL_FILES == 0) then
      srcsid[2] = 1 + 2*(fch-1);
    else
      srcsid[2] = 1;
    end
    codecid[2] = 1 + 2*(fch-1);
  end

  -- We open a bitstream reader and specify the source stream id's in the file and then the mapped id's
  -- that are sent to the codec.  The source id's are the same as the encoder, the codec id's are the one
  -- that were set when the avdecoder's were instantiated.
  
  if (VIDEO_PLAYBACK_ONLY == 0) then
    avdec[dch]:setParam {
                            param   =   Q_AVDEC_CMP_AUDIO_STREAM_ID, 
                            value   =   codecid[2]
                        }
  end
  avdec[dch]:setParam   {
                            param   =   Q_AVDEC_CMP_VIDEO_STREAM_ID, 
                            value   =   codecid[1]
                        }
  avdec[dch]:activateCfg()
  
  
  printf("Video stream ID [%s] Audio stream ID [%s]\n",
         tostring(srcsid[1]), tostring(srcsid[2]))
  
  rs0:bindDecoder   {   dec     =   avdec[dch]    }

  bsr[dch] = rs0:open {
                            type    =   READ_STREAMER_PROTOCOL,
                            file    =   MainInputFileName[fch],
                            srcId   =   srcsid,
                            codecId =   codecid
                      }
  rs0:setReaderParam  {
                            bsr     =   bsr[dch],
                            param   =   Q_RS_CFG_BSR_DMA_XFER_MODE,
                            value   =   MEMORY_XFER_MODE
                      }

  if VIDEO_PLAYBACK_ONLY == 0 and dch <= AUDIO_NUM_CHANNELS then
    showdecodeaudio(dch)
  end
    
  avsndec[dch]:start()
  avdec[dch]:start()
  
  rs0:play{bsr=bsr[dch]}
  
  FileToDecCh[fch] = dch
  DecChActive[dch] = 1
  
end



------------------------------------------------------------------------------
--- Stops rendering a live encode channel.
--
-- This function stops rendering the video channel associated to the main
-- encode channel i.
--
-- @param fch File channel to stop
--
-- @return nil

function stopplayback(fch)
  local ch = FileToDecCh[fch]
  if (ch == nil) then
    return
  end
  rs0:close{bsr=bsr[ch]}
  avdec[ch]:stop()
  avdec[ch]:waitForVideoEOS()
  avsndec[ch]:stop()
  rs0:unbindDecoder {   dec     =   avdec[ch]    }
  FileToDecCh[fch] = nil
  DecChActive[ch] = nil
end

-- TRICK MODE FUNCTIONS 



------------------------------------------------------------------------------
--- Pause the decoder.
--
-- This function sets the decoder in pause state.
--
-- @param fch File channel
--
-- @return nil

function pauseplayback(fch)
  local ch = FileToDecCh[fch]
  if (ch == nil) then
    return
  end
  avsndec[ch]:pause()
end

------------------------------------------------------------------------------
--- Resumes the decoding.
--
-- Resumes decoding of channel i after the channel was paused.
--
-- @param fch file channel
--
-- @return nil

function resumeplayback(fch)
  local ch = FileToDecCh[fch]
  if (ch == nil) then
    return
  end
  avsndec[ch]:resume()
end

------------------------------------------------------------------------------
--- Slow motion playback.
--
-- Sets the specified decode channel to slow motion.
--
-- @param fch file channel
-- @param n decoding speed
--
-- @return nil
function slowplayback(fch, n)
  local ch = FileToDecCh[fch]
  if (ch == nil) then
    return
  end
  avsndec[ch]:slow { speed   =   n }
end

------------------------------------------------------------------------------
--- Single step.
--
-- Performs a single step decode.
--
-- @param fch file channel
--
-- @return nil
function stepplayback(fch)
  local ch = FileToDecCh[fch]
  if (ch == nil) then
    return
  end
  avsndec[ch]:step()
end



-- Internal function that generalizes the query operation.
local function __queryobject(max, id, name, list)
    if tonumber(id) == nil or id <= 0 or id > max then
        printf("ERROR : Specify channel number from 1 to %d\n", max)
        return nil        
    end
    
    local tbl = {}
    
    printf("%s CHANNEL %d OUT OF %s\n", name, id, max)
    
    local function q(name, obj) 
        if obj and type(obj.query) == "function" then tbl[name] = obj:query()  end 
    end
    
    for _, key in pairs(list) do
        if _G[key] and _G[key][id] then
            q(key, _G[key][id])
        end
    end
    
    return tbl
end

-------------------------------------------------------------------------------
--- Prints the query information of all objects pertaining to the channel.
--
-- Queries all objects pertaining to the specified channel and returns the 
-- information which has the following indexes.
--
-- lvpp     - lvpp query
-- avcenc   - avcenc query
-- avsnenc  - avsn recon preview query
-- avsnlive - avsn live preview
--
function qmain(i)
    return __queryobject(ENCCH, i, "MAIN", {'lvpp', 'sain', 'asplit', 'avcenc', 'audenc', 'avsnenc', 'avsnlive'})
end

-------------------------------------------------------------------------------
--- Prints the query information of all objects pertaining to the channel.
--
-- Queries all objects pertaining to the specified channel and returns the 
-- information which has the following indexes.
--
-- lvpp     - lvpp query
-- avcenc   - avcenc query
-- avsnenc  - avsn recon preview query
-- avsnlive - avsn live preview
--
function qsec(i)
    return __queryobject(ENCCH_2, i, "SEC", {'nvppenc2','avcenc2'})
end
-------------------------------------------------------------------------------
--- Prints the query information of all objects pertaining to the channel.
--
-- Queries all objects pertaining to the specified channel and returns the 
-- information which has the following indexes.
--
-- avsndec     - lvpp query
-- avdec       - jpegenc query
--
-- @param i channel id
--
-- @return table containing the query results 
--
function qjpeg(i)
    return __queryobject(MJPEGCH, i, "MJPEG", {'lvpp','jpegenc'})
end
-------------------------------------------------------------------------------
--- Prints the query information of all objects pertaining to the channel.
--
-- Queries all objects pertaining to the specified channel and returns the 
-- information which has the following indexes.
--
-- avsndec     - avsn query
-- avdec       - avdec query
--
-- @param i channel id
--
-- @return table containing the query results 
--
function qdec(i)
    return __queryobject(DECCH, i, "DECODE", {'avdec','avsndec'})
end

-----------------------------
-- JPEG Snapshot Functions --
-----------------------------

-- Takes one snapshot
function shot()
   if (SNAPSHOTCH > 0) then
      snapshotenc:snapshot({shots=1});
      print("Snapshot taken.\n");
   end
end

-- Takes multiple snapshots
function shots(numShots)
   if (SNAPSHOTCH > 0) then
      snapshotenc:snapshot({shots=numShots});
      print(numShots.." snapshots taken.\n");
   end
end

function shotquality(q)
   if (SNAPSHOTCH > 0) then
      snapshotenc:setQualityFactor({qualityfactor=q});
      print("JPEG snapshot quality changed to "..q..".\n");
   end
end

function qj()
   ws:query();
  for i = 1,MJPEGCH do
     jpegenc[i]:query();
  end
  if (SNAPSHOTCH > 0) then
     snapshotenc:query();
  end
end


------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- All objects are now configured!
-- Starting
--

-- set display layout
if CUSTOM_DISPLAY_LAYOUT == 0 then
  avoc0:activateVideoChannelCfg()
end

if (ActiveAudioChannel ~= nil) then
  avoc0:activateAudioChannelCfg()
end

-- start display engine
avoc0:start()

local ZOOM = 8
if(OBJECTTRACKING_DISP > 0) then
    osdinit({avoc=avoc0,width=720,height=480,x=0,y=0,memory=OSD_MEMORY_SIZE});
    if (FORMAT == "1080i") then
        os.execute("fbset 1920x1080 LUT8");  -- LUT8 format
    elseif (FORMAT == "720p") then
        os.execute("fbset 1280x720 LUT8");  -- LUT8 format
    elseif (FORMAT == "480i") then
        os.execute("fbset 720x480 LUT8");  -- LUT8 format
    elseif (FORMAT == "576i") then
        os.execute("fbset 720x576 LUT8");  -- LUT8 format
    end

    os.execute("fbset enable 1");   
    os.execute("fbset ck_mode 0");
    os.execute("fbset alpha_select 0");
    os.execute(string.format("fbset hzoom %d", ZOOM))
    os.execute(string.format("fbset vzoom %d", ZOOM))
end


-- start the live avsn
for i = 1,MAX_LVPP_USED do
  avsnlive[i]:start()
end

-- start encoders and their outputs for recon preview
for i = 1,ENCCH do
  if (PREVIEW == 2) then
    avsnenc[i]:start()
  end
  avcenc[i]:record()
end

for i = 1, AUDIO_NUM_CHANNELS do
    if (i <= ENCCH) then
        audenc[i]:record()
    end
end

-- start secondary encoders
for i = 1,ENCCH_2 do
  avcenc2[i]:record()
  if (SECONDARY_NVPP_BYPASS == 0) then
    nvppenc2[i]:start()
  end
end

-- start jpeg encoders
for i = 1,MJPEGCH do
  jpegenc[i]:record()
end

if (SNAPSHOTCH == 1) then
   snapshotenc:record();
end

for i = 1, AUDIO_MAX_SAIN do
   if (VA85_WORK_AROUND == 1) then
      audfil0:start(); -- starts audio filter before SAIN does
   end
   sain[i]:start()
end

if MAIN_DEINT_MODE ~= 0 then
  for i = 1,MAX_LVPP_USED do
    nvpp[i]:start()
  end
end


if (OBJECTTRACKING_DISP == 1) then
-- The DISPLAY will be shown only one of the four encoders.
-- Right now the display is for avcenc[0].
-- To use the other encoders the avcenc handle needs to be modified. The xoff and yoff needs to be modified too.
 -- The trip wire and tracking is the left hand top quadrant
   enableTracking({avcenc=avcenc[1]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
   setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
   enableTripwire({id=0, enable=1});
   setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
   print ("DISPLAY 1")
   -- The trip wire and tracking is the top right hand quadrant
elseif (OBJECTTRACKING_DISP == 2) then
    enableTracking({avcenc=avcenc[2]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    os.execute("fbset hoffset 360")
    print ("DISPLAY 2")
    -- The trip wire and tracking is the bottom left hand quadrant
elseif (OBJECTTRACKING_DISP == 3) then
    enableTracking({avcenc=avcenc[3]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    print ("DISPLAY 3")
    os.execute("fbset voffset 240")
     -- The trip wire and tracking is the bottom right hand quadrant
elseif (OBJECTTRACKING_DISP == 4) then
    enableTracking({avcenc=avcenc[4]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    os.execute("fbset hoffset 360")
    os.execute("fbset voffset 240")
    print ("DISPLAY 4")
elseif (OBJECTTRACKING_DISP == 5) then
-- Multichannel tracking and display enabled.
    enableTracking({avcenc=avcenc[1]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    print ("DISPLAY 1")
    enableTracking({avcenc=avcenc[2]["handle"], xoff=45, yoff=0, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    print ("DISPLAY 2")
    enableTracking({avcenc=avcenc[3]["handle"], xoff=0, yoff=30, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    print ("DISPLAY 3")
    enableTracking({avcenc=avcenc[4]["handle"], xoff=45, yoff=30, width=360, height=240, width_org=720, height_org=480});
    setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
    enableTripwire({id=0, enable=1});
    setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
    print ("DISPLAY 4")
end

for i = 1,VININTERLEAVE do
  lvpp[i]:start()
end


-------------------------------------------------------------------------------
--                                 END OF FILE                               --
-------------------------------------------------------------------------------


