

--This script is used to decode the input file specified. 

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
--VOUT        	= VOUT or "hdmi";
VOUT        	= VOUT or "svideo";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
-- "576i"
--FORMAT      	= FORMAT or "1080i";
FORMAT      	= FORMAT or "480i";

VIDEO_CODEC 	= VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_M2V;
AUDIO_CODEC     = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QMA;
STREAM_TYPE     = STREAM_TYPE or Q_AVDEC_CFP_VIDEO_STREAM_TS;
--STREAM_TYPE     = STREAM_TYPE or Q_AVDEC_CFP_VIDEO_STREAM_PS;
--STREAM_TYPE     = STREAM_TYPE or Q_AVDEC_CFP_VIDEO_STREAM_QBOX;

FILENAME 	= FILENAME or "/directmnt/qa/streams/merlin/hbo_trailer_6ksd.ts"
--FILENAME 	= FILENAME or "/directmnt/qa/streams/merlin/hbo_trailer_6k_1920x1080.ts" 
--FILENAME 	= FILENAME or "/directmnt/qa/streams/merlin/ray_pal.vod"
--FILENAME 	= FILENAME or "/directmnt/qa/streams/merlin/travel.ts"
--FILENAME 	= FILENAME or "/directmnt/qa/streams/merlin/Shrek-001.vob"

-- must pre-define pid for transport stream, in the above examples, hbo_trailer is 17, and travel is 33 
STREAM_PID      = STREAM_PID or 17;

-- Direct memory transfer mode 
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 0;

-- Allow the display to be disabled for power testing
DISABLE_DISPLAY = DISABLE_DISPLAY or 0

if (FORMAT == "720p") then
  HSIZE = 1280;
  VSIZE = 720;
 elseif (FORMAT == "1080i") then
  HSIZE = 1920;
  VSIZE = 1080;
 elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
 elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
else
  print("Invalid format\n");
end

AUDIO_BUFFER_SIZE = 1000000;
VIDEO_BUFFER_SIZE = 2000000;
VIDEO_STREAM_ID = 0;
AUDIO_STREAM_ID = 1;

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

-- Set up the output port
if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end

-- create objects
avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC, vtype=VIDEO_CODEC, 
                              vstream=STREAM_TYPE, audioBufferSize=AUDIO_BUFFER_SIZE, 
			      videoBufferSize=VIDEO_BUFFER_SIZE, maxWidth=HSIZE, maxHeight=VSIZE,
			      numoutputframes=6,videoAUs=1024});
rs0 = sysctl0:createReadStreamer({name="rs0"})

-- set video hw params
if (FORMAT == "480i") then
  avoc0:setOutput480i();
elseif (FORMAT == "576i") then
  avoc0:setOutput576i();
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
end

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEO_STREAM_ID});

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_MARGIN, value=5000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});

if (FORMAT == "576i") then
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_FIELD_ORDER, value=Q_AVDEC_CFP_VIDEO_FIELD_ORDER_FORCE_BOTTOM_FIRST});
end
if (STREAM_TYPE == Q_AVDEC_CFP_VIDEO_STREAM_TS) then 
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TS_PID, value=STREAM_PID});
end

avdec0:activateCfg();

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});
if (DISABLE_DISPLAY ~= 0) then
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_ENABLE, value=0});
end
avoc0:activateCfg();

avsn0:bindVideoInput({src=avdec0});

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

avoc0:activateVideoChannelCfg();

avoc0:start();
function play()
    --startTime = os.time();
    rs0:bindDecoder({dec=avdec0});
    if (STREAM_TYPE == Q_AVDEC_CFP_VIDEO_STREAM_QBOX) then
      bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    else 
      bsr0 = rs0:open({type="dump", file=FILENAME, srcId={VIDEO_STREAM_ID, AUDIO_STREAM_ID}});
    end

    rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});

    avsn0:start();
    avdec0:start();
    rs0:play({bsr=bsr0});
    --avdec0:waitForVideoEOS();
    --endTime = os.time();
    --totalTime = os.difftime(endTime, startTime);
    --print(string.format("Decode Time = %d seconds\n", totalTime));
end


function stop()
  --rs0:close({bsr=bsr0});
  avdec0:stop();
  --avdec0:waitForVideoEOS();
  avsn0:stop();
end

play();

--[[
for i = 0, 2, 1 do
  os.execute("sleep 1");
end

print("STOP!!!\n");
stop();

for i = 0, 2, 1 do
  os.execute("sleep 1");
end

print("RESTART!!!\n");
avsn0:start();
avdec0:start();
--]]
