
--[[

This script is used to decode 2 streams simultaneously 
and display on the screen. 

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of decoded channels
DECCH       = DECCH or 2;

-- Number of rendered images and their scaled size
OUTCH       = OUTCH or 2;
FORMAT_OUT  = FORMAT_OUT or "1080i";
FORMAT_IN   = FORMAT_IN or "480i";
VIN         = VIN or "svideo";
VOUT        = VOUT or "hdmi";
OUTW        = OUTW or 720;
OUTH        = OUTH or 480;

-- Width and height of the output grid
OUTGRIDW    = OUTGRIDW or 2;
OUTGRIDH    = OUTGRIDH or 2;

STREAMING   = STREAMING or 0;

if (STREAMING == 0) then
  -- Input file names (# of files should match # of decoded channels)
  INPUTFILENAME    = INPUTFILENAME or {};
  INPUTFILENAME[0] = INPUTFILENAME[0] or "/directmnt/qa/streams/merlin/480i_48k_qac_spider1.qbx";
  INPUTFILENAME[1] = INPUTFILENAME[1] or "/directmnt/qa/streams/merlin/480i_48k_qac_spider2.qbx";
else
  -- IP Address of server for RTP
  SERVER_IP        = SERVER_IP or {};
  SERVER_IP[0]     = SERVER_IP[0] or "10.32.84.176";
  SERVER_IP[1]     = SERVER_IP[1] or "10.32.84.176";

  -- Server port used in RTP
  SERVER_PORT      = SERVER_PORT or {};
  SERVER_PORT[0]   = SERVER_PORT[0] or 554;
  SERVER_PORT[1]   = SERVER_PORT[1] or 554;

  -- session identifier for rtp server
  SESSION_ID       = SESSION_ID or {};
  SESSION_ID[0]    = SESSION_ID[0] or "test";
  SESSION_ID[1]    = SESSION_ID[1] or "test";

  -- RTP Data protocol - "udp" or "tcp"
  RTP_DATA_PROTO    = RTP_DATA_PROTO or {};
  RTP_DATA_PROTO[0] = RTP_DATA_PROTO[0] or "udp";
  RTP_DATA_PROTO[1] = RTP_DATA_PROTO[1] or "udp";

  -- Don't set Target Latency Values to test bypassing logic
  TARGET_LATENCY_TOLERANCE    = TARGET_LATENCY_TOLERANCE or {};
  TARGET_LATENCY_TOLERANCE[0] = TARGET_LATENCY_TOLERANCE[0] or 0; -- in milliseconds
  TARGET_LATENCY_TOLERANCE[1] = TARGET_LATENCY_TOLERANCE[1] or 0; -- in milliseconds
  TARGET_LATENCY_UNDERFLOW    = TARGET_LATENCY_UNDERFLOW or {};
  TARGET_LATENCY_UNDERFLOW[0] = TARGET_LATENCY_UNDERFLOW[0] or 0;  -- in milliseconds
  TARGET_LATENCY_UNDERFLOW[1] = TARGET_LATENCY_UNDERFLOW[1] or 0;  -- in milliseconds
  TARGET_LATENCY_OVERFLOW     = TARGET_LATENCY_OVERFLOW or {};
  TARGET_LATENCY_OVERFLOW[0]  = TARGET_LATENCY_OVERFLOW[0] or 0;   -- in milliseconds
  TARGET_LATENCY_OVERFLOW[1]  = TARGET_LATENCY_OVERFLOW[1] or 0;   -- in milliseconds

  -- Audio ring buffer size
  AUDIO_RING_BUFF_SIZE        = AUDIO_RING_BUFF_SIZE or {};
  AUDIO_RING_BUFF_SIZE[0]     = AUDIO_RING_BUFF_SIZE[0] or 524288;
  AUDIO_RING_BUFF_SIZE[1]     = AUDIO_RING_BUFF_SIZE[1] or 524288;

  -- Video ring buffer size
  VIDEO_RING_BUFF_SIZE        = VIDEO_RING_BUFF_SIZE or {};
  VIDEO_RING_BUFF_SIZE[0]     = VIDEO_RING_BUFF_SIZE[0] or 2097152;
  VIDEO_RING_BUFF_SIZE[1]     = VIDEO_RING_BUFF_SIZE[1] or 2097152;
end

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;

-- Play repeatly 
REPEAT_MODE = REPEAT_MODE or 0;

-- audio parameters

SAMPLERATE       = SAMPLERATE or {};
SAMPLERATE[0]    = SAMPLERATE[0] or 48000;
SAMPLERATE[1]    = SAMPLERATE[1] or 48000;

SAMPLESIZE       = SAMPLESIZE    or 16;

AUDIO_CODEC      = AUDIO_CODEC or {};
AUDIO_CODEC[0]   = AUDIO_CODEC[0] or Q_AVDEC_CFP_AUDIO_CODEC_QAC;
AUDIO_CODEC[1]   = AUDIO_CODEC[1] or Q_AVDEC_CFP_AUDIO_CODEC_QAC;

OUTAUDCH         = OUTAUDCH or 0;

--[[ Low delay decoder options
  0 = disable low delay mode.
  1 = Low Delay mode, works with IPPP only bit streams.
--]]
LOW_DELAY_MODE = LOW_DELAY_MODE or 0;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

videoInputBufferSize = 500 * 2048;
audioInputBufferSize = 50 * 2048;

BITSTREAM_OUTPUT = BITSTREAM_OUTPUT or 1;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=DECCH*videoInputBufferSize+DECCH*audioInputBufferSize});
board.setAudioSampleFreq({port=0, freq=SAMPLERATE[0]});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format=FORMAT_OUT, mode=VOUT, sync=vout_sync});
board.setVideoInput({port=0, format=FORMAT_IN, mode=VIN});

sysctl0:setVerboseLevel({param=1});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

if (STREAMING > 0) then
  -- Restore the socket receive buffer kernel parameters to default
  -- This is needed as they might be increased if avdecode.lua was executed earlier
  os.execute("sysctl -w net.core.rmem_default=104448");
  os.execute("sysctl -w net.core.rmem_max=104448");
end

-- Create Decoders
avdec = {};
avsndec = {};
ach = {};
DECCH = DECCH - 1;

for i = 0,DECCH do
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;

    avdec[i] = sysctl0:avdecCreate({name="avdec", atype=AUDIO_CODEC[i], vtype=VIDEO_CODEC, audioBufferSize=audioInputBufferSize, videoBufferSize=videoInputBufferSize, maxWidth=720, maxHeight=480, numoutputframes=8, videoAUs=200, ioMode=Q_AVDEC_CFP_TRUE});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audioStreamId});
    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=videoStreamId});
    --avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=Q_AVDEC_CFP_TRUE});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE[0]});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
    avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
--    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
--    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3000});
--    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=250000});
--    avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
    if (LOW_DELAY_MODE == 1) then
      avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_OUTPUT_TIMING, value=Q_AVDEC_CFP_VIDEO_OUTPUT_TIMING_LOW_DELAY});
    end
    avdec[i]:activateCfg();

    ach[i] = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
end

vch = {};
x = 0;
y = 0;
OUTCH = OUTCH - 1;

for i = 0,OUTCH do
    avsndec[i] = sysctl0:avsnCreate({name="avsndec"});
    
    vch[i] = avoc0:addVideoChannel();
    
    avsndec[i]:bindVideoInput({src=avdec[i]});
        
    avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
    
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
    avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});
    
    x = x + OUTW;
    if x == COMPWIDTH then
        x = 0;
        y = y + OUTH;
    end
    
    avsndec[i]:bindAudioInput({src=avdec[i]});
    avoc0:setAudioChannelSource({ch=ach[i], src=avsndec[i]});
end

avoc0:activateVideoChannelCfg();

if (OUTAUDCH == 0) then  
    avoc0:enableAudioChannel({ch=ach[0], enable=1});
else
    avoc0:enableAudioChannel({ch=ach[1], enable=1});
end

avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE[0]});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=SAMPLESIZE});
avoc0:activateCfg();
avoc0:activateAudioChannelCfg();

bsr = {};
rs0 = sysctl0:createReadStreamer({name="rs0"});

for i = 0,DECCH do
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;
    
    rs0:bindDecoder({dec=avdec[i]});
    if (STREAMING == 0) then
      bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={1, 2}, codecId={audioStreamId, videoStreamId}});
    else
      bsr[i] = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,vbs=%d,abs=%d",
                SERVER_IP[i], SERVER_PORT[i], RTP_DATA_PROTO[i], SESSION_ID[i],
                TARGET_LATENCY_TOLERANCE[i], TARGET_LATENCY_OVERFLOW[i], TARGET_LATENCY_UNDERFLOW[i],
                VIDEO_RING_BUFF_SIZE[i], AUDIO_RING_BUFF_SIZE[i]),
                srcId={videoStreamId, audioStreamId}});

    end
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_LOOPMODE, value=REPEAT_MODE});
    rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
end


-- START


avoc0:start();

for i = 0,OUTCH do
    avsndec[i]:start();
end

for i = 0,DECCH do
    avdec[i]:start();
    rs0:play({bsr=bsr[i]});
end

--SWITCH
for i = 0, 25, 1 do
  sysctl0:echo({param=0});
  os.execute("sleep 1");
end

avoc0:enableAudioChannel({ch=ach[OUTAUDCH], enable=0});

if (OUTAUDCH == 0) then
    OUTAUDCH = 1;
else
    OUTAUDCH = 0;
end

print("switch !!!!!!!!!!!!!!!!!!!!\n");
avoc0:enableAudioChannel({ch=ach[OUTAUDCH], enable=1});
avoc0:activateCfg();
avoc0:activateAudioChannelCfg();

--SWITCH BACK 
for i = 0, 25, 1 do
  sysctl0:echo({param=0});
  os.execute("sleep 1");
end

avoc0:enableAudioChannel({ch=ach[OUTAUDCH], enable=0});

if (OUTAUDCH == 0) then
    OUTAUDCH = 1;
else
    OUTAUDCH = 0;
end

print("switch back!!!!!!!!!!!!!!!!!!!!\n");
avoc0:enableAudioChannel({ch=ach[OUTAUDCH], enable=1});
avoc0:activateCfg();
avoc0:activateAudioChannelCfg();







function q()
    for i = 0,DECCH do
        avdec[i]:query();
    end
end

function DecStart(i)
    local audioStreamId = 2*i + 1;
    local videoStreamId = 2*i + 2;
    
    rs0:bindDecoder({dec=avdec[i]});
    if (STREAMING == 0) then
      bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={1, 2}, codecId={audioStreamId, videoStreamId}});
    else
      bsr[i] = rs0:open({type="rtp", file=string.format("sip=%s,prt=%d,dat=%s,ses=%s,tlt=%d,tlo=%d,tlu=%d,vbs=%d,abs=%d",
                SERVER_IP[i], SERVER_PORT[i], RTP_DATA_PROTO[i], SESSION_ID[i],
                TARGET_LATENCY_TOLERANCE[i], TARGET_LATENCY_OVERFLOW[i], TARGET_LATENCY_UNDERFLOW[i],
                VIDEO_RING_BUFF_SIZE[i], AUDIO_RING_BUFF_SIZE[i]),
                srcId={videoStreamId, audioStreamId}});
    end
    avsndec[i]:start();
    avdec[i]:start();
    rs0:play({bsr=bsr[i]});
end


function DecStop(i)
    rs0:close({bsr=bsr[i]});
    avdec[i]:stop();
    avsndec[i]:stop();
    rs0:unbindDecoder({dec=avdec[i]});
end
