--[[

This script is used to encode four D1 streams from 1 NTSC stream via svideo input.  
The output is 4 frames displayed simultaneously on the display. 

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Init
BITRATE         = BITRATE or {};
BITRATE_2       = BITRATE_2 or {};

-- Video output can be 1080i or 480i (note composite also is driven in svideo)
FORMAT          = FORMAT or '480i';

-- Video input can be svideo or composite
VIN             = VIN or "composite";

-- Video output is set to HDMI for 1080i mode and svideo for 480i mode
VOUT            = VOUT or "svideo";

-- Number of live encoded channels
ENCCH           = ENCCH or 4;

-- Number of derived channels
ENCCH_2         = ENCCH_2 or 0;
MOTIONDETECTION        = MOTIONDETECTION 	or 0;
OBJECTTRACKING         = OBJECTTRACKING 	or 1;
OBJECTTRACKING_DISP    = OBJECTTRACKING_DISP 	or 0;     --set to 1 from obj.sh

-- Enable Analytics Streaming
-- 0 : Don't stream analytics
-- 1 : Stream analytics as RTP metadata with rtpwriter
-- 2 : Stream analytics in Private SEI with rtpwriter
ENABLE_ANALYTICS_STREAMING = ENABLE_ANALYTICS_STREAMING or 1 

-- 0 for nothing, 1 for live, 2 for reconstructed frames
PREVIEW         = PREVIEW or 1;

-- Host IP and base port for udp live stream (port, port+1, port+2...)
-- 0 is no streaming, 1 for UDP, 2 for RTP
STREAMING       = STREAMING or 2;
HOST            = HOST or "192.168.40.38";
SERVER_IP       = SERVER_IP or "NULL";
PORT            = PORT or 554;
MAX_PACKET_SIZE = MAX_PACKET_SIZE or 1472;

-- Enable multiple rtp servers on different ports
-- Default is multiple session on one rtp server
MULTIPLE_RTPSERVERS     = MULTIPLE_RTPSERVERS or 0;

-- Port for 2nd streams (port, port+1...)
PORT_2          = PORT_2 or 9750;

-- Width and height of each live encoded stream
ENCW            = ENCW or 720;
ENCH            = ENCH or 480;

-- Bitrate of each live encoded stream
BITRATE[0]      = BITRATE[0] or 2500000;
BITRATE[1]      = BITRATE[1] or 2000000;
BITRATE[2]      = BITRATE[2] or 1500000;
BITRATE[3]      = BITRATE[3] or 1000000;

-- Width and height of each derived encoded stream
ENCW_2          = ENCW_2 or 352;
ENCH_2          = ENCH_2 or 240;

-- Bitrate of each derived encoded stream
BITRATE_2[0]    = BITRATE_2[0] or 500000;
BITRATE_2[1]    = BITRATE_2[1] or 400000;
BITRATE_2[2]    = BITRATE_2[2] or 300000;
BITRATE_2[3]    = BITRATE_2[3] or 200000;

-- Number of reference frames, should be larger than 4 for recon preview,
-- exactly 4 for IP encode
NUM_AVC_REFERENCES = NUM_AVC_REFERENCES or 8;

-- Renderer field order.  This should match the field order of the input.
-- For example, DVD input is typically BFF.
RENDER_BFF = 1;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

BITSTREAM_OUTPUT=1;

if (FORMAT == "1080i") then
  DISPLAYWIDTH = 1920;
  DISPLAYHEIGHT = 1080;
  OUTW = 720;
  OUTH = 480;
elseif (FORMAT == "480i") then
  DISPLAYWIDTH = 720;
  DISPLAYHEIGHT = 480;
  OUTW=360;
  OUTH=240;
end

-- Number of rendered images
OUTCH=4;

-- Width and height of the output grid
OUTGRIDW=2;
OUTGRIDH=2;

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end


board.setVideoInput({port=0, format='480i', mode=VIN});
if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
else
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
end



sysctl0:setVerboseLevel({param=1});

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (DISPLAYHEIGHT-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (DISPLAYWIDTH-(OUTGRIDW*OUTW))/2;

-- show or hide bitrate 0 for off, 1 for on
SHOWRATE = 0;
if (FORMAT == "720p") or (FORMAT == "1080i") then
    OSD_MEMORY_SIZE     = 8*1024*1024;
else
    OSD_MEMORY_SIZE     = 4*1024*1024;
end

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT, gfx0Size=OSD_MEMORY_SIZE, allocGfx=1});
if (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
elseif (FORMAT == "480i") then
  avoc0:setOutput480i();
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_BFF, value=RENDER_BFF});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();


ENCCH = ENCCH - 1;
OUTCH = OUTCH - 1;
ENCCH_2 = ENCCH_2 - 1;
if (OUTCH > ENCCH) then
  OUTCH = ENCCH;
end

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=ENCW, maxHeight=ENCH, numVidFrame=11, maxPMEDSFactor=1});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:showrate({enable=SHOWRATE});
ws:start()

if (ENABLE_ANALYTICS_STREAMING > 0) then
	print("Enable metadata....");
	ws:configure({param=Q_WS_CFG_METADATA, value=1});
end

Setup_VPP(lvpp0, 720, 480, 1, 0, 2, 4);
lvpp0:crop({cropWidth=720,cropHeight=480, horzOffset=0, vertOffset=0});
lvpp0:size({outputWidth=ENCW, outputHeight=ENCH});
lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9});
lvpp0:activateCfg();

-- CREATE ENCODERS
nvppenc2 = {};
avcenc = {};
avcenc2 = {};
avsnenc = {};
videoStreamId = {};

-- AVC SETUP

for i = 0,ENCCH do
  avcenc[i] = sysctl0:avcencCreate({name="avcenc", ws=ws, maxwidth=ENCW, maxheight=ENCH, numreferenceframes=NUM_AVC_REFERENCES, objectTrackingEnable=OBJECTTRACKING});
  avsnenc[i] = sysctl0:avsnCreate({name="avsn"});
  videoStreamId = i + 2;

  avcenc_480i(avcenc[i], "high", 31, BITRATE[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, 0)
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  if PREVIEW == 2 then
    avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW, value=1});
  end

  avcenc[i]:bindVideoInput({src=lvpp0});
  if STREAMING == 1 then
    ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", HOST, PORT+i, MAX_PACKET_SIZE), sid = {videoStreamId}});
  elseif STREAMING == 2 then
    if MULTIPLE_RTPSERVERS == 1 then
    	ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=stream%d_D1,vid=h264,mod=0,msz=%d,aim=%d", SERVER_IP, 
                PORT+i, 0, MAX_PACKET_SIZE, ENABLE_ANALYTICS_STREAMING), flags = Q_WS_CFP_FLAG_ENABLE_METADATA, sid = {videoStreamId}});
    else
      -- Open one server on port 554 with sessions /stream0_D1, /stream1_D1 etc.
      -- You open the stream in VLC with...
      -- rtsp://server_ip:554/stream0
      ws:open({type="rtp", file=string.format("sip=%s,prt=%d,ses=stream%d_D1,vid=h264,mod=0,msz=%d,aim=%d", SERVER_IP,
                PORT, i, MAX_PACKET_SIZE, ENABLE_ANALYTICS_STREAMING), flags = Q_WS_CFP_FLAG_ENABLE_METADATA, sid = {videoStreamId}});
    end 
  end
  avcenc[i]:setStreamID({streamid=videoStreamId});
  avcenc[i]:setAckInterval({interval=8});

    if MOTIONDETECTION == 1 then
	    avcenc[i]:setVideoEncParam({param=Q_AVCENC_CMP_MOTION_DETECT_ENABLE, value=1});
	    avcenc[i]:addMDROI({index=i, width=1, height=1, offsetX=0, offsetY=0});
	    avcenc[i]:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_ENABLE, value=1});
	    avcenc[i]:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_MOTION_THRESHOLD, value=8});
	    avcenc[i]:setVideoEncROIParam({idx=i, param=Q_AVCENC_CMP_ROI_PERCENT, value=1000});
						            
    end

    if ((OBJECTTRACKING == 1) and (i<4)) then
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                      value=1});
        avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_FACTOR,          value=1});
        avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_OFFSET,          value=0});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,           value=12});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,       value=10});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,       value=15});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,       value=256});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,       value=256});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,           value=9});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                  value=30});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                  value=30});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,        value=1});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,        value=0});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,value=1});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,            value=9});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,     value=600});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,      value=255});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,              value=20});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,       value=2});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,   value=1});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,          value=1});
	avcenc[i]:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,        value=1});
			
     end
		
   avcenc[i]:activateVideoEncCfg();
end

for i = 0, ENCCH_2 do
  nvppenc2[i] = sysctl0:nvppCreate({name="nvpp0", maxWidth=ENCW_2, maxHeight=ENCH_2, maxPMEDSFactor=1});
  avcenc2[i] = sysctl0:avcencCreate({name="avcenc", ws=ws, maxwidth=ENCW_2, maxheight=ENCH_2, numreferenceframes=4});
  -- delta of 16 between live and derived channels
  videoStreamId = i + 16;

  -- workaround - crop instead of scale 2nd channel
  Setup_VPP(nvppenc2[i], ENCW, ENCH, 0, 0, 2, 2)
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PICT_WIDTH,           value=ENCW_2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PICT_HEIGHT,          value=ENCH_2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_X,        value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_CROP_OFFSET_Y,        value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_X,           value=1});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y,           value=1});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PMEDS_H,              value=2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_PMEDS_V,              value=2});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_STC_PER_FRAME,        value=3003});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  nvppenc2[i]:setParam({param=Q_LVPP_CMP_PMEDS_EN,             value=0});
  nvppenc2[i]:setParam({param=Q_NVPP_CMP_HW_BYPASS_TFILTC,     value=1});
  nvppenc2[i]:activateCfg();


  avcenc_480i(avcenc2[i], "high", 31, BITRATE_2[i], Q_AVCENC_CFP_GOP_STRUCTURE_IP, 0)
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=3003});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT, value=BITSTREAM_OUTPUT});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_CLOSED_GOP, value=1});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_NEW_SPS_EACH_IDR, value=1});
  avcenc2[i]:setVideoEncParam({param=Q_AVCENC_CMP_PME_ENABLE, value=0});
  avcenc2[i]:activateVideoEncCfg();

  nvppenc2[i]:bindInput({src=lvpp0});
  avcenc2[i]:bindVideoInput({src=nvppenc2[i]});
  if STREAMING == 1 then
  ws:open({type="net", file=string.format("host=%s,port=%d,protocol=udp,velem=1,max=%d", HOST, PORT_2+i, MAX_PACKET_SIZE), sid = {videoStreamId}});
  end
  avcenc2[i]:setStreamID({streamid=videoStreamId});
  avcenc2[i]:setAckInterval({interval=8});
end

vch = {};
x = 0;
y = 0;
for i = 0,OUTCH do
  vch[i] = avoc0:addVideoChannel();

  if PREVIEW == 1 then
    print("binding video input to live\n");
    avsnenc[i]:bindVideoInput({src=lvpp0});
  elseif PREVIEW == 2  then
    avsnenc[i]:bindVideoInput({src=avcenc[i]});
    avsnenc[i]:setParam({param=Q_AVSN_CMP_EXTERNAL_STC_OFFSET, value=-12012});
    avsnenc[i]:activateCfg();
  end
  avoc0:setVideoChannelSource({ch=vch[i], src=avsnenc[i]});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=ENCW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=ENCH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end
avoc0:activateVideoChannelCfg();

-- START
local ZOOM = 8
if(OBJECTTRACKING_DISP > 0) then
    osdinit({avoc=avoc0,width=720,height=480,x=0,y=0,memory=OSD_MEMORY_SIZE});    
    if (FORMAT == "1080i") then
	    os.execute("fbset 1920x1080 LUT8");  -- LUT8 format
    elseif (FORMAT == "720p") then
	    os.execute("fbset 1280x720 LUT8");  -- LUT8 format
    elseif (FORMAT == "480i") then
	    os.execute("fbset 720x480 LUT8");  -- LUT8 format
    elseif (FORMAT == "576i") then
	    os.execute("fbset 720x576 LUT8");  -- LUT8 format
    end

os.execute("fbset enable 1");   
os.execute("fbset ck_mode 0");
os.execute("fbset alpha_select 0");
os.execute(string.format("fbset hzoom %d", ZOOM))
os.execute(string.format("fbset vzoom %d", ZOOM))
end

avoc0:start();

if (OBJECTTRACKING_DISP == 1) then
-- The DISPLAY will be shown only one of the four encoders.
-- Right now the display is for avcenc[0].
-- To use the other encoders the avcenc handle needs to be modified. The xoff and yoff needs to be modified too.
 -- The trip wire and tracking is the left hand top quadrant
   enableTracking({avcenc=avcenc[0]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
   setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
   enableTripwire({id=0, enable=1});
   setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
   enableCapacity({id=0, enable=1, hysteresis=200});
   print ("DISPLAY 1")
   -- The trip wire and tracking is the top right hand quadrant
elseif (OBJECTTRACKING_DISP == 2) then
	enableTracking({avcenc=avcenc[1]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
	setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
	enableTripwire({id=0, enable=1});
	setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
        enableCapacity({id=0, enable=1, hysteresis=200});
	os.execute("fbset hoffset 360")
	print ("DISPLAY 2")
	-- The trip wire and tracking is the bottom left hand quadrant
elseif (OBJECTTRACKING_DISP == 3) then
	enableTracking({avcenc=avcenc[2]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
	setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
	enableTripwire({id=0, enable=1});
	setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
        enableCapacity({id=0, enable=1, hysteresis=200});
	print ("DISPLAY 3")
    os.execute("fbset voffset 240")
     -- The trip wire and tracking is the bottom right hand quadrant
elseif (OBJECTTRACKING_DISP == 4) then
	enableTracking({avcenc=avcenc[3]["handle"], xoff=0, yoff=0, width=360, height=240, width_org=720, height_org=480});
	setupTripwire({id=0, start_x=720/2, start_y=480/4, end_x=720/2, end_y=(480*3)/4, direction=0});
	enableTripwire({id=0, enable=1});
	setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
        enableCapacity({id=0, enable=1, hysteresis=200});
	os.execute("fbset hoffset 360")
    os.execute("fbset voffset 240")
    print ("DISPLAY 4")
end

for i = 0,OUTCH do
  avsnenc[i]:start();
end

for i = 0,ENCCH do
  avcenc[i]:record();
end

for i = 0,ENCCH_2 do
  avcenc2[i]:record();
  nvppenc2[i]:start();
end

lvpp0:start();

function done()
  lvpp0:flush();

  waitForEvents({objid=avcenc[0], timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});

  os.execute("echo flushed");

  for i = 0,ENCCH do
    ws:close({type="h4v", file=string.format("test%d.h4v", i)});
  end
end

--os.execute("sleep 10");
--done();

function q()
  lvpp0:query();
  for i = 0,ENCCH do
    avcenc[i]:query();
  end
end
