--[[
    
    script to demonstrate object tracking and graphics supported by
    DirectFB. Do not use mobiapp -f to run this script directly,
    because it needs to load dfbobjtack.so. Instead, use ./obj.sh
    to launch it.

--]]

-- Input file
FILEIN          = FILEIN or "./video.yuv";

-- Video output mode
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT            = VOUT or "svideo";

-- Video format for capture and display.
-- "480i"
-- "576i"
FORMAT          = FORMAT or "480i";
OBJECTTRACKING_DISP = OBJECTTRACKING_DISP or 1; 

HSIZE = 720;  
if (FORMAT == "480i") then
	VSIZE         	= 480;
elseif (FORMAT == "576i") then
	VSIZE         	= 576;
	VIN_STC		= 3600;
	VIN_OUTPUT_STC 	= 3600;
end

-- OSD_BUFFER_WIDTH is the physical width of the buffer in pixels which is
-- typically 2048/bytes per pixe if 16bpp
OSD_BUFFER_WIDTH   	= OSD_BUFFER_WIDTH or 2048;		 -- make it 2048
OSD_BUFFER_HEIGHT  	= OSD_BUFFER_HEIGHT or VSIZE;
OSD_X_OFFSET		= OSD_X_OFFSET or 0;
OSD_Y_OFFSET		= OSD_Y_OFFSET or 0;
OSD_MEMORY_SIZE     	= 4*1024*1024;  --1 for "single buffer"

OSD_DISPLAY_WIDTH  	= OSD_DISPLAY_WIDTH or HSIZE;
OSD_DISPLAY_HEIGHT 	= OSD_DISPLAY_HEIGHT or VSIZE;

--output file(s)
OUTFILE_H4V = OUTFILE_H4V or nil;   --"/path/test.h4v"
OUTFILE_QBOX = OUTFILE_QBOX or nil; --"/path/test.qbx"

-- modulo 16 size
VSIZE16 = 16*math.ceil(VSIZE/16)

VIDEOSTREAMID = 2;

sysctl0:setVerboseLevel({param=1});
board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
sysctl0:setVPortMode({port=1, live=0});

hvcap0 = sysctl0:hvcapCreate({name="hvcap0", streamid=VIDEOSTREAMID, maxframeheight=VSIZE16, maxframewidth=HSIZE, numframesinpool=6});
-- hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_WIDTH, value=720});
hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_WIDTH, value=384});
-- hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_HEIGHT, value=480});
hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_HEIGHT, value=288});
hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_INTERLACED, value=1});
hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_IS420, value=1});
hvcap0:setParam({param=Q_HVCAP_CMP_FRAME_ISPLANAR, value=1});
hvcap0:activateCfg();


nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=HSIZE, maxHeight=VSIZE16});   
nvpp0:bindInput({src=hvcap0});

ws = sysctl0:createWriteStreamer({name="ws0"});
ws:start()

--Setup_VPP(nvpp0, HSIZE, VSIZE, 1, 0, 2, 2)
Setup_VPP(nvpp0, HSIZE, VSIZE, 1, 0, 2, 2)

if (FORMAT == "480i") then
nvpp0:size({outputWidth=720, outputHeight=480});
nvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
nvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
nvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
nvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
nvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
nvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9}); 
elseif (FORMAT == "576i") then
nvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=313});
nvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
nvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=24});
nvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=16});
nvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=15}); 
nvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=VIN_STC});
nvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=VIN_OUTPUT_STC});
end
nvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
nvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTY, value=0});
nvpp0:setParam({param=Q_LVPP_CMP_HW_BYPASS_TFILTC, value=0});

--[[
function filterHorzEdgeAdapt(vpp, threshold1, threshold2 , lpLevel)
   // Applies filter specified in registers to flat areas, but not at object edges
   // lp_level can be between 0-4. These represent the lp0 to lp4 filters.
  // defaults threshold: 16, 8
--]]
filterHorzEdgeAdapt(nvpp0, 13, 8, 4); 

--[[
function filterTemporalMotion( vpp, colorMask, ycorrIndex, ylowThr, yhighThr, ygainIndex0, ygainIndex1, cgainIndex0, cgainIndex1, vMotFiltBypass)
// Motion-adaptive recursive temporal filter
--]]
filterTemporalMotion(nvpp0, 0, 0, 16, 240, 0, 0, 0, 0, 0);
-- rescaling (hard coded to D1)
nvpp0:size({outputWidth=720, outputHeight=480});
nvpp0:activateCfg();

avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=HSIZE, maxheight=VSIZE16, numreferenceframes=4, objectTrackingEnable=1});
avcenc_480i(avcenc0, "high", 40, 2000000, Q_AVCENC_CFP_GOP_STRUCTURE_IP, "mbaff")
if (FORMAT == "576i") then
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_VUI_NUM_UNITS_IN_TICK, value=VIN_OUTPUT_STC});
end
avoc0 = sysctl0:avocCreate({name="avoc0", gfx0Size=OSD_MEMORY_SIZE, allocGfx=1});
avsn0 = sysctl0:avsnCreate({name="avsn0"});

if (FORMAT == "480i") then
	avoc0:setOutput480i();
elseif (FORMAT == "576i") then
	avoc0:setOutput576i();
end

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_FIXED_LENGTH_NALU_ENABLE    , value=0});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW             , value=0});
avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_BITSTREAM_OUTPUT            , value=0});

-- object tracking parameter settings
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_ENABLE,                      value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_FACTOR,          value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_DOWN_SAMPLE_OFFSET,          value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD,           value=10});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MIN,       value=8});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MAX,       value=15});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MIN,       value=256});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_DIFF_THRESHOLD_MULTFACT_MAX,       value=256});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_LUMAMOT_THRESHOLD,           value=9});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_X,                  value=30});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MINISIZE_Y,                  value=30});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MEDIAN_FILTER_ENABLE,        value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_MOTION_VECTOR_ENABLE,        value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_USE_DOWNSCALED_MOTION_VECTOR,value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_MB_THRESH,            value=9});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_STATIC_NUMFRAMES_THRESH,     value=600});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIMECONSTANT,      value=255});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_UPDATE_TIME,              value=10});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_APPEARANCETIME_THRESH,       value=2});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_CONTOUR_GENERATION_ENABLE,   value=0});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BLOBTRACKER_ENABLE,          value=1});
avcenc0:setVAParam({param=Q_AVCENC_CMP_VA_OT_BG_FORCE_FIRST_FRAME,        value=1});
-- end object tracking parameter settings

avcenc0:activateVideoEncCfg();
avcenc0:bindVideoInput({src=nvpp0});

if (OUTFILE_H4V) then
ws:open({type="h4v", file="OUTFILE_H4V", sid = {VIDEOSTREAMID}});
end
if (OUTFILE_H4V) then
ws:open({type="file", file="OUTFILE_QBOX", sid = {VIDEOSTREAMID}});
end

avcenc0:setStreamID({streamid=VIDEOSTREAMID});
avsn0:bindVideoInput({src=nvpp0});
vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:activateVideoChannelCfg();

local ZOOM = 2
if(OBJECTTRACKING_DISP==1) then
osdinit({avoc=avoc0,width=OSD_DISPLAY_WIDTH,height=OSD_DISPLAY_HEIGHT,x=OSD_X_OFFSET,y=OSD_Y_OFFSET,memory=OSD_MEMORY_SIZE});

--fbset
if (FORMAT == "1080i") then
	os.execute("fbset 1920x1080 LUT8");  -- LUT8 format
elseif (FORMAT == "720p") then
	os.execute("fbset 1280x720 LUT8");  -- LUT8 format
elseif (FORMAT == "480i") then
	os.execute("fbset 720x480 LUT8");  -- LUT8 format
elseif (FORMAT == "576i") then
	os.execute("fbset 720x576 LUT8");  -- LUT8 format
end

os.execute("fbset enable 1");	
os.execute("fbset alpha_select 0");
os.execute(string.format("fbset hzoom %d", ZOOM))
os.execute(string.format("fbset vzoom %d", ZOOM))
end

avoc0:start();
avcenc0:record();
if(OBJECTTRACKING_DISP==1) then
enableTracking({avcenc=avcenc0["handle"], xoff=0, yoff=0, width=HSIZE, height=VSIZE, width_org=HSIZE, height_org=VSIZE});
setupTracking({zoom=ZOOM, rect_thick=1, outline_contour=0, fill_contour=0, bitmap_contour=0});
enableCapacity({id=0, enable=0, hysteresis=200});
end
avsn0:start();
nvpp0:start();
--avoc0:activateOSDCfg();

-- file
hvcap0:stream({filename=FILEIN});




function done()
	nvpp0:flush();
	waitForEvents(
	{objid=avcenc0, timeout=1000000, any=0, events={Q_AVCENC_EV_BITSTREAM_FLUSHED}});
	os.execute("echo flushed");

	if (OUTFILE_H4V) then
	    ws:close({type="h4v", file="OUTFILE_H4V"});
	end
	if (OUTFILE_QBOX) then    
	    ws:close({type="file", file="OUTFILE_QBOX"});
	end
end
--exit()
