--[[

	This script is to display audio video preview. 
	The format can be set to either 720p, 480i or 1080i.

--]]
----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video input mode
-- "hdmi" HDMI input and output
-- "component"
-- "svideo" 
-- "composite" 
VIN 	= VIN or "component";

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT 	= VOUT or "hdmi";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
-- "576i"
FORMAT = FORMAT or "1080i";

-- Audio sample rate
SAMPLERATE = SAMPLERATE or 32000;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

if (FORMAT == "720p") then
  HSIZE = 1280;  
  VSIZE = 720;
  FRAME_STRUCTURE = "frame"; -- override frame structure
  MAX_PME_DOWNSCALE = 4;
  NUM_VIN_FRAMES = 11;
  if (VIN == "component") then
  	HSTART_PIXEL = 261;
  else
  	HSTART_PIXEL = 171;
  end
elseif (FORMAT == "1080i") then
  HSIZE = 1920;  
  VSIZE = 1080;
  MAX_PME_DOWNSCALE = 4;
  NUM_VIN_FRAMES = 11;
  if (VIN == "component") then
  	HSTART_PIXEL = 235;
  else
  	HSTART_PIXEL = 193;
  end
elseif (FORMAT == "480i") then
  HSIZE = 720;  
  VSIZE = 480;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 11;
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  MAX_PME_DOWNSCALE = 1;
  NUM_VIN_FRAMES = 10;
else
  print("Invalid format\n");
end

-- modulo 16 size
VSIZE16 = 16*math.ceil(VSIZE/16)
AUDIO_FRAME_SIZE = 1024; -- QAC (aac-lc)

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

if (FORMAT == "1080i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00000ba0, interleave=1});
elseif (FORMAT == "720p") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=0, bottomfieldfirst=0, videocontrol=0x00000ba8, interleave=1});
elseif (FORMAT == "480i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
elseif (FORMAT == "576i") then
  board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1}); 
end
board.setVideoInput({port=0, format=FORMAT, mode=VIN});
board.setAudioSampleFreq({port=0, freq=SAMPLERATE});

lvpp0 = sysctl0:lvppCreate({name="lvpp0", slot=0, port=0, maxWidth=HSIZE, maxHeight=VSIZE16, 
                            numVidFrame=NUM_VIN_FRAMES, numPMEDSFrame=NUM_VIN_FRAMES, 
                            maxPMEDSFactor=MAX_PME_DOWNSCALE});

if (FORMAT == "1080i") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=563});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=20});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "720p") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 4, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=25});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=HSTART_PIXEL});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=1500});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=1500});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=1});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=1});
elseif (FORMAT == "480i") then
  -- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
  Setup_VPP(lvpp0, HSIZE, VSIZE, 0, 0, 2, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=263});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=22});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3003});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3003});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=8});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=9}); 
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
elseif (FORMAT == "576i") then
  Setup_VPP(lvpp0, HSIZE, VSIZE, 1, 0, 2, 4)
  lvpp0:setParam({param=Q_LVPP_CMP_FIELD_OFFSET, value=313});
  lvpp0:setParam({param=Q_LVPP_CMP_START_VLINE, value=24});
  lvpp0:setParam({param=Q_LVPP_CMP_START_HPIXEL, value=0});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_FRAME, value=3600});
  lvpp0:setParam({param=Q_LVPP_CMP_STC_PER_OUTPUT_FRAME, value=3600});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_X, value=16});
  lvpp0:setParam({param=Q_LVPP_CMP_PIXEL_AR_Y, value=15});
  lvpp0:setParam({param=Q_LVPP_CMP_DEST_PROG, value=0});
end

lvpp0:activateCfg();

-- create and configure audio input
sain0 = sysctl0:sainCreate({name="sain0",port=0});

sain0:setParam({param=Q_SAIN_CMP_AUD_CHANNEL_NUM, value=Q_SAIN_CFP_CHANNEL_STEREO});
sain0:setParam({param=Q_SAIN_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
sain0:setParam({param=Q_SAIN_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
sain0:setParam({param=Q_SAIN_CMP_AUD_SAMPLE_SIZE, value=16});
sain0:setParam({param=Q_SAIN_CMP_AUD_FRAME_SIZE, value=AUDIO_FRAME_SIZE});
-- sain0:setParam({param=Q_SAIN_CMP_AUD_CODEC, value=SAIN_AUDIO_CODEC});
sain0:activateCfg();

avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
if (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "480i") then
  avoc0:setOutput480i();
elseif (FORMAT == "576i") then
  avoc0:setOutput576i();
end
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});
avoc0:activateCfg();

-- Activate preview
avsn0:bindVideoInput({src=lvpp0});

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});
avoc0:activateVideoChannelCfg();

-- Echo audio to output
avsn0:bindAudioInput({src=sain0});
ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
avoc0:enableAudioChannel({ch=ach0, enable=1});
avoc0:activateAudioChannelCfg();

-- start output
avoc0:start();
avsn0:start();

-- start audio/video input
lvpp0:start();
sain0:start();

