--[[

This script first decodes a 720p AVC stream, then re-encode it back to 
AVC format.  Both the decoded and transcoded stream will be displayed 
on the screen (decoded stream is shown in the lower right subpicture).

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

INPUTFILENAME     = INPUTFILENAME or "/directmnt/qa/streams/merlin/cestest720p.qbox";
VOUT              = VOUT or "hdmi";
FORMAT            = FORMAT or "720p";
MAX_WIDTH         = MAX_WIDTH or 1280;
MAX_HEIGHT        = MAX_HEIGHT or 720;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

audioStreamId = 1;
videoStreamId = 2;

-- Set up the output port
board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});

sysctl0:setVPortMode({port=1, live=0});

rs0 = sysctl0:createReadStreamer({name="rs0"})
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=Q_AVDEC_CFP_AUDIO_CODEC_QMA, vtype=Q_AVDEC_CFP_VIDEO_CODEC_AVC, audioBufferSize=100000, videoBufferSize=1000000, maxWidth=1280, maxHeight=MAX_HEIGHT});
nvpp0 = sysctl0:nvppCreate({name="nvpp0", maxWidth=MAX_WIDTH, maxHeight=MAX_HEIGHT});
ws = sysctl0:createWriteStreamer({name="ws0"});
avcenc0 = sysctl0:avcencCreate({name="avcenc0", ws=ws, maxwidth=MAX_WIDTH, maxheight=MAX_HEIGHT, numreferenceframes=10, outputbuffersize=3000000});
avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=MAX_WIDTH, maxHeight=MAX_HEIGHT});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avsn1 = sysctl0:avsnCreate({name="avsn1"});

-- AVDEC settings --

avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audioStreamId});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=videoStreamId});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=1500});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_THRESHOLD, value=50000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_OVERFLOW_MARGIN, value=20000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=400000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=50000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});
avdec0:activateCfg();

-- NVPP settings --

-- Setup_VPP(vpp_instance, in_width, in_height, is_interlaced, is_420, pme_hscale, pme_vscale)
Setup_VPP(nvpp0, MAX_WIDTH, MAX_HEIGHT, 0, 1, 4, 4)
nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_FRAME, value=1500});
nvpp0:setParam({param=Q_NVPP_CMP_STC_PER_OUTPUT_FRAME, value=1500});
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_X, value=1});
nvpp0:setParam({param=Q_NVPP_CMP_PIXEL_AR_Y, value=1});
nvpp0:activateCfg();

-- AVC Encoder settings --

-- include file for AVC setup function
--avcenc_720p60(avcenc, profile, level, bitrate_qp, gop_structure, picture_coding)
avcenc_720p60(avcenc0, "high", 40, 4000000, Q_AVCENC_CFP_GOP_STRUCTURE_IBBRBP, 0);

--ws:open({type="h4v", file="test.h4v", sid = {videoStreamid}});
avcenc0:setStreamID({streamid=videoStreamId});

nvpp0:bindInput({src=avdec0});
avcenc0:bindVideoInput({src=nvpp0});
avsn0:bindVideoInput({src=avdec0});
avsn1:bindVideoInput({src=avcenc0});

-- Activate reconstructed preview

avcenc0:setVideoEncParam({param=Q_AVCENC_CMP_ENCODER_PREVIEW , value=1});
avcenc0:activateVideoEncCfg();

-- Output settings --

avoc0:setOutput720p60();
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=MAX_WIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=MAX_HEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=0});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=MAX_WIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=MAX_HEIGHT});
avoc0:activateCfg();

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn1});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=MAX_WIDTH});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=MAX_HEIGHT});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=MAX_WIDTH});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=MAX_HEIGHT});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

vch1 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch1, src=avsn0});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=MAX_WIDTH});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=MAX_HEIGHT});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=384});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=216});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=876});
avoc0:setVideoChannelParam({ch=vch1, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=484});

avoc0:activateVideoChannelCfg();

-- Start! --

avoc0:start();
rs0:start();
ws:start();
avcenc0:record();
nvpp0:start();
avsn0:start();
avsn1:start();

rs0:bindDecoder({dec=avdec0});
bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={videoStreamId, audioStreamId}});
avdec0:start();
rs0:play({bsr=bsr0});


-- Stop sequence --
    
-- avdec0:waitForVideoEOS()
-- rs0:close({bsr=bsr0});
-- ws:close({type="h4v", file="test.h4v"});
-- avdec0:stop();
-- avsn0:stop();
-- avsn1:stop();

