--[[

This script demostrates multi-channel decoding with trick plays.  Function
tricktest() can be called at any time to show the different trick modes
implemented.

NOTE: Since all 4 streams are decoded from a single file, trick modes will
be applied to all of them at the same time.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of decoded channels
DECCH = DECCH or 4;

-- Number of rendered images and their scaled size
OUTCH = OUTCH or 4;
OUTW = OUTW or 720;
OUTH = OUTH or 480;

-- Width and height of the output grid
OUTGRIDW = OUTGRIDW or 2;
OUTGRIDH = OUTGRIDH or 2;

-- Input files
-- 4 streams stored in one single file
INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/4movies.qbox";
-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;

VIDEO_CODEC = VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_AVC;
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QMA;
---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

SHOWUPDATE = 0;

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=5*1024*1024});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format='1080i', mode='hdmi', sync=vout_sync});

rs0 = sysctl0:createReadStreamer({name="rs0"});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=SHOWUPDATE});

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

-- Create Decoders
avdec = {};
avsndec = {};
audId = {};
vidId = {};
DECCH = DECCH - 1;

for i = 0,DECCH do
  vidId[i] = i+1;
  audId[i] = i+8;
  
  avdec[i] = sysctl0:avdecCreate({name="avdec", atype=AUDIO_CODEC, vtype=VIDEO_CODEC, audioBufferSize=100000, videoBufferSize=500000, maxWidth=720, maxHeight=480, videoAUs=256,ioMode=MEMORY_XFER_MODE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audId[i]});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=vidId[i]});
--  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=Q_AVDEC_CFP_TRUE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});
  avdec[i]:activateCfg();
  
  rs0:bindDecoder({dec=avdec[i]});
end

vch = {};
x = 0;
y = 0;
OUTCH = OUTCH - 1;

for i = 0,OUTCH do
  avsndec[i] = sysctl0:avsnCreate({name="avsndec"});

  vch[i] = avoc0:addVideoChannel();

  avsndec[i]:bindVideoInput({src=avdec[i]});
  
  avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
  
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end

avoc0:activateVideoChannelCfg();

bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId=vidId});
rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
-- START
avoc0:start();

for i = 0,OUTCH do
  avsndec[i]:start();
end

for i = 0,DECCH do
  avdec[i]:start();
end

rs0:play({bsr=bsr0});

function q()
  for i = 0,DECCH do
    avdec[i]:query();
  end
end

-- Trick mode functions

-- Pause playback
function pause()
  for i = 0, OUTCH do
    avsndec[i]:pause();
  end
end

-- Resume playback from paused state
function resume()
  for i = 0, OUTCH do
    avsndec[i]:resume();
  end
end

-- Slow play: n = "n" times slower
function slow(n)
  for i = 0, OUTCH do
    avsndec[i]:slow({speed=n});
  end
end

-- Forward step: Only applicable in paused state
function step()
  for i = 0, OUTCH do
    avsndec[i]:step();
  end
end

-- Print current file position
function fpos()
    pos = rs0:getFilePos({bsr=bsr0}); print("Current file position = "..pos);
end

-- I-Frame scan Mode: Only display "n"th I-Frames for "dispTime"
-- AVSN stop/start or pause/resume??
function iplay(n, dispTime)
  if ((n == nil) or (dispTime == nil)) then
    print("Must specify n and/or display time");
    return;
  end

  pos = rs0:getFilePos({bsr=bsr0});
  rs0:close({bsr=bsr0});
  
  for i = 0, DECCH do
    avdec[i]:stop();
    avdec[i]:waitForVideoEOS()
  end
  for i = 0, OUTCH do
    avsndec[i]:stop();
  end

  bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId=vidId, startOfst=pos});
  rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=n});
  rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
  
  for i = 0, OUTCH do
    avsndec[i]:start();
    avsndec[i]:iFrameScan({frameTime=dispTime});
  end
  for i = 0, DECCH do
    avdec[i]:start();
  end

  rs0:play({bsr=bsr0});
end

-- Deactivate I-Frame scan mode: Return to normal playback
function istop()
  pos = rs0:getFilePos({bsr=bsr0});
  rs0:close({bsr=bsr0});
  
  for i = 0, DECCH do
    avdec[i]:stop();
    avdec[i]:waitForVideoEOS()
  end
  for i = 0, OUTCH do
    avsndec[i]:stop();
  end
  
  bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId=vidId, startOfst=pos});
  rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=0});
  rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
  
  for i = 0, OUTCH do
    avsndec[i]:start();
  end
  for i = 0, DECCH do
    avdec[i]:start();
  end
  
  rs0:play({bsr=bsr0});
end


-- TRICK MODE TEST --

function tricktest()
  print("=== PAUSE ===");
  pause();
  os.execute("sleep 2");

  for i = 0, 10 do
    print("=== STEP ===");
    step();
    os.execute("sleep 1");
  end

  print("=== RESUME ===");
  resume();
  os.execute("sleep 5");

  print("=== SLOW 2x ===");
  slow(2);
  os.execute("sleep 10");

  print("=== SLOW 8x ===");
  slow(8);
  os.execute("sleep 10");

  print("=== RESUME ===");
  resume();
  os.execute("sleep 5");

  print("=== I-FRAME SCAN (1,4) ===")
  iplay(1, 4);
  os.execute("sleep 10");

  print("=== RESUME ===")
  istop();
end



