--[[

This script demostrates multi-channel decoding with trick plays.  Function
tricktest() can be called at any time to show the different trick modes
implemented.

NOTE: Since all 4 streams are decoded from different files, trick modes can
be applied to each individual stream.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Number of decoded channels
DECCH = DECCH or 4;

-- Number of rendered images and their scaled size
OUTCH = OUTCH or 4;
OUTW = OUTW or 720;
OUTH = OUTH or 480;

-- Width and height of the output grid
OUTGRIDW = OUTGRIDW or 2;
OUTGRIDH = OUTGRIDH or 2;

-- Input files
-- 4 streams stored in four files
INPUTFILENAME    = INPUTFILENAME or {};
INPUTFILENAME[0] = INPUTFILENAME[0] or "/directmnt/qa/streams/merlin/movie1.480i.qbox";
INPUTFILENAME[1] = INPUTFILENAME[1] or "/directmnt/qa/streams/merlin/movie2.480i.qbox";
INPUTFILENAME[2] = INPUTFILENAME[2] or "/directmnt/qa/streams/merlin/movie3.480i.qbox";
INPUTFILENAME[3] = INPUTFILENAME[3] or "/directmnt/qa/streams/merlin/movie4.480i.qbox";

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;

VIDEO_CODEC = VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_AVC;
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QMA;

---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

COMPWIDTH=OUTGRIDW*OUTW;
COMPHEIGHT=OUTGRIDH*OUTH;
COMPYOFFSET= (1080-(OUTGRIDH*OUTH))/2;
COMPXOFFSET= (1920-(OUTGRIDW*OUTW))/2;

SHOWUPDATE = 0;

NUMINPUTFILES = 4;

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

sysctl0:setVPortMode({port=0, live=1, interlaced=1, bottomfieldfirst=0, videocontrol=0x00101000, interleave=1});
sysctl0:setVPortMode({port=1, live=0});
sysctl0:configure({param=Q_SYS_CFG_DECODER_INPUT_PARTITION_SIZE, value=5*1024*1024});

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

board.setVideoOutput({port=0, format='1080i', mode='hdmi', sync=vout_sync});

rs0 = sysctl0:createReadStreamer({name="rs0"});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=SHOWUPDATE});

avoc0 = sysctl0:avocCreate({name="avoc0", maxWidth=COMPWIDTH, maxHeight=COMPHEIGHT});
avoc0:setOutput1080i60({sync=vout_sync});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_X, value=COMPXOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_OFFSET_Y, value=COMPYOFFSET});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_ENABLE, value=1});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_WIDTH, value=COMPWIDTH});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_COMP_HEIGHT, value=COMPHEIGHT});
avoc0:setParam({param=Q_AVOC_CMP_VPU_BG_COLOR, value=0x10808000});
avoc0:activateCfg();

-- Create Decoders
avdec = {};
avsndec = {};
audId = {};
vidId = {};
DECCH = DECCH - 1;

for i = 0,DECCH do
  vidId[i] = i+1;
  audId[i] = i+8;
  
  avdec[i] = sysctl0:avdecCreate({name="avdec", atype=AUDIO_CODEC, vtype=VIDEO_CODEC, audioBufferSize=100000, videoBufferSize=500000, maxWidth=720, maxHeight=480, videoAUs=256,ioMode=MEMORY_XFER_MODE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=audId[i]});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=vidId[i]});
--  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_DISABLE_OUTPUT, value=Q_AVDEC_CFP_TRUE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=24000});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
  avdec[i]:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_OVERRIDE, value=Q_AVDEC_CFP_TRUE});
  avdec[i]:setParam({param=Q_AVDEC_CMP_VIDEO_TICKS_PER_FRAME_DEFAULT, value=3003});
  avdec[i]:activateCfg();
  
  rs0:bindDecoder({dec=avdec[i]});
end

vch = {};
x = 0;
y = 0;
OUTCH = OUTCH - 1;

for i = 0,OUTCH do
  avsndec[i] = sysctl0:avsnCreate({name="avsndec"});

  vch[i] = avoc0:addVideoChannel();

  avsndec[i]:bindVideoInput({src=avdec[i]});
  
  avoc0:setVideoChannelSource({ch=vch[i], src=avsndec[i]});
  
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=OUTW});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=OUTH});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=x});
  avoc0:setVideoChannelParam({ch=vch[i], param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=y});

  x = x + OUTW;
  if x == COMPWIDTH then
    x = 0;
    y = y + OUTH;
  end
end

avoc0:activateVideoChannelCfg();

bsr = {};
NUMINPUTFILES = NUMINPUTFILES - 1;

for i = 0, NUMINPUTFILES do
  bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={2}, codecId={vidId[i]}});
  rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
end

-- START
avoc0:start();

for i = 0,OUTCH do
  avsndec[i]:start();
end

for i = 0,DECCH do
  avdec[i]:start();
end

for i = 0, NUMINPUTFILES do
  rs0:play({bsr=bsr[i]});
end

function q()
  for i = 0,DECCH do
    avdec[i]:query();
  end
end

function avsnq()
  for i = 0,OUTCH do
    avsndec[i]:query();
  end
end

-- Trick mode functions

-- Pause playback
function pause(i)
  avsndec[i]:pause();
end

-- Resume playback from paused state
function resume(i)
  avsndec[i]:resume();
end

-- Slow play: n = "n" times slower
function slow(i, n)
  avsndec[i]:slow({speed=n});
end

-- Forward step: Only applicable in paused state
function step(i)
  avsndec[i]:step();
end

-- Print current file position
function fpos(i)
    pos = rs0:getFilePos({bsr=bsr[i]}); 
    print("Current file position = "..pos);
end

-- I-Frame scan Mode: Only display "n"th I-Frames for "dispTime"
-- AVSN stop/start or pause/resume??
function iplay(i, n, dispTime)
  if ((n == nil) or (dispTime == nil)) then
    print("Must specify n and/or display time");
    return;
  end

  pos = rs0:getFilePos({bsr=bsr[i]});
  
  rs0:close({bsr=bsr[i]});
  avdec[i]:stop();
  avdec[i]:waitForVideoEOS()
  avsndec[i]:stop();

  bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={2}, codecId={vidId[i]}, startOfst=pos});
  rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=n});
  rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
  
  avsndec[i]:start();
  avdec[i]:start();
  rs0:play({bsr=bsr[i]});
  
  avsndec[i]:iFrameScan({frameTime=dispTime});
end

-- Deactivate I-Frame scan mode: Return to normal playback
function istop(i)
  pos = rs0:getFilePos({bsr=bsr[i]});
  
  rs0:close({bsr=bsr[i]});
  avdec[i]:stop();
  avdec[i]:waitForVideoEOS()
  avsndec[i]:stop();
  
  bsr[i] = rs0:open({type="qbox", file=INPUTFILENAME[i], srcId={2}, codecId={vidId[i]}, startOfst=pos});
  rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=0});
  rs0:setReaderParam({bsr=bsr[i], param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
  
  avsndec[i]:start();
  avdec[i]:start();
  rs0:play({bsr=bsr[i]});
end


-- TRICK MODE TEST --

function tricktest(i)
  print("=== ID "..i.." PAUSE ===");
  pause(i);
  os.execute("sleep 2");

  for j = 0, 10 do
    print("=== ID "..i.." STEP ===");
    step(i);
    os.execute("sleep 1");
  end

  print("=== ID "..i.." RESUME ===");
  resume(i);
  os.execute("sleep 5");

  print("=== ID "..i.." SLOW 2x ===");
  slow(i, 2);
  os.execute("sleep 10");

  print("=== ID "..i.." SLOW 8x ===");
  slow(i, 8);
  os.execute("sleep 10");

  print("=== ID "..i.." RESUME ===");
  resume(i);
  os.execute("sleep 5");

  print("=== ID "..i.." I-FRAME SCAN (1,4) ===")
  iplay(i, 1, 4);
  os.execute("sleep 10");

  print("=== ID "..i.." RESUME ===")
  istop(i);
end



