--[[

This script is used to decode the input file specified with trick play
functions available.

NOTE: To start decoding a file from an offset, the encoded AVC stream
must have closed GOP and SPS at each I-Frame.

--]]

----------------------------------------------------------------------------
--                      Configuration parameters                          --
----------------------------------------------------------------------------

-- Video output mode
-- "hdmi" HDMI output (1080i and 720p only)
-- "svideo" SVIDEO input/output (note composite is driven at same time on output)
VOUT            = VOUT or "hdmi";

-- Video format
-- "1080i"
-- "720p"
-- "480i"
-- "576i"
FORMAT          = FORMAT or "720p";

-- Audio sample rate
SAMPLERATE  = SAMPLERATE or 32000;

VIDEO_CODEC = VIDEO_CODEC or Q_AVDEC_CFP_VIDEO_CODEC_AVC;
AUDIO_CODEC = AUDIO_CODEC or Q_AVDEC_CFP_AUDIO_CODEC_QAC;
AUDIO_STEREO = AUDIO_STEREO or 1;

-- Use different streams for different formats

INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/spider.720p.qbox";
-- INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/spider.1080i.qbox";

--INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/av480i.qbx";
--INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/av720p.qbx";
--INPUTFILENAME = INPUTFILENAME or "/directmnt/qa/streams/merlin/av1080i.qbx";

-- Allow video only decode
VIDEO_ONLY = VIDEO_ONLY or 0

-- Direct memory transfer mode (aka AVC instant display mode)
MEMORY_XFER_MODE = MEMORY_XFER_MODE or 1;
---------------------------------------------------------------------
-- Don't change below this line unless you know what you are doing --
---------------------------------------------------------------------

-- Allow the display to be disabled for power testing
DISABLE_DISPLAY = DISABLE_DISPLAY or 0

if (MEMORY_XFER_MODE == 1) then
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC_INSTANT_DISPLAY;
else
    VIDEO_CODEC = Q_AVDEC_CFP_VIDEO_CODEC_AVC;
end

if (FORMAT == "720p") then
  HSIZE = 1280;
  VSIZE = 720;
  VIDEO_FRAME_TIME = 1500;
elseif (FORMAT == "1080i") then
  HSIZE = 1920;
  VSIZE = 1080;
  VIDEO_FRAME_TIME = 3003;
elseif (FORMAT == "480i") then
  HSIZE = 720;
  VSIZE = 480;
  VIDEO_FRAME_TIME = 3003;
elseif (FORMAT == "576i") then
  HSIZE = 720;  
  VSIZE = 576;
  VIDEO_FRAME_TIME = 3003;
else
  print("Invalid format\n");
end

AUDIO_BUFFER_SIZE = 1000000;
VIDEO_BUFFER_SIZE = 2000000;
VIDEO_STREAM_ID = 2;
AUDIO_STREAM_ID = 1;

SHOW_UPDATE = 0;

vout_sync = 'external'
if board.getType() == "evp" then vout_sync = nil end

-- Set up the output port
--if (DISABLE_DISPLAY == 0) then
  if (FORMAT == "1080i") then
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT, sync=vout_sync});
  else
    board.setVideoOutput({port=0, format=FORMAT, mode=VOUT});
  end
--end

if (VIDEO_ONLY == 0) then
  if (SAMPLERATE == 8000) then
    board.setAudioSampleFreq({port=0, freq=16000});
    sysctl0:configure({param=Q_SYS_CFG_PLL3_CLOCK_HZ, value=2048000});
    sysctl0:configure({param=Q_SYS_CFG_AUD0_CLOCK_HZ, value=2048000});
  else
    board.setAudioSampleFreq({port=0, freq=SAMPLERATE});
  end
end

-- create objects
avoc0 = sysctl0:avocCreate({name="avoc0"});
avsn0 = sysctl0:avsnCreate({name="avsn0"});
avdec0 = sysctl0:avdecCreate({name="avdec0", atype=AUDIO_CODEC, vtype=VIDEO_CODEC, 
                              audioBufferSize=AUDIO_BUFFER_SIZE, videoBufferSize=VIDEO_BUFFER_SIZE, maxWidth=HSIZE, maxHeight=VSIZE,ioMode=MEMORY_XFER_MODE});
rs0 = sysctl0:createReadStreamer({name="rs0"})


avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_STREAM_ID, value=AUDIO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_STREAM_ID, value=VIDEO_STREAM_ID});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_RATE, value=SAMPLERATE});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_SAMPLE_SIZE, value=16});
if (AUDIO_STEREO == 1) then
  avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_STEREO});
else
  avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_CHANNEL_NUM, value=Q_AVDEC_CFP_AUDIO_CHANNEL_MONO});
end

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_UNDERFLOW_MARGIN, value=10000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_THRESHOLD, value=500000});
avdec0:setParam({param=Q_AVDEC_CMP_AUDIO_UNDERFLOW_MARGIN, value=5000});

avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_INITIAL_BUFFER_LEVEL, value=0});

if (FORMAT == "576i") then
  avdec0:setParam({param=Q_AVDEC_CMP_VIDEO_FIELD_ORDER, value=Q_AVDEC_CFP_VIDEO_FIELD_ORDER_FORCE_BOTTOM_FIRST});
end

avdec0:activateCfg();

-- set video hw params
if (FORMAT == "480i") then
  avoc0:setOutput480i();
elseif (FORMAT == "576i") then
  avoc0:setOutput576i();
elseif (FORMAT == "720p") then
  avoc0:setOutput720p60();
elseif (FORMAT == "1080i") then
  avoc0:setOutput1080i60({sync=vout_sync});
end

avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_WIDTH, value=HSIZE});
avoc0:setParam({param=Q_AVOC_CMP_VID_0_DISPLAY_HEIGHT, value=VSIZE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SERIAL_MODE, value=Q_AVOC_CFP_AUD_SERIAL_MODE_I2S});
avoc0:setParam({param=Q_AVOC_CMP_AUD_MASTER_CLOCK, value=Q_AVOC_CFP_AUD_MASTER_CLOCK_256FS});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_RATE, value=SAMPLERATE});
avoc0:setParam({param=Q_AVOC_CMP_AUD_SAMPLE_SIZE, value=16});
if (DISABLE_DISPLAY ~= 0) then
  avoc0:setParam({param=Q_AVOC_CMP_VID_0_ENABLE, value=0});
end
avoc0:activateCfg();

avsn0:bindVideoInput({src=avdec0});

if (VIDEO_ONLY == 0) then
  avsn0:bindAudioInput({src=avdec0});

  if (AUDIO_STEREO == 1) then
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_STEREO});
  else
    ach0 = avoc0:addAudioChannel({channel=Q_AVOC_CFP_AUD_MONO_L});
  end
  avoc0:setAudioChannelSource({ch=ach0, src=avsn0});
  avoc0:enableAudioChannel({ch=ach0, enable=1});
  avoc0:activateAudioChannelCfg();
end

vch0 = avoc0:addVideoChannel();
avoc0:setVideoChannelSource({ch=vch0, src=avsn0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_SRC_OFFSET_Y, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_WIDTH, value=HSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_HEIGHT, value=VSIZE});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_X, value=0});
avoc0:setVideoChannelParam({ch=vch0, param=Q_AVOC_CMP_VIDEO_CHANNEL_DST_OFFSET_Y, value=0});

avoc0:activateVideoChannelCfg();

rs0:bindDecoder({dec=avdec0});
bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={AUDIO_STREAM_ID, VIDEO_STREAM_ID}});
rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
rs0:setParam({param=Q_RS_CFG_SHOW_UPDATE, value=SHOW_UPDATE});

avoc0:start();
avsn0:start();

avdec0:start();
rs0:play({bsr=bsr0});

-- TRICK PLAY FUNCTIONS --

-- Pause playback
function pause()
    avsn0:pause();
end

-- Resume playback from paused state
function resume()
    avsn0:resume();
end

-- Slow play: n = "n" times slower
function slow(n)
    avsn0:slow({speed=n});
end

-- Forward step: Only applicable in paused state
function step()
    avsn0:step();
end

-- Print current file position
function fpos()
    pos = rs0:getFilePos({bsr=bsr0}); print("Current file position = "..pos);
end

-- I-Frame scan Mode: Only display "n"th I-Frames for "dispTime"
-- AVSN stop/start or pause/resume??
function iplay(n, dispTime)
    if ((n == nil) or (dispTime == nil)) then
        print("Must specify n and/or display time");
        return;
    end

    pos = rs0:getFilePos({bsr=bsr0});
    
    rs0:close({bsr=bsr0});
    avdec0:stop();
    avdec0:waitForVideoEOS()
    avsn0:stop();
    
    bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={AUDIO_STREAM_ID, VIDEO_STREAM_ID}, startOfst=pos});
    rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=n});
    rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
    
    avsn0:start();
    avdec0:start();
    rs0:play({bsr=bsr0});
    
    avsn0:iFrameScan({frameTime=dispTime});
end

-- Deactivate I-Frame scan mode: Return to normal playback
function istop()
    pos = rs0:getFilePos({bsr=bsr0});
    
    rs0:close({bsr=bsr0});
    avdec0:stop();
    avdec0:waitForVideoEOS()
    avsn0:stop();
    
    bsr0 = rs0:open({type="qbox", file=INPUTFILENAME, srcId={AUDIO_STREAM_ID, VIDEO_STREAM_ID}, startOfst=pos});
    rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_IFRAME_SCAN_SPEED, value=0});
    rs0:setReaderParam({bsr=bsr0, param=Q_RS_CFG_BSR_DMA_XFER_MODE, value=MEMORY_XFER_MODE});
    
    avsn0:start();
    avdec0:start();
    rs0:play({bsr=bsr0});
end

-- TRICK MODE TEST --

function tricktest()
    print("=== PAUSE ===");
    pause();
    os.execute("sleep 2");
    
    for i = 0, 10 do
        print("=== STEP ===");
        step();
        os.execute("sleep 1");
    end
    
    print("=== RESUME ===");
    resume();
    os.execute("sleep 5");
    
    print("=== SLOW 2x ===");
    slow(2);
    os.execute("sleep 10");
    
    print("=== SLOW 8x ===");
    slow(8);
    os.execute("sleep 10");
    
    print("=== RESUME ===");
    resume();
    os.execute("sleep 5");
    
    print("=== I-FRAME SCAN (1,4) ===")
    iplay(1, 4);
    os.execute("sleep 10");
    
    print("=== RESUME ===")
    istop();
end

